------------------------------------------------------------------------------ 
-- | 
-- Maintainer	: Ralf Laemmel, Joost Visser
-- Stability	: experimental
-- Portability	: portable
--
-- This module is part of 'Sdf2Haskell', a tool for generating a set of
-- Haskell data types from an SDF grammar. This module provides functionality
-- for calling the external parser SGLR from within Haskell and marshalling
-- the resulting abstract syntax tree to a strongly-typed Haskell term.

------------------------------------------------------------------------------

module SGLR where

import System (ExitCode(..),getProgName)
import System.Cmd (system)
import Data.Unique (newUnique, hashUnique)
import System.IO (readFile,hPutStrLn,stderr)
import ATermLib (fromATerm, readATerm, ATermConvertible, 
                 dehyphen, afunCap, ATerm(..)
                )

------------------------------------------------------------------------------
-- * Interaction with the system.

-- | Call the external sglr parser.
sglr :: ATermConvertible a
     => FilePath	-- ^ table
     -> FilePath	-- ^ term
     -> String          -- ^ top sort
     -> IO a
sglr tableName termName sortName
  = do unique <- newUnique
       let newName = termName++(show . hashUnique $ unique)
       let asfixName = newName++".asfix"
       let afName = newName++".af"
       let sglrCmd = "sglr -p "++tableName++
                         " -i "++termName++
			 " -o "++asfixName++
			 " -s "++sortName
       let implodeCmd = "implodePT -t -ALclpqX"++ 
                                 " -i "++asfixName++
				 " -o "++afName
       progName <- getProgName
       errLn $ "["++progName++"] "++sglrCmd			 
       exitOK $ system sglrCmd
       errLn $ "["++progName++"] "++implodeCmd
       exitOK $ system implodeCmd
       af <- readFile afName
       --return . fromATerm . afunCap . implodeLit . readATerm . dehyphen $ af
       let t1 = readATerm . dehyphen $ af
       --putStrLn $ "After dehyphen: \n"++show t1
       let t2 = rmSTART . implodeBracket . implodeIter . implodeOpt $ t1
       --putStrLn $ "After implodeOpt: \n"++show t2
       let t3 = afunCap t2
       return . fromATerm $ t3
       
-- | Helper function for reporting errors and progress to stderr
errLn :: String -> IO ()
errLn str = hPutStrLn stderr str

-- | Ensure that the given computation exits with an error code
--   that indicates successful execution.       
exitOK :: IO ExitCode -> IO ()
exitOK io
  = do exitCode <- io
       case exitCode of
         (ExitSuccess)   -> return ()
	 (ExitFailure code) -> fail $ "Exit code: " ++ show code

	 
------------------------------------------------------------------------------
-- * Functions to compensate for bugs in implodePT

-- | Implode literals.
implodeLit :: ATerm -> ATerm
implodeLit (AAppl "lit" [AAppl str []])	= AAppl "" []
implodeLit (AAppl f ts) 		= AAppl f (map implodeLit ts)
implodeLit (AList ts)    		= AList (map implodeLit ts)
implodeLit t				= t

-- | Implode optionals.
implodeOpt :: ATerm -> ATerm
implodeOpt (AAppl "appl" [ AAppl "prod" [ AList [],
                                          AAppl "cf" [ AAppl "opt" [ symbol ]],
			                  attrs
					],
                           AList []
			 ])
  = AAppl "None" [ ]
implodeOpt (AAppl "appl" [ AAppl "prod" [ AList [ symbol ],
                                          AAppl "cf" [ AAppl "opt" [ symbol' ]],
					  attrs
					],
                           AList [ term ]
			 ])
  = AAppl "Some" [ implodeOpt term ]
implodeOpt (AAppl f ts) 		= AAppl f (map implodeOpt ts)
implodeOpt (AList ts)    		= AList (map implodeOpt ts)
implodeOpt t				= t

-- | Implode iterations.
implodeIter :: ATerm -> ATerm
implodeIter (AAppl "appl" 
              [ AAppl "prod" 
	          [ AList [],
                    AAppl "cf" [AAppl "iter" [ sort ]],
		    attrs
		  ],
		AList []
	      ]
	    )
  = AList []	    
implodeIter (AAppl f ts) 		= AAppl f (map implodeIter ts)
implodeIter (AList ts)    		= AList (map implodeIter ts)
implodeIter t				= t

-- | Implode brackets.
implodeBracket :: ATerm -> ATerm
implodeBracket (AAppl "appl"
                 [ AAppl "prod"
		     [ AList [ symbol ],
		       symbol',
		       AAppl "attrs" [AList [AAppl "bracket" []]]
		     ],
		   AList [ term ]
		 ]
	)
  = term
implodeBracket (AAppl f ts) 		= AAppl f (map implodeBracket ts)
implodeBracket (AList ts)    		= AList (map implodeBracket ts)
implodeBracket t			= t

-- | Remove start symbol.
rmSTART :: ATerm -> ATerm
rmSTART (AAppl "appl" 
          [ AAppl "prod" 
	     [ AList [AAppl "cf" [AAppl "sort" [AAppl sort []]]],
	       AAppl "sort" [AAppl "\"<START>\"" []],
	       attrs
	     ],
	    AList [ term ] 
	  ]
	)
  = term	
rmSTART t = t

------------------------------------------------------------------------------
	 
