-----------------------------------------------------------------------------
-- |
-- Maintainer  :  joost.visser@di.uminho.pt
-- Stability   :  experimental
-- Portability :  portable
--
-- A framework for the Model-View-Controller design pattern. In particular,
-- this module allows easy constructions of on-line applications that offer
-- a simple menu structure for manipulation of an underlying model.
--
-----------------------------------------------------------------------------

module Control.ModelViewController where

import CGI
import Control.WashExtras

-----------------------------------------------------------------------------
-- * Representation

-- | The type of controllers parameterized by the type of models.
data Controller model = Controller {
  initModel :: FileReference -> IO model,
  -- ^ Action to initialize the model.
  modelOperations :: [(String, ModelOperation model)],
  -- ^ List of operations on the model.
  modelViews :: [(String, ModelView model)]
  -- ^ List of views on the model.
}

-- | The type of operations on the model.
data ModelOperation model
  = forall p l v h . (ConnectCGI p l v h, InputHandle h)
    => MO l (model -> p) (model -> v -> model)

-- | The type of views on the model.
data ModelView' model
  = forall p l v h . (ConnectCGI p l v h, InputHandle h)
    => MV' l (model -> p) (model -> v -> WithHTML () CGI ())

data ModelView model
  = MV (model -> WithHTML () CGI ())

-----------------------------------------------------------------------------
-- * Create a full online application.

-- | Create application for a model and its controller.
mvc :: (Read model, Show model) => Controller model -> CGI ()
mvc c = do
  mkInputPage "Load File" "Load" "Select file to load" () initModelCGI
  where
    initModelCGI fileRef = do
      model <- io $ initModel c fileRef
      let firstView = Prelude.head $ modelViews c
      menuQuery c model firstView

  
-----------------------------------------------------------------------------
-- * Create parts.
  
-- | Create the main application window with menu and selected view.
menuQuery :: (Read model, Show model) 
          => Controller model   -- ^ Controller
          -> model              -- ^ Current model
          -> (String,ModelView model)  -- ^ Selected view
          -> CGI ()             -- ^ Main application window.
menuQuery c model view = standardQuery' ("Menu & "++(fst view)) $ do
  simpleMenuFor (mkMenuButtons c model (modelViews c) (modelOperations c))
  mkView view model
      
-- | Create menu buttons for continuation to other views.
mkMenuButtons :: (Read model, Show model) 
              => Controller model
              -> model
              -> [(String,ModelView model)]
              -> [(String,ModelOperation model)]
              -> [(String, CGI ())]
mkMenuButtons c model labeledViews labeledOperations
  = ("Init",mvc c):
    Prelude.map mkViewButton labeledViews ++
    Prelude.map mkOperationButton labeledOperations
    where
      mkViewButton lv 
        = ( fst lv, 
            once $
            menuQuery c model lv
          )
      mkOperationButton lo@(operationLabel,(MO l mkx f))
        = ( operationLabel,
            once $
            mkInputPage 
                ("Operation: "++operationLabel)
                "Go"
                l
                (mkx model) 
                (\x -> do let model' = f model x
                          let firstView = Prelude.head $ modelViews c
                          menuQuery c model' firstView
                )
          )

-- | Create a view on the model.
mkView :: (String,ModelView model)
       -> model
       -> WithHTML () CGI ()
mkView (viewLabel,(MV f)) model
  = f model
  
-----------------------------------------------------------------------------
-- * Creating specific views.
  
-- | View a graph generated from the current model.
graphView :: String               -- ^ Label
          -> (model -> String)    -- ^ create dot string from model
          -> ModelView model
graphView label f = MV (\model -> do
   let dotString = f model
   imageFileName <- ioHTML $ callDot dotString "gif" label
   let image = presentImage imageFileName
   table $ do
     tr $ th $ text label
     tr $ td image           
 )

-----------------------------------------------------------------------------

