-----------------------------------------------------------------------------
-- |
-- Maintainer  :  joost.visser@di.uminho.pt
-- Stability   :  experimental
-- Portability :  portable
--
-- Some higher-level functionality built on top of WASH.
--
-- WASH, by Peter Thiemann, provides support for creating
-- dynamic web pages, in particular CGI programs.
--
-----------------------------------------------------------------------------

module Control.WashExtras where

import CGI
import qualified HTMLWrapper as H hiding (map)
import Fields (Reason(..))
import System (system)


-----------------------------------------------------------------------------
-- * Type-level derivation of handles from values

-- | The possible values and the needed input value together determine
--   the appropriate input handle and the the appropriate labeling information.
class ConnectCGI possibleValues labeling value handle 
    | possibleValues value -> handle labeling where
  connectCGI
    :: ( handle VALID -> value
         -- ^ the function to obtain values from valid handles.
       , labeling -> possibleValues -> WithHTML x CGI (handle INVALID)
         -- ^ the function to obtain an appropriate input handle.
       )

-- | Create an input form which asks the user for the necessary information
--   to create a given parameterized page.
mkInputForm :: (ConnectCGI possibleValues labeling value handle,
                InputHandle handle)
            => String               -- ^ Button label
            -> labeling             -- ^ labeling information
            -> possibleValues       -- ^ description of possible values
            -> (value -> CGI ())    -- ^ parameterized page
            -> WithHTML x CGI ()    -- ^ input from
mkInputForm buttonLabel labeling possibleValues cgiGen = do
  let (getValue,mkHandle) = connectCGI
  h <- mkHandle labeling possibleValues
  submit h (cgiGen . getValue) (fieldVALUE buttonLabel)

-- | Create an input page which asks the user for the necessary information
--   to create a given parameterized page.
mkInputPage :: (ConnectCGI possibleValues labeling value handle,
                InputHandle handle)
            => String               -- ^ Page title
            -> String               -- ^ Button label
            -> labeling             -- ^ labeling information
            -> possibleValues       -- ^ description of possible values
            -> (value -> CGI ())    -- ^ parameterized page
            -> CGI ()               -- ^ input page
mkInputPage title buttonLabel labeling possibleValues cgiGen 
  = standardQuery' title $
      mkInputForm buttonLabel labeling possibleValues cgiGen
  
-- ** Instances for various input value types.
  
instance ConnectCGI () String FileReference (InputField FileReference) where
  connectCGI 
    = (value, \label () -> text_S label >> fileInputField empty)
  
instance ConnectCGI () () () F0 where
  connectCGI = (\F0 -> (), \() () -> return F0)

instance (ConnectCGI p1 l1 v1 h1, ConnectCGI p2 l2 v2 h2)
      => ConnectCGI (p1,p2) (l1,l2) (v1,v2) (F2 h1 h2) where
  connectCGI =
    ( \(F2 h1 h2) -> (getValue1 h1, getValue2 h2),
      \(l1,l2) (p1,p2) -> do
          h1 <- mkHandle1 l1 p1
          h2 <- mkHandle2 l2 p2
          return (F2 h1 h2)
    )
    where
      (getValue1,mkHandle1) = connectCGI
      (getValue2,mkHandle2) = connectCGI

instance (Show v, Eq v)
      => ConnectCGI [v] String [v] (InputField [v]) where
  connectCGI =
    ( value,
      \label possibleValues -> do
        <table>
        <tr><td><text><% label %></text></td></tr>
        <tr><td>
          <% selectMultiple show [] possibleValues (0,maxBound) empty %>
        </td></tr>
        </table>
    )

instance (Show v, Eq v)
      => ConnectCGI [v] String v (InputField v) where
  connectCGI =
    ( value,
      \label possibleValues -> do
        <table>
        <tr><td><text><% label %></text></td></tr>
        <tr><td>
          <% selectSingle show Nothing possibleValues empty %>
        </td></tr>
        </table>
    )

data PreSelect v = PreSelect Int [v]

instance (Show v, Eq v)
      => ConnectCGI (PreSelect v) String v (InputField v) where
  connectCGI =
    ( value,
      \label (PreSelect idx possibleValues) -> do
        let preSelect = if (0 <= idx && idx < (length possibleValues))
                          then Just (possibleValues!!idx)
                          else Nothing
        <table>
        <tr><td><text><% label %></text></td></tr>
        <tr><td>
          <% selectSingle show preSelect 
                          possibleValues empty %>
        </td></tr>
        </table>
    )
    
instance ConnectCGI String String String (InputField String) where
  connectCGI =
    ( value,
      \label possibleValue -> 
        <table>
        <tr><td><text><% label %></text></td></tr>
        <tr><td>
          <% makeTextarea possibleValue empty %>
        </td></tr>
        </table>
    ) 

instance ConnectCGI () String Bool (InputField Bool) where
  connectCGI =
    ( value,
      \label possibleValue -> do
        <text><% label %></text>
        checkboxInputField empty
    ) 

-- | Bool indicates echoing user input. HACK!
instance (Reason a, Read a) 
      => ConnectCGI Bool String a (InputField a) where
  connectCGI =
    ( value,
      \label hidden -> 
        if hidden 
          then text_S label >> passwordInputField empty
          else text_S label >> inputField empty 
    ) 

instance ConnectCGI () String String (InputField String) where
  connectCGI =
    ( value,
      \label () -> text_S label >> textInputField empty 
    ) 

type StringCheck = Maybe String -> Maybe String
instance ConnectCGI StringCheck String String (InputField String) where
  connectCGI =
    ( value,
      \label check -> text_S label >> checkedTextInputField check empty 
    ) 

newtype OneOf v = OneOf [v]
unOneOf (OneOf vs) = vs

instance (Read v, Show v) 
      => ConnectCGI (OneOf v) () v (RadioGroup v) where
  connectCGI =
    ( value,
      \() possibleValues -> do
         rg <- radioGroup
         table $
           mapM_ (\v -> tr $ td $ radioButton rg v empty ## text (show v)) 
                 (unOneOf possibleValues)
         radioError rg
         return rg
    )
         
-----------------------------------------------------------------------------
-- * Menus.

-- | Create a simple menu from a list of button labels and associated
--   CGIs.
simpleMenuFor :: [MenuButton]         -- ^ List of menu button specifications
              -> WithHTML x CGI ()
simpleMenuFor labeledCGIs =
  <table><tr><% menuButtons %></tr></table>
  where
    menuButtons = mapM_ (\(l,cgi) -> buttonFor l cgi) labeledCGIs

-- | Type of pull down menu specifications.
type PullDownMenu = (String,[MenuButton])

-- | Create a menu with pull-down submenus. Not very cool, actually.
fullMenuFor :: [PullDownMenu]    -- ^ List of sub-menu specifications. 
            -> WithHTML x CGI ()
fullMenuFor pullDownMenus =
  <table>
    <tr><% headers %></tr>
    <tr><% pullDowns %></tr>
  </table>
  where
    headers = mapM_ mkHeader pullDownMenus
    mkHeader (l,_) = 
      <th><text><% l %></text></th>      
    pullDowns = mapM_ mkPullDown pullDownMenus
    mkPullDown (l,buttons) = do
      let labels = Prelude.map fst buttons
      let buttonPressed = selectSingle id Nothing labels
      <td><% activate (\l -> assoc l buttons) buttonPressed empty %></td>
    assoc _ [] = error "Should not occur"
    assoc x' ((x,y):xys) | x==x' = y
                         | otherwise = assoc x' xys

-- | Type of menu button specifications.
type MenuButton = (String, CGI ())

-- | Create a continuation button for a non-parameterized CGI. Used to
--   create menu buttons.
buttonFor :: String     -- ^ Button label
          -> CGI ()     -- ^ Next page
          -> WithHTML x CGI ()
buttonFor label cgi =
  submit0 cgi (fieldVALUE label)


-----------------------------------------------------------------------------
-- * Low-level stuff.

-- | Embedd an IO operation into the WithHTML monad.
ioHTML :: (Show a, Read a) => IO a -> WithHTML x CGI a
ioHTML ioa = lift (io ioa)

-- | Like standardQuery, but without h1 caption.
standardQuery' :: String -> WithHTML x CGI a -> CGI ()
standardQuery' ttl elems =
  ask (htmlHeader ttl (body_S (makeForm elems)))

-----------------------------------------------------------------------------
-- * Graphics stuff.

-- | Call the GraphViz program dot.
callDot :: String      -- ^ Content for dot file. 
        -> String      -- ^ Output format, e.g. gif.
        -> FilePath    -- ^ Base name of image file.
        -> IO FilePath -- ^ Full file name of produced image file.
callDot dotString outFormat fileBaseName
  = do writeFile dotFileName dotString
       putStrLn ("Command: "++dotCommand)
       system dotCommand
       return imageFileName
    where
      dotFileName = "/tmp/"++fileBaseName++".dot"
      imageFileName  = "/CGI-Files/"++fileBaseName++"."++outFormat
      dotCommand = "dot -T"++outFormat++" "++dotFileName++
                   " > "++documentRoot++imageFileName

-- | Set the document root of the webserver.
documentRoot = "/Library/WebServer/Documents"

-- | Create HTML image element from image file name.
presentImage :: FilePath -- ^ Image file name
             -> WithHTML x CGI ()
presentImage fileName
  = <img src=<% fileName %> alt=<% fileName %> />
  
-----------------------------------------------------------------------------
-- * Multiple files stuff

callUnzip :: FileReference -> IO FilePath
callUnzip fileRef = do
  let fileName = fileReferenceName fileRef
  let dirName = fileName++".unzipped"
  system $ "mkdir -p "++dirName
  system $ "unzip -d "++dirName++" "++fileName
  return dirName

findByNamePattern :: FilePath -> String -> IO [FilePath]
findByNamePattern dir pat = do
  let resultFile = "/tmp/findResults"
  system $ "find "++dir++" -iname "++pat++" > "++resultFile
  content <- readFile resultFile
  return $ words content

-----------------------------------------------------------------------------


