-----------------------------------------------------------------------------
-- |
-- Module      :  Data.FiniteMapExtras
-- Copyright   :  (c) Alexandra Mendes, Joo Ferreira
-- License     :  GPL
--
-- Maintainer  :  Joo Ferreira, Alexandra Mendes
-- Stability   :  experimental
-- Portability :  experimental
--
-- Extra functions to use with FiniteMaps (includes all VDM-SL functions)
-- 
-----------------------------------------------------------------------------


module Data.FiniteMapExtras (
-- * FiniteMaps' basic functions
   domFM,             -- | VDM: dom m
   rngFM,             -- | VDM: rng m
   munion,            -- | VDM: munion m1 m2
   munionRel,   
   (+++), override,   -- | VDM: m1 ++ m2
   merge,             -- | VDM: merge ms
   (<:),              -- | VDM: s <: m 
   (<-:),             -- | VDM: s <-: m 
   (>:),              -- | VDM: m :> s
   (>-:),             -- | VDM: m :-> s
   compFM,            -- | VDM: m1 comp m2
   (***),             -- | VDM: m ** n 
   inverse,           -- | VDM: inverse m 
   inverse2,          -- | VDM: inverse m 
   m,                 -- | VDM: m(d) 
   injective,

-- * Extra functions
   mkr,fmToSet,
   setOfKeysFM,
   setOfEltsFM,

-- *  File IO
   readFile_FM,
   interact_FM
) where


import Maybe(fromJust)
import Data.FiniteMap
import Data.Set
import Data.Relation.SetOfPairs

import Camila.Combinadores

infix 5 +++ -- Override
infix 5 *** -- Map iteration
infix 5 <:  -- Domain restrict to
infix 5 <-: -- Domain restrict by
infix 5 >:  -- Range restrict to
infix 5 >-: -- Range restrict by


-- | Yields the domain (the set of keys) of a map.
domFM :: (Ord a) => FiniteMap a b -> Set a
domFM = foldFM (\key elt rest -> addToSet rest key) emptySet 


-- Yields the range (the set of information values) of a map.
rngFM :: (Ord a,Ord b) => FiniteMap a b -> Set b
rngFM = foldFM (\key elt rest -> addToSet rest elt) emptySet 


-- | Yields a map combined by two other maps, such that the resulting map
--   maps the elements of the domain of both maps. The two maps must have
--   disjoint domains.
munion :: (Ord a) => FiniteMap a b -> FiniteMap a b -> Maybe (FiniteMap a b)
munion m1 m2 | isEmptySet (intersect (domFM m1) (domFM m2)) = Just (m1 +++ m2)
             | otherwise = Nothing

-- | Yields a relation that has all pairs (key,elt) of the two given maps.
munionRel :: (Ord a,Ord b) => FiniteMap a b -> FiniteMap a b -> Rel a b
munionRel m1 m2 = (fmToSet m1) `union` (fmToSet m2)

-- | Overrides and merges two maps. It is like munion, except that both maps
--   don't need to be compatible; the values of the second map override the 
--   ones of the first.
(+++) :: (Ord a) => FiniteMap a b -> FiniteMap a b -> FiniteMap a b
(+++) = plusFM


-- | Same as (+++).
override :: (Ord a) => FiniteMap a b -> FiniteMap a b -> FiniteMap a b
override = (+++)


-- | Given a set of maps, yields the map that is contructed by merging them all.
--   The maps must be compatible.
merge :: (Ord a) => Set (FiniteMap a b) -> Maybe (FiniteMap a b)
merge s = foldr maux (Just emptyFM) (setToList s)
   where
      maux _ Nothing = Nothing
      maux a (Just c) = munion a c


-- | Given a set and a map, creates the map consisting of the elements whose key is in the set.
--   The set don't need to be a subset of the given map's domain.
(<:) :: (Ord a) => Set a -> FiniteMap a b -> FiniteMap a b
(<:) s = filterFM (\a b -> elementOf a s)  --- MAIS EFICIENTE :-)


-- | Given a set and a map, creates the map consisting of the elements whose key is not in the set.
--   The set don't need to be a subset of the given map's domain.
(<-:) :: (Ord a) => Set a -> FiniteMap a b -> FiniteMap a b
(<-:) s m = delListFromFM m (setToList s)


-- | Given a map and a set, creates the map consisting of the elements whose information value is in the set.
--   The set don't need to be a subset of the given map's range.
(>:) :: (Ord a,Ord b) => FiniteMap a b -> Set b -> FiniteMap a b
(>:) m s = filterFM (\a b -> elementOf b s) m


-- | Given a map and a set, creates the map consisting of the elements whose information value is not in the set.
--   The set don't need to be a subset of the given map's range.
(>-:) :: (Ord a, Ord b) => FiniteMap a b -> Set b -> FiniteMap a b
(>-:) m s = filterFM (\a b -> not (elementOf b s)) m


-- | Given a map and a key, yields the information value associated with that key, which must be in 
--   the domain of m.
m :: (Ord key) => FiniteMap key elt -> key -> Maybe elt
m = lookupFM


-- | Given two maps m1 and m2, yields the map that is created by composing m2
--   elements with m1 elements. The resulting map is a map with the same domain as m2.
--   The information value corresponding to a key is the one found by first applying m2 to the
--   key and then applying m1 to the result. rngFM m2 must be a subset of domFM m1.
compFM :: (Ord a) => FiniteMap a a -> FiniteMap a a -> Maybe (FiniteMap a a)
compFM m1 m2 | intersect (rngFM m2) (domFM m1) == (rngFM m2) = Just (listToFM (caux (fmToList m1) (fmToList m2)))
             | otherwise = Nothing
 where
  caux _ [] = []
  caux l2 ((a,b):ps) = let value = lookup b l2
                       in if value /= Nothing then (a, fromJust (lookup b l2)) : caux ps l2
                                              else caux ps l2


-- | Given a map m and a positive integer n, yields the map where m is composed with itself
--   n times. n=0 yields the identity map where each element of domFM m is map into itself;
--   n=1 yields m itself. For n>1, the range of m must be a subset of domFM m.
(***) :: (Ord a, Num b) => FiniteMap a a -> b -> Maybe (FiniteMap a a)
(***) m 0 = Just (listToFM [(a,a) | a<-setToList (domFM m)])
(***) m 1 = Just m
(***) m n = compFM m (fromJust (m *** (n-1)))


-- Equality
-- VDM: m1 = m2
-- FiniteMap module defines an instance of Eq


-- Inequality
-- VDM: m1 <> m2
-- FiniteMap module defines an instance of Eq

-- | Given a map m, yields the inverse map of m. m must be a 1-to-1 mapping.
inverse :: (Ord key,Ord elt) => FiniteMap key elt -> Maybe (FiniteMap elt key)
inverse m | injective m = Just $ listToFM.(map swap).fmToList $ m 
          | otherwise = Nothing
    where swap (a,b) = (b,a)

-- | Given a map m, returns true if m is injective.
injective :: (Ord key,Ord elt) => FiniteMap key elt -> Bool
injective m = (cardinality.rngFM $ m) == (cardinality.domFM $ m)


-- | Given a map m, yields the inverse map of m. m must be a 1-to-1 mapping.
--   This is a slightly more efficient version than inverse. 
inverse2 :: (Ord key,Ord elt) => FiniteMap key elt -> Maybe (FiniteMap elt key)
inverse2 m | injective m = Just $ foldFM (\ key elt rest -> addToFM rest elt key) emptyFM $ m
           | otherwise = Nothing


-- Show and Ord instances for FiniteMap

instance (Show a, Show b) => Show (FiniteMap a b) where
 showsPrec _ fm | isEmptyFM fm = showString "{|->}"
                | otherwise = showChar '{' . showListFM (fmToList fm) . showChar '}'
   where showListFM [] = showString ""
         showListFM ((k,e):t) = shows k . showString "|->" . shows e . showTail t

         showTail [] = showString ""
         showTail l = showChar ',' . showListFM l

instance (Ord a, Ord b) => Ord (FiniteMap a b) where
 m1 <= m2  =  (fmToList m1) <= (fmToList m2)


-- | Given a map m, yields the set of pairs (key,elt) where m(key)=elt, ie,
--   builds the relation defined by the map. mkr means 'make relation'.
mkr :: (Ord key, Ord elt) => FiniteMap key elt -> Rel key elt
mkr fm = foldFM (\ key elt rest -> addToSet rest (key,elt)) emptySet fm


-- |  Same as mkr. 
fmToSet :: (Ord key, Ord elt) => FiniteMap key elt -> Set (key,elt)
fmToSet = mkr 


-- | Given a map, yields the set of keys. It is the same as domFM.
setOfKeysFM :: (Ord key,Ord elt) => FiniteMap key elt -> Set key
setOfKeysFM = domFM


-- | Given a map, yields the set of elements. It is the same as rngFM.
setOfEltsFM :: (Ord key,Ord elt) => FiniteMap key elt -> Set elt
setOfEltsFM = rngFM


instance Functor (FiniteMap a) where
 fmap = mapFM . \f x y->f y

instance (Read a, Read b, Ord a) => Read (FiniteMap a b) where
  readsPrec n = map (\(x,y) -> (listToFM x, y)) .
                 chavetas (readSeqSep "," readElem)
   where
    --readElem :: (Read a,Read b) => ReadS (a,b)
    readElem = map (\ (((a,_),b),r) -> ((a,b),r) ) . 
               ((reads `depois` readConst "|->") `depois` reads)


-- | Applies a given function to a map read from a given file.  
readFile_FM :: (Read a, Read b, Ord a, Show c) =>
         FilePath -> ((FiniteMap a b) -> c) -> IO c
readFile_FM file f = readFile file >>= return.f.read


-- | Applies readFile_FM and writes the result in a given file.
interact_FM :: (Read a, Read b, Ord a, Show c) =>
         FilePath -> FilePath -> ((FiniteMap a b) -> c) -> IO ()
interact_FM inn out f = readFile_FM inn f >>= writeFile out.show
