-----------------------------------------------------------------------------
-- |
-- Copyright   :  (c) Joost Visser 2004
-- License     :  LGPL
-- 
-- Maintainer  :  joost.visser@di.uminho.pt
-- Stability   :  experimental
-- Portability :  portable
--
-- An implementation of relations as sets of pairs.
--
-----------------------------------------------------------------------------

module Data.Relation.FormalConceptAnalysis where

import Data.Relation.SetOfPairs
import Data.Relation.GraphViz

-----------------------------------------------------------------------------
-- * Representation

type Context g m = Rel g m
type Concept g m = (Set g, Set m)
type ConceptLattice g m = Rel (Concept g m) (Concept g m)

extent :: Concept g m -> Set g
extent (gs,ms) = gs

intent :: Concept g m -> Set m
intent (gs,ms) = ms

-----------------------------------------------------------------------------
-- * Basic operations

{-
-- | The neighborhood of a set of nodes in a graph are the nodes to which 
--   they are connected direclty, excluding the nodes themselves.
neighborhood :: Ord a => Set a -> Rel a a -> Set a
neighborhood seeds r
  = rngWith seeds r `minusSet` seeds

-- | The universal nodes in a graph are the nodes that are directly 
--   connected to all other nodes in the graph.
universals :: Ord a => Rel a a -> Set a
universals r = mkSet [ a | a <- setToList entr, 
                           entr == addToSet (seenFrom a) a ]
  where
    entr = ent r
    seenFrom a
      = mkSet [ y | (x,y) <- setToList r, x==a ]
        `union`
        mkSet [ x | (x,y) <- setToList r, y==a ]
-}

-- | Compute the extent of a concept from a given intent, given a
--   formal context.
extentFromIntent :: (Ord g, Ord m) => Context g m -> Set m -> Set g
extentFromIntent cxt intent
  = mkSet [ g | g <- setToList (dom cxt),  
                and [ (g,m) `elementOf` cxt | m <- setToList intent ]
          ]

-- | Compute the intent of a concept from a given extent, given a
--   formal context.
intentFromExtent :: (Ord g, Ord m) => Context g m -> Set g -> Set m
intentFromExtent cxt extent
  = mkSet [ m | m <- setToList (rng cxt),  
                and [ (g,m) `elementOf` cxt | g <- setToList extent ]
          ]

conceptsFromIntent :: (Ord g, Ord m) => Context g m -> Set m -> [Concept g m]
conceptsFromIntent cxt intent
  = undefined

-- | From the extent of a given context, compute all concepts greater than
--   that context.
conceptsFromExtent :: (Ord g, Ord m) => Context g m -> Set g -> [Concept g m]
conceptsFromExtent cxt extent
  = [ let g' = extent `union` (unitSet g)
          i  = intentFromExtent cxt g'
          e  = extentFromIntent cxt i
      in (e,i)
      | g <- setToList (dom cxt), not (g `elementOf` extent) ]

-- | Produce the dual of a given lattice.
dualConceptLattice :: (Ord g, Ord m) 
                   => ConceptLattice g m -> ConceptLattice m g
dualConceptLattice l
  = mapSet dual l
    where
      dual ((g1,m1),(g2,m2)) = ((m2,g2),(m1,g1))

-----------------------------------------------------------------------------
-- * Predicates

isConceptOf :: (Ord g, Ord m) => Concept g m -> Context g m -> Bool
isConceptOf (g,m) cxt
  = g `isSubsetOf` dom cxt &&
    m `isSubsetOf` rng cxt &&
    extentFromIntent cxt m == g &&
    intentFromExtent cxt g == m 
    where
      isSubsetOf sub super = isEmptySet (sub `minusSet` super)

conceptGT :: (Ord g, Ord m) => Concept g m -> Concept g m -> Bool
conceptGT (g1,m1) (g2,m2)
  = g2 `isStrictSubsetOf` g1 &&
    m1 `isStrictSubsetOf` m2
    where
      isStrictSubsetOf sub super 
        = isEmptySet (sub `minusSet` super) &&
          not (isEmptySet (super `minusSet` sub))
      
-----------------------------------------------------------------------------
-- * Fast Concept Analysis by Christian Lindig

-- | Compute the upper neighbors of a concept.
neighbors :: (Ord g, Ord m) 
          => Set g             -- ^ extent of concept 
          -> Context g m       -- ^ formal context
          -> [Concept g m]     -- ^ list of neighbors
neighbors extent cxt
  = worker (setToList extentComp) extentComp []
    where
      extentComp = dom cxt `minusSet` extent 
      worker [] min ns = ns
      worker (g:gs) min ns
        = worker gs min' ns'
          where
            m1 = intentFromExtent cxt (addToSet extent g)
            g1 = extentFromIntent cxt m1
            new = (g1 `minusSet` extent) `delFromSet` g
            (min',ns')
               = if (isEmptySet (min `intersect` new))
                  then (min, (g1,m1):ns)
                  else (delFromSet min g,ns)

-- | Computes the concept lattice of a given formal context.
lattice :: (Ord g, Ord m) 
        => Context g m         -- ^ formal context
        -> ConceptLattice g m  -- ^ corresponding concept lattice
lattice cxt = worker emptyRel [] [(initExtent,initIntent)]
  where
    initIntent = intentFromExtent cxt emptySet
    initExtent = extentFromIntent cxt initIntent
    worker l done [] = l
    worker l done (c@(extent,intent):todo)
      | extent `elem` done
      = worker l done todo
      | otherwise
      = worker l' done' todo'
        where
          ns = neighbors extent cxt
          l' = mkSet (map (\c' -> (c',c)) ns) `union` l
          done' = (extent:done)
          todo' = ns ++ todo
{-
condenseLattice :: (Ord g, Ord m) 
        => ConceptLattice g m -> ConceptLattice g m
condenseLattice l
  = mapSet (\(x,y) -> (condenseConcept x, condenseConcept y)) l
    where
      condenseConcept c@(g,m) 
        = ( g `minusSets` mapSet fst (domWith (unitSet c) l)
          , m `minusSets` mapSet snd (domWith (unitSet c) l)
          )

minusSets :: Ord g => Set g -> Set (Set g) -> Set g
--minusSets g ss =  (foldr (\s g -> g `minusSet` s) g (setToList ss))
minusSets g ss = intersectManySets (setToList ss)

intersectManySets [] = emptySet
intersectManySets ss = foldr1 intersect ss 
-}

-----------------------------------------------------------------------------
-- * Printing concept lattices.

-- | Compute the difference between a concept and its immediate neighbors.
--   Thus, the concept's extent is reduced to the objects that are not 
--   present among its lower neighbors, and and its intent is reduced to
--   the attributes that are not present among its upper neighbors.
--   Useful when displaying a concept lattice in condensed form.
conceptDelta :: (Ord g, Ord m)
             => ConceptLattice g m 
             -> Concept g m
             -> Concept g m
conceptDelta l c@(g,m)
  = (g',m')
    where 
      g' = g `minusSet` unionManySets lowerExtents
      m' = m `minusSet` unionManySets upperIntents
      lowerExtents = [ g'' | (c',(g'',_)) <- setToList l, c'==c ]
      upperIntents = [ m'' | ((_,m''),c') <- setToList l, c'==c ]

-- | Print a concept lattice as a dot graph.
printConceptLattice :: (Ord g, Ord m)
                    => (Set g -> String)
                    -> (Set m  -> String)
                    -> ConceptLattice g m
                    -> String
printConceptLattice showG showM l
  = printConceptLatticeWith showG showM showConcept l
    where
      showConcept (g,m)
        = (showG g)++"\\n"++(showM m)

-- | Print a concept lattice as a dot graph. Extra parameters for
--   controlling the labels for concept nodes.
printConceptLatticeWith :: (Ord g, Ord m)
                    => (Set g -> String)
                    -> (Set m  -> String)
                    -> (Concept g m -> String)
                    -> ConceptLattice g m
                    -> String
printConceptLatticeWith showG showM showConcept l
  = printGraph "lattice" l
    where
      printGraph 
        = printGraphWith conceptToNodeName showConceptLabel
      conceptToNodeName (g,m)
        = (showG g)++"\\n"++(showM m)
      showConceptLabel l c
        = showConcept (conceptDelta l c)

sepWith :: a -> [a] -> [a]
sepWith _ [] = []
sepWith a [x] = [x]
sepWith a (x:xs) = x:a: sepWith a xs

-----------------------------------------------------------------------------

-- a formal context for testing purposes.
cxt1
  = mkSet [('a','2'),('a','3'),('a','6'),
           ('b','1'),('b','2'),('b','3'),
           ('c','1'),('c','2'),('c','5'),
           ('d','1'),('d','4'),('d','5'),
           ('e','1'),('e','4'),
           ('f','3')]

-----------------------------------------------------------------------------

