-----------------------------------------------------------------------------
-- |
-- Module      :  Data.Relation.MapFromPairs
-- Copyright   :  (c) Joost Visser 2004
-- License     :  LGPL
-- 
-- Maintainer  :  joost.visser@di.uminho.pt
-- Stability   :  experimental
-- Portability :  portable
--
-- An implementation of labeled relations as map from pairs to values.
--
-----------------------------------------------------------------------------

module Data.Relation.MapFromPairs (
  module Data.Relation.MapFromPairs,
  module Data.FiniteMap
) where

import Data.FiniteMap
import Data.Set
import Data.Relation.SetOfPairs

-----------------------------------------------------------------------------
-- * Representation

-- | The type of labeled releations.
type LRel a b c  = FiniteMap (a,b) c

instance (Show a, Show b) => Show (FiniteMap a b) where
  show xs = "{" ++ show' (fmToList xs) ++  "}"
            where
             show' []        = ""
             show' (x:xs)    = showItem x ++ showl xs
             showl []        = ""
             showl (x:xs)    = "," ++ (showItem x) ++ (showl xs)
             showItem (a,b)  = show a ++ "->" ++ show b

-- | Convert labeled relation to one without labels. The first argument
--   is a predicate that determines, based on the label, which pairs are
--   to be included in the result relation.
rel         :: (Ord a, Ord b) 
            => (c -> Bool)       -- ^ Predicate on labels.
            -> LRel a b c 
            -> Rel a b
rel p fm    = mkSet (keysFM (filterFM (\(a,b) c -> p c) fm))

-- | Convert relation to a labeled one. The first argument is the 
--   label with which all pairs in the result relation will be labeled.
lrel        :: (Ord a, Ord b) 
            => c                -- ^ Initialization label
            -> Rel a b 
            -> LRel a b c
lrel c r    = listToFM (zip (pairs r) (repeat c)) 

-- | Carrier set.
entities    :: (Ord a) => LRel a a c -> Set a
entities lr = mkSet ((map fst pairs) ++ (map snd pairs))
              where
                pairs = keysFM lr       

-----------------------------------------------------------------------------
-- * Extremal paths.

-- | Generic extremal path algorithm, based on Roland Backhouse's 
--   lecture notes, page 192 (draft version).
extremal :: Ord a 
            => b                -- ^ Zero of multiplication.
            -> (b -> b -> b)    -- ^ Multiplication. 
            -> (b -> b -> b)    -- ^ Addition.
            -> LRel a a b -> LRel a a b
extremal zero and or lr
            = foldr step lr nodes
              where
                nodes         = setToList (entities lr)
                step k lr     = mk [((i,j), value i j k lr) 
                                             | i <- nodes
                                             , j <- nodes 
                                   ]
                value i j k lr 
                              = (p lr (i,j)) 
                                `or` ( (p lr (i,k))
                                     `and` (p lr (k,j)) )
                mk            = listToFM
                p lr (i,j)    = lookupWithDefaultFM lr zero (i,j)

-- | Reachability. This is transitive closure following Roy-Warshall.
reach       :: Ord a => LRel a a Bool -> LRel a a Bool
reach r     = extremal False (&&) (||) r

-- | Least cost, or shortest path. 
leastcost       :: (Num cost, Ord cost, Ord a) 
                => cost               -- ^ Maximum bound. 
                -> LRel a a cost      -- ^ Cost labels must be non-negative.
                -> LRel a a cost
leastcost maxBound r 
                = extremal maxBound (+) min r

-- | Bottle neck.
bottleneck      :: (Ord height, Num height, Ord a) 
                => LRel a a height      -- ^ Height labels must be non-negative.
                -> LRel a a height
bottleneck r    = extremal 0 min max r

-- | Least cost, or shortest path. DOES NOT WORK, because it relies on 
--   maxBound from the Bounded class, leading to erroneous arithmetic.
leastcost'      :: (Num cost, Ord cost, Bounded cost, Ord a) 
                => LRel a a cost -> LRel a a cost
leastcost' r    = extremal maxBound (+) min r

-----------------------------------------------------------------------------
