{-# OPTIONS -fglasgow-exts #-}
-----------------------------------------------------------------------------
-- |
-- Module      :  Data.Relation.SetOfPairs
-- Copyright   :  (c) Joost Visser 2004
-- License     :  LGPL
-- 
-- Maintainer  :  joost.visser@di.uminho.pt
-- Stability   :  experimental
-- Portability :  portable
--
-- An implementation of relations as sets of pairs.
--
-----------------------------------------------------------------------------

module Data.Relation.SetOfPairs (
  module Data.Relation.SetOfPairs,
  module Data.Set
) where

import Data.Set
import Control.Monad.State

-----------------------------------------------------------------------------

-- * Representation

-- | Type of relations
type Rel a b = Set (a,b)

instance Show a => Show (Set a) where
  show xs = "{" ++ show' (setToList xs) ++  "}"
            where
              show' []     = ""
              show' (x:xs) = show x ++ showl xs
              showl []     = ""
              showl (x:xs) = "," ++ show x ++ showl xs

instance (Ord a, Read a) => Read (Set a) where
  readsPrec p = readSet'

readSet'  :: (Ord a,Read a) => ReadS (Set a)
readSet' str = map (\(a,b)->(mkSet a,b)) (readSet str)

readSet   :: Read a => ReadS [a]
readSet   = readParen False (\r -> [pr | ("{",s) <- lex r,
                                         pr      <- readl s ])
            where readl  s = [([],t)   | ("}",t) <- lex s] ++
                             [(x:xs,u) | (x,t)   <- reads s,
                                         (xs,u)  <- readl' t]
                  readl' s = [([],t)   | ("}",t) <- lex s] ++
                             [(x:xs,v) | (",",t) <- lex s,
                                         (x,u)   <- reads t,
                                         (xs,v)  <- readl' u]

instance Ord a => Ord (Set a) where
  compare a b
    = compare (setToList a) (setToList b)  

-- | Type of graphs: relations with domain and range of the same type.
type Gph a = Rel a a       

-----------------------------------------------------------------------------
-- * Building relations           

-- | Build an empty relation.              
emptyRel        :: Rel a b
emptyRel        = emptySet

-- | Build a relation from a list of pairs.
mkRel           :: (Ord a, Ord b) => [(a,b)] -> Rel a b
mkRel pairs     = mkSet pairs

-- | Build a relation from distributing an element to a set of elements
mkRelNeighbors      :: (Ord a, Ord b) => a -> [b] -> Rel a b
mkRelNeighbors a l  = mkSet [ (a, x) | x <- l ]

-- | Build identity relation, which contains an edge from each node to itself.
identityRel      :: Ord a => Set a -> Rel a a
identityRel s    = mapSet (\x -> (x,x)) s

-- | Build total relation, which contains an edge from each node to 
--   each other node and to itself.
totalRel         :: Ord a => Set a -> Rel a a
totalRel s       = mkSet [ (x,y) |  x <- l, y <- l ]
                   where
                     l = setToList s

-- | Build a chain relation of given number of numerals.
chainRel        :: (Enum n, Num n, Ord n) =>  n -> Rel n n
chainRel n      = mkSet (map (\i -> (i,i+1)) [1..n])

-----------------------------------------------------------------------------
-- * Basic operations

-- | Obtain the domain of a relation
dom             :: Ord a => Rel a b -> Set a
dom xs          = mapSet fst xs

-- | Obtain the range of a relation
rng             :: Ord b => Rel a b -> Set b
rng xs          = mapSet snd xs

-- | Obtain all entities in a relation (union of domain and range)
ent             :: Ord a => Rel a a -> Set a
ent xs          = dom xs `union` rng xs

-- | Convert relation to a list of pairs.
pairs           :: (Ord a, Ord b) => Rel a b -> [(a,b)]
pairs r         = setToList r

-- | Take the inverse of a relation
inv            :: (Ord a, Ord b) => Rel a b -> Rel b a
inv xs         = mapSet (\(x,y) -> (y,x)) xs

-- | Compose two relations
comp            :: (Ord a, Eq b, Ord c) => Rel b c -> Rel a b -> Rel a c
comp yz xy      = mkSet 
                    [ (x,z) | (x,y) <- setToList xy
                            , (y',z) <- setToList yz
                            , y==y'
                    ]

-----------------------------------------------------------------------------
-- * Closures

-- | Compute the reflective closure
reflClose       :: Ord a => Rel a a -> Rel a a
reflClose xs    = xs `union` mapSet (\x -> (x,x)) (ent xs)

-- | Compute the symmetric closure
symmClose       :: Ord a => Rel a a -> Rel a a
symmClose xs    = xs `union` mapSet (\(x,y) -> (y,x)) xs
            
-- | Compute the transitive closure
transClose       :: Ord a => Rel a a -> Rel a a
transClose xs    = if xxs == xs then xs else (transClose xxs)
                   where 
                     xxs = union xs (comp xs xs)
            
-- | Compute the reflexive, transitive closure
reflTransClose    :: Ord a => Rel a a -> Rel a a
reflTransClose xs = reflClose (transClose xs)

-- | Compute the reflexive, transitive, symmetric closure, i.e. the
--   induced equivalence relation.
equiv      :: Ord b => Rel b b -> Rel b b
equiv r    = reflTransClose (r `union` (inv r))


-----------------------------------------------------------------------------
-- * Selections (project, slice, chop)

-- | Obtain the subdomain of a relation given a subrange.
domWith             :: (Ord a, Ord b) => Set b -> Rel a b -> Set a
domWith bs r        = mkSet [ a | (a,b) <- setToList r , b `elementOf` bs ]

-- | Obtain the subrange of a relation given a subdomain.
rngWith             :: (Ord a, Ord b) => Set a -> Rel a b -> Set b
rngWith as r        = mkSet [ b | (a,b) <- setToList r , a `elementOf` as ]

-- | Obtain the subrange and subdomain of a relation given a subdomain.
entWith             :: Ord a => Set a -> Rel a a -> Set a
entWith as r        = domWith as r `union` rngWith as r

-- | Remove all edges of the form (x,x).
removeSelfEdges :: Ord a => Gph a -> Gph a
removeSelfEdges r
  = mkRel [ (x,y) | (x,y) <- setToList r, x /= y ]
  
-- | Retrieve a subrelation given predicates on domain and range.
projectWith :: (Ord a, Ord b) 
            => (a -> b -> Bool) 
            -> Rel a b -> Rel a b
projectWith p r
            = mkSet [ (x,y) | (x,y) <- setToList r, p x y]

-- | Projection of set through relation
project     :: (Ord a, Ord b) => Set a -> Rel a b -> Rel a b
project    s r     = mkSet [ (x,y) | (x,y) <- setToList r, x `elementOf` s ]

-- | Projection of set backward through relation
projectBackward    :: (Ord a, Ord b) => Set b -> Rel a b -> Rel a b
projectBackward    s r
             = mkSet [ (x,y) | (x,y) <- setToList r, y `elementOf` s ]

-- | Compute forward slice starting from seeds.
slice            :: Ord a => Set a -> Rel a a -> Rel a a
slice seed r        = sliceUntil seed emptySet r

-- | Compute forward slice starting from seeds, stopping at stoppers.
sliceUntil        :: Ord a => Set a -> Set a -> Rel a a -> Rel a a
sliceUntil seed stop r    
                = if (isEmptySet s) then emptyRel 
                                    else r' `union`
                                         (sliceUntil seed' stop' r)
              where
                s = seed `minusSet` stop    -- live seeds
                r' = project s r            -- step
                seed' = rng r'              -- new seeds
                stop' = stop `union` s      -- extend stoppers

-- | Compute backward slice starting from seeds.
sliceBackward        :: Ord a => Set a -> Rel a a -> Rel a a
sliceBackward seed r    = inv (slice seed (inv r))

-- | Compute chop between seeds and sinks.
chop                    :: Ord a => Set a -> Set a -> Rel a a -> Rel a a
chop seeds sinks r  = slicesWith intersect seeds sinks r
                      
-- | Compute union of backward and forward slice.
slicesUnion :: Ord a => Set a -> Set a -> Rel a a -> Rel a a
slicesUnion seeds sinks r
  = (slice seeds r) `union` (sliceBackward sinks r)

-- | Compute intersection of backward and forward slice.
--   This is the same as the computing the chop between seeds and sinks.
slicesIntersect :: Ord a => Set a -> Set a -> Rel a a -> Rel a a
slicesIntersect seeds sinks r  
  = (slice seeds r) `intersect` (sliceBackward sinks r)
  
-- | Compute combination of backward and forward slice, where
--   a binary operator is supplied to specify the kind of combination.
slicesWith :: Ord a 
           => (Rel a a -> Rel a a -> Rel a a) 
           -> Set a -> Set a -> Rel a a -> Rel a a
slicesWith binop seeds sinks r
  = (slice seeds r) `binop` (sliceBackward sinks r)

-- | Compute slice or chop, depending on whether the source or sink set or both 
--   are empty.
sliceOrChopWith :: (Ord a) 
                => (Rel a a -> Rel a a -> Rel a a)  -- ^ binary graph operation
                -> [a]                              -- ^ sources
                -> [a]                              -- ^ sinks
                -> Rel a a                          -- ^ input relation
                -> Rel a a                          -- ^ output relation
sliceOrChopWith binop [] [] rel         = rel
sliceOrChopWith binop [] sinks rel      = sliceBackward (mkSet sinks) rel
sliceOrChopWith binop sources [] rel    = slice (mkSet sources) rel
sliceOrChopWith binop sources sinks rel 
  = slicesWith binop (mkSet sources) (mkSet sinks) rel
      
-----------------------------------------------------------------------------
-- * Partitions

-- | Compute subrelation connected to seeds, stopping at stoppers.
gobbleUntil        :: Ord a => Set a -> Set a -> Rel a a -> Rel a a
gobbleUntil seed stop r    
                = if (isEmptySet s) then emptyRel 
                                    else r' `union`
                                         (gobbleUntil seed' stop' r)
              where
                s = seed `minusSet` stop    -- live seeds
                r' = (project s r) `union` (projectBackward s r)   -- step
                seed' = ent r'              -- new seeds
                stop' = stop `union` s      -- extend stoppers

-- | Compute weakly connected components.
weakComponents :: Ord a => Rel a a -> [Rel a a]
weakComponents r
  = weakComponentsUntil emptySet r
    where
      weakComponentsUntil done r
        = case setToList ((ent r) `minusSet` done) of
           []    -> []
           (a:_) -> (r':s)
                    where
                      r' = gobbleUntil (unitSet a) emptySet r
                      s  = weakComponentsUntil (done `union` (ent r')) r   

-- | Compute a single strong component (set of nodes).
strongComponent :: (Ord a) => a -> Gph a -> Set a
strongComponent x r
  = addToSet reach x
    where
      reach = (reachableFrom x r) `intersect` (reachableFrom x (inv r))
      reachableFrom x r
        = ent (slice (unitSet x) r)
        
-- | Compute the set of strong components (sets of nodes).
strongComponentSet :: (Ord a, Ord (Set a)) => Gph a -> Set (Set a)
strongComponentSet r
  = mapSet (\x -> strongComponent x r) (ent r)

-- | Type of components: node set tupled with subrelation.
type Component a = (Set a, Rel a a)

-- | Compute a single strong component (node set tupled with subrelation).
strongComponent' :: (Ord a) => a -> Rel a a -> Component a
strongComponent' x r
  = (scc,subrel)
    where
      scc = strongComponent x r
      subrel = mkRel [ (x,y) | (x,y) <- setToList r, 
                               x `elementOf` scc, y `elementOf` scc ]

-- | Compute the graph of strong components (node sets).
strongComponentRel :: (Ord a, Ord (Set a)) => Gph a -> Gph (Set a) 
strongComponentRel r
  = removeSelfEdges $ mapSet edgeT r
    where
      edgeT (x,y) = (nodeT x, nodeT y)
      nodeT x = (strongComponent x r)

-- | Compute the graph of stong components (node sets tupled with subrelations).
strongComponentRel' :: (Ord a, Ord (Set a)) 
                    => Gph a -> Gph (Component a)
strongComponentRel' r
  = removeSelfEdges $ mapSet edgeT r
    where
      edgeT (x,y) = (nodeT x, nodeT y)
      nodeT x = strongComponent' x r

-- | Compute the set of strong components that have more than a single node
strongNonSingletonComponentSet r
  = mkSet [ c | c <- setToList $ strongComponentSet r, cardinality c > 1 ]
    
-----------------------------------------------------------------------------
-- * Integration

-- | Integrate two graphs with respect to a base graph into a
--   new graph that contains the differences of each input graph with
--   respect to the base graph. The boolean value indicates whether the
--   graphs are interference-free, i.e. whether the integration is valid. 
integrate :: Ord a => Rel a a -> Rel a a -> Rel a a -> (Rel a a,Bool)
integrate a b base
  = (m,ok)
    where
      apa = affectedPoints a base
      apb = affectedPoints b base
      aapa = a // apa
      bapb = b // apb
      basepp  = base // (preservedPoints a b base)
      m = aapa `union` bapb `union` basepp
      ok = m//apa == aapa && m//apb == bapb
      g / v = sliceBackward (unitSet v) g
      g // s = sliceBackward s g

-- | Points in the first graph that are affected by changes with respect
--   to the base graph.
affectedPoints :: Ord a => Rel a a -> Rel a a -> Set a
affectedPoints x base
        = mkSet [ v | v <- setToList (ent x), (base/v) /= (x/v) ]
          where
            g / v = sliceBackward (unitSet v) g

-- | Points in the base graph that are not affected by changes in either
--   input graph.
preservedPoints ::  Ord a => Rel a a -> Rel a a -> Rel a a -> Set a
preservedPoints x y base
        = mkSet [ v | v <- setToList (ent base), 
                      let  base' = base/v in base' == (x/v) && base' == (y/v)
                ]
          where
            g / v = sliceBackward (unitSet v) g
          
-----------------------------------------------------------------------------
-- | Traversal (under construction)


dfs r
  = dfsFrom r [] (setToList $ ent r)
    

dfsFrom r stop []
  = []
dfsFrom r stop (n:ns)
  | n `elem` stop
  = dfsFrom r stop ns
  | otherwise
  = fromKids++(n:(dfsFrom r stop'' ns))
    where
      kids = [ y | (x,y) <- setToList r, x==n ]
      stop' = n:stop
      fromKids = dfsFrom r stop' kids
      stop'' = stop'++fromKids
      
dfsFrom' r (stop,(n:ns))
  | n `elem` stop
  = dfsFrom' r (stop,ns)
  | otherwise
  = (stop''',fromKids++(n:fromRest))
    where
      kids = [ y | (x,y) <- setToList r, x==n ]
      stop' = n:stop
      (stop'',fromKids)  = dfsFrom' r (stop',kids)
      (stop''',fromRest) = dfsFrom' r (stop'',ns)
dfsFrom' r (stop,[])
  = (stop,[])
            
dfs' r
  = snd $ dfsFrom' r ([],(setToList $ ent r))
   
dfsS r
  = evalState (dfsState r) ([],setToList $ ent r)
dfsState r  = do
  (stop,ns) <- get
  case ns of
    (n:ns)
      -> if (n `elem` stop)
          then do
            put (stop,ns)
            dfsState r
          else do
            let kids = [ y | (x,y) <- setToList r, x==n ]
            put (n:stop,kids)
            fromKids <- dfsState r
            (stop,_) <- get
            put (stop,ns)
            fromRest <- dfsState r
            return (fromKids++(n:fromRest))
    []
      -> return ns

dfsT r
  = evalState (dfsStateT r) ([],setToList $ ent r,emptyRel)
dfsStateT r  = do
  (stop,ns,t) <- get
  case ns of
    (n:ns)
      -> if (n `elem` stop)
          then do
            put (stop,ns,t)
            dfsStateT r
          else do
            let entT = ent t
            let edges = [ (x,y) | (x,y) <- setToList r, 
                                  x==n, not (y `elementOf` entT) ]
            let kids = map snd edges
            put (n:stop,kids,t `union` (mkRel edges))
            dfsStateT r
            (stop,_,t) <- get
            put (stop,ns,t)
            dfsStateT r
    []
      -> return t


-- http://www.ics.uci.edu/~eppstein/161/960220.html
-----------------------------------------------------------------------------
