-----------------------------------------------------------------------------
-- |
-- Module      :  Data.Relation.SetOfPairsQuickCheck
-- Copyright   :  (c) Joost Visser 2004
-- License     :  LGPL
-- 
-- Maintainer  :  joost.visser@di.uminho.pt
-- Stability   :  experimental
-- Portability :  portable
--
-- Properties of the implementation of relations as sets of pairs.
--
-----------------------------------------------------------------------------

module Data.Relation.SetOfPairsQuickCheck where

import Data.Set
import Data.Relation.SetOfPairs
import Debug.QuickCheck

-----------------------------------------------------------------------------

instance (Ord a, Arbitrary a)
      => Arbitrary (Set a) where
  arbitrary = do xs <- arbitrary
                 return (mkSet xs)

-- | Inverting twice is identity.
prop_InvInv r	
  = classify (cardinality r == 0) "empty" $
    inv (inv r) == r
    where 
      types = r::Rel Int Integer

-- | Domain of inverse is range.
prop_DomInvRng :: Rel Int Integer -> Property
prop_DomInvRng r
  = classify (cardinality r == 0) "empty" $
    dom (inv r) == rng r    
    where 
      types = r::Rel Int Integer

-- | Range of inverse is domain.
prop_RngInvDom r
  = classify (cardinality r == 0) "empty" $
    rng (inv r) == dom r    
    where 
      types = r::Rel Int Integer

-- | Carrier of inverse is cariere of original.
prop_EntInvEnt r
  = classify (cardinality r == 0) "empty" $
    ent (inv r) == ent r    
    where 
      types = r::Rel Int Int

-- | Inverse distributes over composition.
prop_CompInvDist r1 r2
  = classify (cardinality r1 == 0 && cardinality r2 == 0) "both empty" $
    classify (cardinality r2 /= 0 && cardinality r2 /= 0) "none empty" $
    inv (r2 `comp` r1) == (inv r1) `comp` (inv r2)
    where 
      types = (r1::Rel Int Integer, r2::Rel Integer Double)

-- | 
close_prop close r 
  = classify (cardinality r == 0) "empty" $
    close r == close (close r)
    where
      types = r:: Rel Int Int
      
prop_ReflClose r
  = close_prop reflClose r
  
prop_SymmClose r
  = close_prop symmClose r
  
prop_TransClose r
  = close_prop transClose r

prop_ReflTransClose r
  = close_prop reflTransClose r

close_prop_Ent close r 
  = classify (cardinality r == 0) "empty" $
    ent r == ent (close r)
    where
      types = r:: Rel Int Int

prop_RelfCloseEnt r
  = close_prop_Ent reflClose r  
      
prop_SymmCloseEnt r
  = close_prop_Ent symmClose r      
  
prop_TransCloseEnt r
  = close_prop_Ent transClose r    
    
prop_ReflTransCloseEnt r
  = close_prop_Ent reflTransClose r      

prop_ReflCloseDomRng r
  = classify (cardinality r == 0) "empty" $
    dom (reflClose r) == rng (reflClose r)
    where
      types = r:: Rel Int Int  
      
prop_ReflCloseDomEnt r
  = classify (cardinality r == 0) "empty" $
    dom (reflClose r) == ent r
    where
      types = r:: Rel Int Int     

prop_ProjectInv s r
  = classify (cardinality s == 0 && cardinality r == 0) "both empty" $
    classify (cardinality s /= 0 && cardinality r /= 0) "none empty" $
    project s r == inv (projectBackward s (inv r))
    where
      types = (s::Set Int, r::Rel Int Integer)

prop_ChopInv seeds sinks r
  = classify (cardinality seeds /= 0 && cardinality sinks /= 0 && cardinality r /= 0)
        "none empty" $
    chop seeds sinks r == inv (chop sinks seeds (inv r))
    where
      types = (r::Rel Int Int)

prop_SliceIdentity s
  = classify (cardinality s /= 0) "non-empty" $
    slice s (identityRel s) == (identityRel s)
    where
      types = (s::Set Int)

prop_SliceTotal s
  = classify (cardinality s /= 0) "non-empty" $
    slice s (totalRel s) == (totalRel s)
    where
      types = (s::Set Int)

prop_SliceIntersect seeds sinks r
  = classify noneEmpty "none empty" $
    classify resultNonEmpty "result non-empty" $
    slicetwice == intersectslices
    where
      slicetwice = sliceBackward sinks (slice seeds r)
      intersectslices = (slice seeds r) `intersect` (sliceBackward sinks r)
      noneEmpty = cardinality seeds /= 0 && cardinality sinks /= 0 && cardinality r /= 0
      resultNonEmpty = cardinality slicetwice /= 0
      types = (r::Rel Int Int)
  
prop_WeakComponentsIdempotent r
  = classify (cardinality r /= 0) "non-empty" $
    classify (length once >= 2) "several components" $
    once == twice
    where
      once = weakComponents r
      twice = concatMap weakComponents once  
      types = (r::Rel Int Int)
  
prop_WeakComponentsNoLoss r
  = classify (cardinality r /= 0) "non-empty" $
    classify (length components >= 2) "several components" $
    r == foldr union emptySet components
    where
      components = weakComponents r
      types = (r::Rel Int Int)

----

prop_IntegrateNoInterferenceBothEmpty r
  = classify (cardinality r /= 0) "non-empty" $
    classify connected "connected" $
    (ok && m==emptyRel)
    where
      (m,ok) = integrate emptyRel emptyRel r
      connected = length (weakComponents r) == 1
      types = (r::Rel Int Int)

prop_IntegrateNoInterferenceBaseEmptyBothEqual r
  = classify (cardinality r /= 0) "non-empty" $
    classify connected "connected" $
    (ok && m==r)
    where
      (m,ok) = integrate r r emptyRel
      connected = length (weakComponents r) == 1
      types = (r::Rel Int Int)

prop_IntegrateNoInterferenceOneEmpty r
  = classify (cardinality r /= 0) "non-empty" $
    classify connected "connected" $
    (ok && m==emptyRel)
    where
      (m,ok) = integrate emptyRel r r
      connected = length (weakComponents r) == 1
      types = (r::Rel Int Int)

prop_IntegrateNoInterferenceOneNonEmpty r
  = classify (cardinality r /= 0) "non-empty" $
    classify connected "connected" $
    (ok && m==r)
    where
      (m,ok) = integrate emptyRel r emptyRel
      connected = length (weakComponents r) == 1
      types = (r::Rel Int Int)

prop_IntegrateNoInterferenceSliceOfBase s r
  = classify (cardinality r /= 0) "non-empty" $
    classify connected "connected" $
    (ok && m==r')
    where
      r' = slice s r
      (m,ok) = integrate r' r r
      connected = length (weakComponents r) == 1
      types = (r::Rel Int Int)

prop_IntegrateNoInterferenceSliceOfOne s r
  = classify (cardinality r /= 0) "non-empty" $
    classify connected "connected" $
    (ok && m==r)
    where
      r' = slice s r
      (m,ok) = integrate r r' r'
      connected = length (weakComponents r) == 1
      types = (r::Rel Int Int)

prop_IntegratePreservedpoints a b base
  = --classify allEmpty "all empty" $
    classify (not allEmpty) "not all empty" $
    classify ok "no interference" $
    if ok then m//pp == base//pp else True
    where
      allEmpty = cardinality (a`union`b`union`base`union`m) == 0
      (m,ok) = integrate a b base
      pp = preservedPoints a b base
      g // s = sliceBackward s g
      types = (a::Rel Int Int)

-----------------------------------------------------------------------------

