-----------------------------------------------------------------------------
-- |
-- Module      :  Data.Relation.TypeInference
-- Copyright   :  (c) Joost Visser 2004
-- License     :  LGPL
-- 
-- Maintainer  :  joost.visser@di.uminho.pt
-- Stability   :  experimental
-- Portability :  portable
--
-- An implementation of the type reconstuction algorithm from
--
--   * Arie van Deursen and Leon Moonen, 
--     /An Empirical Study into Cobol type inferencing./
--     In /Science of Computer Programming/, 40(2-3):189-211,
--     July 2001.
--
-----------------------------------------------------------------------------

module Data.Relation.TypeInference where

import Data.Relation.SetOfPairs

-----------------------------------------------------------------------------
-- * Representation


-- | A variable graph is a 3-tuple of three relations:
--
--     * expression: equivalence relation between variables based on occurance
--       in the same expression.
--
--     * array: relation between index variables and array variables.
--
--     * assign: relation between variables based on assignment of one to 
--       the other.
--
type VariableGraph v array
  = (Rel v v, Rel v array, Rel v v)

-- | A type graph is a tuple of two relations: a type equivalence relation,
--   and a subtype relation.
type TypeGraph x 
  = (Rel x x, Rel x x)
  
-----------------------------------------------------------------------------
-- * Inference of a type graph.

-- | The main function for type inference. It computes a type graph on the
--   basis of a variable graph. 
typeInference :: (Ord v, Ord array) =>
                 VariableGraph v array -> TypeGraph v
typeInference (expression,arrayIndex,assign)
  = let arrayIndexEquiv = (inv arrayIndex) `comp` arrayIndex
        typeEquiv = arrayIndexEquiv `union` expression
        subtypeOf = assign
        loop = typeInferenceLoop (typeEquiv,subtypeOf)
    in loop

-- | The loop function for type inference. Starting from an initial
--   type graph, it iteratively derives a more accurate type graph
--   until a fixpoint occurs.
typeInferenceLoop :: Ord x => TypeGraph x -> TypeGraph x
typeInferenceLoop
  = repeatUntilFixpoint
    (\ (typeEquiv,subtypeOf) ->
      let subtypeEquiv' 
             = equiv (tc `intersect` inv tc) where tc = transClose subtypeOf
          typeEquiv'    
	     = equiv (typeEquiv `union` subtypeEquiv')
	  subtypeOf'
	     = subtypeOf `minusSet` typeEquiv'
	  subtypeOf''
	     = subtypeOf'
	         `union` (subtypeOf' `comp` typeEquiv')
		 `union` (typeEquiv' `comp` subtypeOf')
      in (typeEquiv', subtypeOf'')
    )


-----------------------------------------------------------------------------
-- * Inference of types (groups of variables).

-- | Derive a list of equivalence classes from an equivalence relation.
--   If the relation is a type equivalence relation over variables, the
--   equivalence classes are groups of variables, i.e. types.
types :: Ord x => Rel x x -> [ Set x ]
types xs
  = types' [] (setToList xs)
    where
      types' ts []        = ts
      types' ts (xy:xys)  = types' (store xy ts) xys
      store (x,y) []     = [ mkSet [x,y] ]
      store xy@(x,y) (t:ts) 
       | x `elementOf` t = (t `addToSet` y :ts)
       | y `elementOf` t = (t `addToSet` x :ts)
       | otherwise       = (t:store xy ts)
       
       
-----------------------------------------------------------------------------
-- * Helper

-- | Keep applying the argument function until the output equals the
--   input, i.e. until a fixpoint is reached.
repeatUntilFixpoint :: Eq a => (a -> a) -> a -> a
repeatUntilFixpoint f xs
  = if fxs == xs 
       then xs
       else repeatUntilFixpoint f fxs
    where
      fxs = f xs

-----------------------------------------------------------------------------
