-----------------------------------------------------------------------------
-- |
-- Module      :  Language.ContextFree.CfgM
-- Copyright   :  (c) Joo Saraiva 2001,2002,2003,2004
-- License     :  LGPL
-- 
-- Maintainer  :  jas@di.uminho.pt
-- Stability   :  provisional
-- Portability :  portable
--
-- Regular Expressions in Haskell. Code Included in the Lecture Notes on 
-- Language Processing (with a functional flavour).   
--
-----------------------------------------------------------------------------

module Language.ContextFree.CfgM where

import Data.List
import Control.Monad.State

-----------------------------------------------------------------------------

infixl 2 |-> ; infixl 3 <*> ; infixl 4 <$>

-----------------------------------------------------------------------------
-- * Context-free grammars with actions.

-- | Datatype of context-free grammars with actions.
data Cfg m sy = Cfg { terminals :: [sy]
                    , nonterminals :: [sy]
                    , root :: sy
                    , prods :: [[(sy,m)]] 
                    }
   deriving (Show)

-- | Type synonym for productions with actions
type ProdM m sy
  = (sy,m)

-- | Create a production from LHS and RHS.
(|->) :: a -> [a] -> [a]
l |-> r = l : r

-- | Create a production with action from a production and an action.
(<$>) :: a -> b -> (a,b)
s <$> a = (s,a)

-- | Create a production with empty LHS action.
($->) :: Monad m => a -> [(a,m())] -> [(a,m())]
l $-> r = (l,return ()) : r

-- | Create a production with a state-modifying action
(<$$>) :: (MonadState s m) => t -> (s -> s) -> (t, m ())
s <$$> f = (s,modify f)

-- | Construct list of RHS symbols with actions. The first argument
--   is a symbol with action.
(<*>) :: a -> [a] -> [a]
h <*> t = h : t 

-- | Construct list of RHS symbols with actions. The first argument
--   is a symbol without action.
(<$*>) :: (Monad m) => a -> [(a, m ())] -> [(a, m ())]
l <$*> r = l $-> r 


-----------------------------------------------------------------------------
-- * Examples

ex = Cfg ['a','b','c']
         ['A','B','C','D']
         'A'
         [ ('A',f) |-> [ 'B' <$> do { putStrLn "Ola" ; return () } ]
         ]

f = do putStrLn "Ola"
       return ()


ex2 = Cfg ['a','b','c']
          ['A','B','C','D']
          'A'
          [ 'A' <$> f |-> 'B' <$> do { putStrLn "Ola" ; return () } <*> 
                          ('C' <$> f <*> [])
          ]

exExpa :: (MonadState Int m) => Cfg (m ()) [Char]
exExpa = Cfg ["int","var","+","*"]
             ["Exp"]
             "Exp"
             [ ("Exp",n) |-> [("Exp", n) , ("+",n) , ("Exp",n)]
             , ("Exp",n) |-> [("Exp",n) , ("*",n) , ("Exp",n)]
             , ("Exp",n) |-> [("int",n) ]
             , ("Exp",n) |-> [("var",n) ]
             ]
 where n = do modify (\ s -> s + 1)
              return ()



data Exp = AddExp Exp Exp
         | MulExp Exp Exp
         | Number Int
         | Var    String 

exExpb :: (MonadState [Exp] m) => Cfg (m ()) [Char]
exExpb = Cfg ["int","var","+","*"]
             ["Exp"]
             "Exp"
             [ ("Exp",n) |-> ["Exp" <$> n , "+" <$> n , "Exp" <$> a1]
             , ("Exp",n) |-> ["Exp" <$> n , "*" <$> n , "Exp" <$> a2]
             , ("Exp",n) |-> ["int" <$> a3 ]
             , ("Exp",n) |-> ["var" <$> a4 ]
             ]
 where n = return ()

       a1 = do modify (\ (h1:h2:[]) -> [AddExp h1 h2])
       a2 = do modify (\ (h1:h2:[]) -> [MulExp h1 h2])
       a3 = do modify (\ _ -> [Number 1])                    -- value of the token
       a4 = do modify (\ _ -> [Var "val"])


exExpc :: (MonadState [Exp] m) => Cfg (m ()) [Char]
exExpc = Cfg ["int","var","+","*"]
             ["Exp"]
             "Exp"
             [ "Exp" $-> ["Exp" <$> n , "+" <$> n , "Exp" 
                         <$$> (\ s -> [AddExp (_1 s) (_2 s)])
                         ]
             , "Exp" $-> ["Exp" <$> n , "*" <$> n , "Exp" 
                         <$$> (\ s -> [MulExp (_1 s) (_2 s)])
                         ]
             , "Exp" $-> ["int" 
                         <$$> (\ _ -> [Number 1]) ]
             , "Exp" $-> ["var" 
                         <$$> (\ _ -> [Var "val"]) ]
             ]
 where n = return ()

_1 l = l !! 0 
_2 l = l !! 1
_3 l = l !! 2

-----------------------------------------------------------------------------

