-----------------------------------------------------------------------------
-- |
-- Maintainer  :  joost.visser@di.uminho.pt
-- Stability   :  experimental
-- Portability :  portable
--
-- Derive data flow graph for spreadsheets.
--
-----------------------------------------------------------------------------

module Language.Gnumeric.Dataflow where

import Data.Relation.SetOfPairs
import Language.Gnumeric.Syntax
import Language.Gnumeric.SyntaxTermInstances
import Language.Gnumeric.Formula
import Language.Gnumeric.FormulaTermInstances

import StrategyLib
import Control.Monad.Identity
import Data.Monoid

-----------------------------------------------------------------------------

-- | Collect all cell references from a given term.
collectReferences :: Term a => a -> [CellRef]
collectReferences a
  = runIdentity $ applyTU (collect (constTU [] `adhocTU` worker)) a
    where
      worker :: CellRef -> Identity [CellRef]
      worker cellRef = return [cellRef]

-- | Collect all cell references from a given term.
collectBlockRefs :: Term a => a -> [CellBlock]
collectBlockRefs a
  = runIdentity $ applyTU (collect (constTU [] `adhocTU` worker)) a
    where
      worker :: Formula -> Identity [CellBlock]
      worker (Ref c1 c2) = return [(c1,c2)]
      worker _ = return []


-- | Create data flow edges for a given cell.
dataflowCell :: Gmr'Cell -> Rel CellRef CellBlock
dataflowCell (Gmr'Cell attrs content) 
  = mkRel [ (cellRef,y) | y <- cellBlocks ] 
    where
      cellBlocks = collectBlockRefs (gmr'CellAst attrs)
      cellRef 
        = CellRef { 
            colNr = read $ gmr'CellCol attrs, 
            rowNr = read $ gmr'CellRow attrs
          }

-- | Create data flow edges for a given term.
dataflow :: Term a => a -> Rel CellRef CellBlock
dataflow a
  = maybe mempty id $ 
      applyTU (crush (constTU emptyRel `adhocTU` worker)) a
    where
      worker :: Gmr'Cell -> Maybe (Rel CellRef CellBlock)
      worker cell = Just $ dataflowCell cell
      
dataflow' :: Term a => a -> Gph CellBlock
dataflow' a
  = flow' `union` nest
    where
      flow = dataflow a
      flow' = mapSet (\(c,b) -> ((c,Nothing),b)) flow
      nest = nesting $ setToList $ ent flow' 

nesting :: [CellBlock] -> Gph CellBlock
nesting bs
  = mkRel $ concatMap worker bs
    where
      worker b 
        = map (\x -> (x,b)) (filter (isInside b) bs)
      b1 `isInside` b2 
        = b1 `compareCellBlocks` b2 == Just LT
      
compareCellBlocks :: CellBlock -> CellBlock -> Maybe Ordering
compareCellBlocks (cell1, Nothing) cellBlock
  = compareCellBlocks (cell1, Just cell1) cellBlock
compareCellBlocks cellBlock (cell2, Nothing)
  = compareCellBlocks cellBlock (cell2, Just cell2)
compareCellBlocks (cell1,Just cell2) (cell3,Just cell4)
  = case (cell3 `compare` cell1, cell2 `compare` cell4) of
      (Nothing,_) -> Nothing
      (_,Nothing) -> Nothing
      (Just x, Just y) -> oi x y
    where
      compare = compareCellRefs

oi x y
  = case (x,y) of
      (GT,LT) -> Nothing
      (LT,GT) -> Nothing
      (EQ,EQ) -> Just EQ
      (LT,LT) -> Just LT
      (EQ,LT) -> Just LT
      (LT,EQ) -> Just LT
      (GT,GT) -> Just GT
      (EQ,GT) -> Just GT
      (GT,EQ) -> Just GT

compareCellRefs :: CellRef -> CellRef -> Maybe Ordering
compareCellRefs cell1 cell2
  = case (r1 `compare` r2, c1 `compare` c2) of
      (GT,LT) -> Nothing
      (LT,GT) -> Nothing
      (EQ,EQ) -> Just EQ
      (LT,LT) -> Just LT
      (EQ,LT) -> Just LT
      (LT,EQ) -> Just LT
      (GT,GT) -> Just GT
      (EQ,GT) -> Just GT
      (GT,EQ) -> Just GT
  where
    r1 = rowNr cell1
    c1 = colNr cell1
    r2 = rowNr cell2
    c2 = colNr cell2
    (r,c) = (r1 `compare` r2, c1 `compare` c2)


-----------------------------------------------------------------------------

instance Ord a => Monoid (Set a) where
  mempty = emptySet
  mappend = union
  
-----------------------------------------------------------------------------
