-----------------------------------------------------------------------------
-- |
-- Maintainer  :  joost.visser@di.uminho.pt
-- Stability   :  experimental
-- Portability :  portable
--
-- Syntax for spreadsheet formulas.
--
-----------------------------------------------------------------------------

module Language.Gnumeric.Formula where

import Text.ParserCombinators.Parsec
import qualified Text.ParserCombinators.Parsec.Token as P
import Text.ParserCombinators.Parsec.Language
import Text.ParserCombinators.Parsec.Expr

import Data.List (elemIndices)

-----------------------------------------------------------------------------
-- * Abstract syntax

data Formula 
  = Var String
  | Fun String [Formula]
  | Ref CellRef (Maybe CellRef)
{-
  | Number String
  | Constant String
  | Error String
-}
  deriving (Eq,Show,Read)

data CellRef 
  = CellRef {colNr :: Int, rowNr :: Int}
    deriving (Eq,Show,Read,Ord)

type CellBlock = (CellRef, Maybe CellRef)

showCellRef :: CellRef -> String
showCellRef cr              = (fromColNr $ colNr cr)++(fromRowNr $ rowNr cr)

showCellBlock :: CellBlock -> String
showCellBlock (cr,Nothing)  = showCellRef cr
showCellBlock (cr,Just cr') = (showCellRef cr)++":"++(showCellRef cr')

-----------------------------------------------------------------------------
-- * Parsers, based on Parsec.

-- | Top-level invocation of parser.
parseFormula :: String -> String -> Formula
parseFormula sourceName s
  = case (parse formula sourceName s) of
      (Left parseError)
        -> error $ show parseError
      (Right formula)
        -> formula

-- | Formulas.
formula :: Parser Formula
formula = do
  optional (symbol "=")
  expr

-- | Expressions.
expr :: Parser Formula
expr    =   buildExpressionParser table term
        <?> "expression"

-- | Table to build expression grammar.        
table :: OperatorTable Char () Formula
table   = [ [prefix "-", prefix "+"]
          , [postfix "++"]
          , [binary "*" AssocLeft, binary "/"  AssocLeft ]
          , [binary "+" AssocLeft, binary "-"  AssocLeft ]
          , [binary "=" AssocLeft, binary "<="  AssocLeft ]
          ]

-- | Helper.
binary :: String -> Assoc -> Operator Char () Formula        
binary name assoc 
  = Infix (do{ reservedOp name; return $ \x y -> Fun name [x,y] }) assoc

-- | Helper.
prefix :: String -> Operator Char () Formula        
prefix name       
  = Prefix (do{ reservedOp name; return $ \x -> Fun name [x] })

-- | Helper.
postfix :: String -> Operator Char () Formula        
postfix name       
  = Postfix (do{ reservedOp name; return $ \x -> Fun name [x] })

-- | Terms (leafs of expressions).
term :: Parser Formula
term    =   parens expr 
        <|> do { n <- stringLiteral; return $ Var n }
        <|> do { n <- natural; return $ Var (show n) }
        <|> try cellReference
        <|> functionApplication
        <?> "simple expression"

singleCellReference :: Parser CellRef
singleCellReference = do
  ls <- many1 upper
  ds  <- many1 digit
  return $ CellRef {colNr=toColNr ls, rowNr=toRowNr ds}

cellReference :: Parser Formula
cellReference = do
  c1 <- singleCellReference
  c2 <- optionMaybe $ do
    char ':'
    singleCellReference
  return $ Ref c1 c2

-- | Function application.
functionApplication :: Parser Formula
functionApplication = do
  i <- identifier
  fs <- option [] (parens (sepBy expr (symbol ",")))
  return $ Fun i fs

optionMaybe p
  = option Nothing (do{a <- p; return $ Just a})
-----------------------------------------------------------------------------
-- * Lexical issues.

lexer :: P.TokenParser ()
lexer  = P.makeTokenParser gnumericStyle

gnumericStyle
 = LanguageDef 
   { commentStart   = ""
   , commentEnd     = ""
   , commentLine    = ""
   , nestedComments = True
   , identStart     = letter <|> char '_' <|> char '#'
   , identLetter    = alphaNum <|> oneOf "_':!"
   , opStart        = opLetter emptyDef
   , opLetter       = oneOf ":!#$%&*+./<=>?@\\^|-~"
   , reservedOpNames= []
   , reservedNames  = []
   , caseSensitive  = True
   }

whiteSpace= P.whiteSpace lexer
lexeme    = P.lexeme lexer
symbol    = P.symbol lexer
natural   = P.natural lexer
parens    = P.parens lexer
semi      = P.semi lexer
identifier= P.identifier lexer
reserved  = P.reserved lexer
reservedOp= P.reservedOp lexer
stringLiteral = P.stringLiteral lexer

-----------------------------------------------------------------------------

-- | Infinite sequence of letter combinations that identify spreadsheet columns.
columns :: [String]  
columns 
  = [ xs++[x] | xs <- "":columns, x <- ['A'..'Z'] ]

-- | Convert a column name into a column number. For instance, "A" becomes
--   0 and "AA" becomes 26.
toColNr :: String -> Int
toColNr colName
  = head (elemIndices colName columns)

-- | Convert a row name into a row number. For instance, "1" becomes
--   0 and "10" becomes 9.
toRowNr :: String -> Int
toRowNr rowName
  = (read rowName)-1
  
-- | Convert a column number into a column name. Inverse of 'toColNr'.
fromColNr :: Int -> String
fromColNr colNr
  = columns!!colNr

-- | Convert a row number into a row name. Inverse of 'toRowNr'.
fromRowNr :: Int -> String
fromRowNr rowNr
  = show (rowNr+1)

-----------------------------------------------------------------------------
