-----------------------------------------------------------------------------
-- |
-- Module      :  Language.HaLex.Dfa
-- Copyright   :  (c) Joo Saraiva 2001,2002,2003,2004
-- License     :  LGPL
-- 
-- Maintainer  :  jas@di.uminho.pt
-- Stability   :  provisional
-- Portability :  portable
--
-- Non-Deterministic Finite Automata in Haskell. Code Included in the Lecture Notes on 
-- Language Processing (with a functional flavour).   
--
-----------------------------------------------------------------------------


module Language.HaLex.Ndfa ( 
            -- * Data type
              Ndfa (..) 
            , toHaskell
            , renameNdfa 
            , transitionTableNdfa
            , ndfaaccept
            , ndfawalk
            , epsilon_closure
            , sizeNdfa 
            , limit
            , ndfadeadstates
            , ndfaIsStDead
            ) where

import List
import Language.HaLex.Dfa

-----------------------------------------------------------------------------
-- * Data type with the acceptance function

-- | The type of Non-Deterministic Finite Automata. Paramterized with
--   the type @st@ of states and @sy@ of symbols.
data Ndfa st sy = Ndfa [sy]                     -- Vocabulary
                       [st]                     -- Finite set of states
                       [st]                     -- The set of start state
                       [st]                     -- The set of final states
                       (st -> Maybe sy -> [st]) -- Transition function


-- | Test whether the given automaton accepts the given list of
--   input symbols.
ndfaaccept :: Ord st 
           => Ndfa st sy    -- ^ Automaton
           -> [sy]          -- ^ Input symbols
           -> Bool
ndfaaccept (Ndfa _ _ s z delta) symbs = 
       (ndfawalk delta (epsilon_closure delta s) symbs) `intersect` z /= []

-- | Execute the transition function of a 'Ndfa' on an initial state
--   and list of input symbol. Return the final state when all input
--   symbols have been consumed.
ndfawalk :: Ord st 
         => (st -> Maybe sy -> [st])   -- ^ Transition function
         -> [st]                       -- ^ Initial states
         -> [sy]                       -- ^ Input symbols
         -> [st]                       -- ^ Reached states
ndfawalk delta sts []     = sts
ndfawalk delta sts (x:xs) = 
        ndfawalk delta (epsilon_closure delta (delta' delta sts (Just x))) xs

-- | Apply the transition function to a list of states. 
delta' :: Eq st 
       => (st -> (Maybe sy) -> [st])   -- ^ Transition function
       -> [st]                         -- ^ Current states
       -> (Maybe sy)                   -- ^ Symbol to consume
       -> [st]                         -- ^ Reached states
delta' delta []       sy = []
delta' delta (st:sts) sy = (delta st sy) `union` (delta' delta sts sy)

-- | COmpute the eplison closure of a 'Ndfa'.
epsilon_closure :: Ord st 
                => (st -> Maybe sy -> [st]) -- ^ Transition function
                -> [st]                     -- ^ Current states
                -> [st]                     -- ^ Reached states
epsilon_closure delta = limit f
  where f sts = sort (sts `union` (delta' delta sts Nothing))

limit :: Eq a => (a -> a) -> a -> a
limit f s | s == next = s
          | otherwise = limit f next
            where next = f s 

-- | The size of an automaton is the number of its states.
sizeNdfa :: Ndfa st sy -> Int
sizeNdfa (Ndfa _ q _ _ _) = length q


-----------------------------------------------------------------------------
-- * Printing


-- | Print a 'Ndfa' as a Haskell function.
instance (Eq st , Show st, Show sy) => Show (Ndfa st sy) where
      showsPrec p (Ndfa v q s z delta) =
                showString("ndfa = Ndfa v q s z delta") .
                showString ("\n  where \n\t v = ") .
                showList v . 
                showString ("\n\t q = ") .
                showList q . 
                showString ("\n\t s = ") .
                shows s . 
                showString ("\n\t z = ") .
                showList z . 
                showString ("\n\t -- delta :: st -> sy -> st \n") .
                (showNdfaDelta q v delta) .
                showString ("\t delta _ _ = []\n")

-- | Helper function to show the transition function of a 'Ndfa'.
showNdfaDelta q v d = foldr (.) (showChar '\n') f
  where f     = zipWith3 showF m n q'
        (m,n) = unzip l'
        q'    = map (uncurry d) l'
        l'    = filter ((/= []) . (uncurry d)) l
        l     = [(a, c) | a <- q  
                        , b <- v
                        , c <- [(Just b)]
                        ]  ++ [ (a,Nothing) | a <- q ]

        showF st sy st' = showString("\t delta ") .
                          shows st .
                          showString(" (") .
                          shows sy .
                          showString(") = ") .
                          shows st' .
                          showChar('\n')

-- | Produce the transition table of a given finite automaton. 
toHaskell :: Show fa 
          => fa        -- ^ Automaton
          -> [Char]    -- ^ Haskell module or file name
          -> IO ()
toHaskell fa modulename =  writeFile (modulename ++ ".hs") 
                                ("module " ++ modulename ++ " where\n\n" ++
                                "import Language.HaLex.Dfa\n\n" ++
                                "import Language.HaLex.Ndfa\n\n" ++ 
                                (show fa))


-----------------------------------------------------------------------------
-- * Transformation


-- | Produce the transition table of a given 'Ndfa'. 
--
-- Given a 'Ndfa' it returns a list of triples of the form
-- @
--                 (Origin,Symbol,Destination)
-- @
-- defining all the transitions of the 'Ndfa'.
--

transitionTableNdfa :: Ndfa st sy            -- ^ Automaton
                    -> [(st,Maybe sy,st)]    -- ^ Transition table
transitionTableNdfa (Ndfa vs qs s z delta) = [ (q,Just v,r) 
                                             | q <- qs , v <- vs 
                                             , r <- delta q (Just v)
                                             ] ++ 
                                             [ (q,Nothing,r)
                                             | q <- qs
                                             , r <- delta q Nothing
                                             ]


-- | Renames a 'Ndfa'. 
renameNdfa :: Eq st => Ndfa st sy -> Int -> Ndfa Int sy
renameNdfa (Ndfa v q s z d) istid = (Ndfa v q' s' z' d')
  where newSts   = zipWith (\ a b -> (a,b)) q [istid .. ]
        q'       = old2new newSts q
        s'       = old2new newSts s
        z'       = old2new newSts z
        d' st sy = old2new newSts (d (lookupSnd newSts st) sy) 


old2new :: Eq st => [(st,Int)] -> [st] -> [Int]
old2new nsts sts = map (lookupFst nsts) sts

lookupFst :: Eq st => [(st,Int)] -> st -> Int
lookupFst nsts ost = snd $ head (filter (\ (a,b) -> a == ost) nsts)


lookupSnd :: [(st,Int)] -> Int -> st
lookupSnd nsts nst = fst $ head (filter (\ (a,b) -> b == nst) nsts)



--
-- Dead States
--

-- | Compute the dead states of a 'Ndfa' (to be re-written)
ndfadeadstates (Ndfa v qs s z d) = deadstates qs v z d

deadstates [] _ _ _                            = []
deadstates (q:qs) v z d | ndfaIsStDead q v z d = q : (deadstates qs v z d)
                        | otherwise           = deadstates qs v z d


-- isStDead = isThereAPath q v z d

ndfaIsStDead :: Ord st => st -> [sy] -> [st] -> (st -> Maybe sy -> [st]) -> Bool
ndfaIsStDead st v z d = isSyncState st v z d

--
--
isSyncState :: Ord st => st -> [sy] -> [st] -> (st -> Maybe sy -> [st]) -> Bool
isSyncState st vs z d = (not (st `elem` z)) && (and qs)
  where qs = [ [st] == (d st (Just v)) 
               && (([st] == d st Nothing) || ([] == d st Nothing))
             | v <- vs 
             ]




