-----------------------------------------------------------------------------
-- |
-- Module      :  Language.HaLex.RegExp
-- Copyright   :  (c) Joo Saraiva 2001,2002,2003,2004
-- License     :  LGPL
-- 
-- Maintainer  :  jas@di.uminho.pt
-- Stability   :  provisional
-- Portability :  portable
--
-- Regular Expressions in Haskell. Code Included in the Lecture Notes on 
-- Language Processing (with a functional flavour).   
--
-----------------------------------------------------------------------------

module Language.HaLex.RegExp ( 
              -- * Data type with recursion pattern
                RegExp (..)
              , cataRegExp
              -- * Matching
              , matchesRE
              , matches'
              -- * Printing
              , showRE
              -- * Normalization
              , extREtoRE
              ) where

-----------------------------------------------------------------------------
-- * Data type with recursion pattern

-- | Type of regular expressions.
data RegExp = Epsilon
            | Literal   Char             -- ^ Literals
            | Or        RegExp RegExp    -- ^ Disjuncion
            | Then      RegExp RegExp    -- ^ Sequence
            | Star      RegExp           -- ^ Repetition, possibly zero time
            | OneOrMore RegExp           -- ^ One or more times (extended RegExp)
            | Optional  RegExp           -- ^ Optional (extended RegExp)
   deriving (Read, Show , Eq)
   
-- | Catamorphism induced by the RegExp inductive data type
cataRegExp :: ( re
              , re -> re -> re
              , re -> re
              , Char -> re
              , re -> re -> re
              , re -> re
              , re -> re
              ) -> RegExp -> re
cataRegExp (epsilon,or,star,lit,th,one,opt) = cata
 where cata Epsilon        = epsilon
       cata (Or er1 er2)   = or (cata er1) (cata er2)
       cata (Star er)      = star (cata er)
       cata (Literal a)    = lit a
       cata (Then er1 er2) = th (cata er1) (cata er2)
       cata (OneOrMore er) = one (cata er)
       cata (Optional er)  = opt (cata er)

-----------------------------------------------------------------------------
-- * Matching

-- | Test whether a match can be found for the given regular expression
--   in the given sequence of characters. The regular expression is
--   assumed not to contain 'OneOrMore' or 'Optional'. See also @matches'@.
matchesRE :: RegExp -> [Char] -> Bool
matchesRE Epsilon inp        = inp == ""
matchesRE (Literal l) inp    = ([l] == inp)
matchesRE (Or re1 re2) inp   = matchesRE re1 inp || matchesRE re2 inp 
matchesRE (Then re1 re2) inp = or [ matchesRE re1 s1 && matchesRE re2 s2 
                                  | (s1,s2) <- splits inp]
matchesRE (Star re) inp      = matchesRE Epsilon inp ||
                               or [ matchesRE re s1 && matchesRE (Star re) s2 
                                  | (s1,s2) <- frontSplits inp ]

-- | Test whether a match can be found for the given regular expression
--   in the given sequence of characters. The regular expression is
--   allowed to contain 'OneOrMore' or 'Optional'.
matches' :: RegExp -> [Char] -> Bool
matches' = matchesRE . extREtoRE

-- | Produce a list of all possible ways of splitting the input list
--   into two parts. For instance, 
-- @ 
--   splits "foo" 
--     = [(\"\","foo"),("f","oo"),("fo","o"),("foo",\"\")] 
-- @
splits :: [a] -> [ ([a],[a]) ]
splits s = [ splitAt n s | n <- [ 0 .. length s ] ]

-- | Produce a list of all possible ways of splitting the input list
--   into two parts where the first part is non-empy. For instance, 
-- @ 
--   splits "foo" 
--     = [("f","oo"),("fo","o"),("foo",\"\")]
-- @
frontSplits :: [a] -> [ ([a],[a]) ]
frontSplits s = [ splitAt n s | n <- [ 1 .. length s ] ]

-----------------------------------------------------------------------------
-- * Printing

-- | Print regular expression to String.
showRE :: RegExp -> [Char]
showRE = cataRegExp ("@"
                     , \ l r -> "(" ++ l ++ "|" ++ r ++ ")"
                     , \ er  -> "(" ++ er ++ ")*"
                     , \ a   -> [a]
                     , \ l r -> "(" ++ l ++ r ++ ")"
                     , \ er  -> "(" ++ er ++ ")+"
                     , \ er  -> "(" ++ er ++ ")?"
                     )

-----------------------------------------------------------------------------
-- * Normalization

-- | Rewrite extended regular expressions to
--   plain regular expression. This means that the 'OneOrMore' 
--   and 'Optional' constructors are normalized away.
extREtoRE :: RegExp -> RegExp
extREtoRE  = cataRegExp ( Epsilon 
                        , \ l r -> Or l r
                        , \ er  -> Star er
                        , \ a   -> Literal a 
                        , \ l r -> Then l r 
                        , \ er  -> Then er (Star er)
                        , \ er  -> Or Epsilon er
                        ) 

-----------------------------------------------------------------------------


