-----------------------------------------------------------------------------
-- |
-- Copyright   :  (c) Joost Visser 2004
-- License     :  LGPL
-- 
-- Maintainer  :  joost.visser@di.uminho.pt
-- Stability   :  experimental
-- Portability :  portable
--
-- Unit tests for "Language.Haskell.DeadCode".
--
-----------------------------------------------------------------------------

module Language.Haskell.DeadCodeHUnit where

import HUnit
import Language.Haskell.ModuleCollection
import Language.Haskell.SyntaxAccessors
import Language.Haskell.Syntax
import Language.Haskell.Parser
import Language.Haskell.DeadCode

-----------------------------------------------------------------------------

parseWrap :: String -> HsModule
parseWrap x = forceResult (parseModule x)

forceResult :: ParseResult a -> a
forceResult (ParseOk x)         = x
forceResult (ParseFailed _ msg) = error msg

parseModules :: [String] -> ModuleCollection
parseModules ms
  = map (worker . parseWrap) ms
    where
      worker m@(HsModule _ (Module n) is _ _)
        = (n,getImports m,m)
        
testModules ms
  = hsExtrAppl (parseModules ms)

main
  = runTestTT . TestList $
     [labeledTest "empty module" (do 
        result <- testModules ["module Main where"]
        let expected = parseModules ["module Main where"]
        assertEqual "No change" expected result
      ),
      labeledTest "trivial module" (do 
        result <- testModules ["module Main where main = undefined"]
        let expected = parseModules ["module Main where main = undefined"]
        assertEqual "No change" expected result
      ),
      labeledTest "two modules" (do 
        result <- testModules 
                        ["module Main where \nimport X \nmain = x",
                         "module X where x = undefined"]
        let expected = parseModules 
                        ["module Main where \nimport X \nmain = x",
                         "module X where x = undefined"]
        assertEqual "No change" expected result
      ),
      labeledTest "dead bird (1)" (do 
        result <- testModules 
                        ["module Main where \nimport X \nmain = x",
                         "module X where \nx = undefined \nbird=undefined"]
        let expected = parseModules 
                        ["module Main where \nimport X \nmain = x",
                         "module X where \nx = undefined"]
        assertEqual "Bird removed" expected result
      ),
      labeledTest "dead bird (2)" (do 
        result <- testModules 
                        ["module Main where \nimport X \nmain = x \nbird=undefined",
                         "module X where \nx = undefined"]
        let expected = parseModules 
                        ["module Main where \nimport X \nmain = x \nbird=undefined",
                         "module X where \nx = undefined"]
        assertEqual "Bird in main NOT removed" expected result
      ),
      labeledTest "dead bird (3)" (do 
        result <- testModules 
                        ["module Main where \nimport X \nmain = x",
                         "module X where \nx = y \ny=undefined \nbird=bird2 \nbird2=undefined"]
        let expected = parseModules 
                        ["module Main where \nimport X \nmain = x",
                         "module X where \nx = y \ny=undefined"]
        assertEqual "Birds removed" expected result
      ),
      labeledTest "dead chain" (do 
        result <- testModules 
                        ["module Main where import X",
                         "module X where \na=b \nb=c \nc=d \nd=e"]
        let expected = parseModules 
                        ["module Main where import X",
                         "module X where"]
        assertEqual "Chain removed" expected result
      ),
      labeledTest "live chain" (do 
        result <- testModules 
                        ["module Main where \nimport X \nmain=a",
                         "module X where \na=b \nb=c \nc=d \nd=e"]
        let expected = parseModules 
                        ["module Main where \nimport X \nmain=a",
                         "module X where \na=b \nb=c \nc=d \nd=e"]
        assertEqual "Chain NOT removed" expected result
      ),
      labeledTest "live chain" (do 
        result <- testModules 
                        ["module Main where \nimport X \nmain=a",
                         "module X where \na=b \nb=c \nc=d \nd=e"]
        let expected = parseModules 
                        ["module Main where \nimport X \nmain=a",
                         "module X where \na=b \nb=c \nc=d \nd=e"]
        assertEqual "Chain NOT removed" expected result
      ),
      labeledTest "dead circle" (do 
        result <- testModules 
                        ["module Main where import X",
                         "module X where \na=b \nb=c \nc=d \nd=a"]
        let expected = parseModules 
                        ["module Main where import X",
                         "module X where \na=b \nb=c \nc=d \nd=a"]
        assertEqual "Circle NOT removed" expected result
      ),
      labeledTest "dead pair" (do
        result <- testModules 
                        ["module Main where import X",
                         "module X where \n(a,b)=c"]
        let expected = parseModules 
                        ["module Main where import X",
                         "module X where"]
        assertEqual "Dead pair removed" expected result
      ),
      labeledTest "with type signature" (do
        result <- testModules 
                        ["module Main where \nimport X \nmain=a",
                         "module X where \na::Int \na=1 \nb::Int \nb=2"]
        let expected = parseModules 
                        ["module Main where \nimport X \nmain=a",
                         "module X where \na::Int \na=1"]
        assertEqual "Dead type signature removed" expected result
      ),
      labeledTest "keep type definitions" (do
        result <- testModules 
                        ["module Main where \nimport X \nmain=a",
                         "module X where \ntype T = Int \ndata D = D Int"]
        let expected = parseModules 
                        ["module Main where \nimport X \nmain=a",
                         "module X where \ntype T = Int \ndata D = D Int"]
        assertEqual "No type definitions removed" expected result
      ),
      labeledTest "keep dead recursive function" (do
        result <- testModules 
                        ["module Main where import X",
                         "module X where \nf a = f (f a)"]
        let expected = parseModules 
                        ["module Main where import X",
                         "module X where \nf a = f (f a)"]
        assertEqual "Dead function NOT removed (unfortunately)" expected result
      )
     ]  

labeledTest l t
  = TestLabel l (TestCase t)
  
-----------------------------------------------------------------------------

