-----------------------------------------------------------------------------
-- |
-- Copyright   :  (c) Joost Visser 2004
-- License     :  LGPL
-- 
-- Maintainer  :  joost.visser@di.uminho.pt
-- Stability   :  experimental
-- Portability :  portable
--
-- Unit tests for "Language.Haskell.FreeNames".
--
-----------------------------------------------------------------------------
module Language.Haskell.FreeNamesHUnit where

import HUnit
import Language.Haskell.FreeNames
import Language.Haskell.Syntax
import Language.Haskell.Parser

parseWrap :: String -> HsModule
parseWrap x = forceResult (parseModule x)

forceResult :: ParseResult a -> a
forceResult (ParseOk x)         = x
forceResult (ParseFailed _ msg) = error msg

testModule m
  = hsFreeAndDeclared (parseWrap m)

testDecls ds
  = let (HsModule _ _ _ _ ds') = parseWrap ("module Main where\n" ++ ds)
    in hsFreeAndDeclared ds'  
  
testExpr b
  = let (HsModule _ _ _ _ ds) = parseWrap ("module Main where dummy = " ++ b)
        [HsPatBind _ _ (HsUnGuardedRhs e) []] = ds
    in --do putStrLn $ "Expr: " ++ show e
          hsFreeAndDeclared e  

labeledTest l t
  = TestLabel l (TestCase t)

main
  = runTestTT . TestList $
     [labeledTest "empty module" (
        do (fs,ds) <- testModule "module Main where"
	   assertEqual "Nothing free" [] fs
	   assertEqual "Nothing declared" [] ds
      ),
      labeledTest "trivial module" (
        do (fs,ds) <- testModule "module Main where main = undefined"
	   assertEqual "1 free" [UnQual . HsIdent $ "undefined"] fs
	   assertEqual "1 declared" [UnQual . HsIdent $ "main"] ds
      ),
      labeledTest "function application" (
        do (fs,ds) <- testExpr "f x"
	   assertEqual "2 free" 2 (length fs)
	   assertBool "f free" (elem (UnQual . HsIdent $ "f") fs)
	   assertBool "x free" (elem (UnQual . HsIdent $ "x") fs)
	   assertEqual "1 declared" [] ds
      ),
      labeledTest "lamda expression" (
        do (fs,ds) <- testExpr "\\x y -> f x"
	   assertEqual "1 free" 1 (length fs)
	   assertBool "f free" (elem (UnQual . HsIdent $ "f") fs)
	   assertEqual "Nothing declared" [] ds
      ),
      labeledTest "let expression" (
        do (fs,ds) <- testExpr "let g x y = f x in g X Y"
	   assertEqual "3 free" 3 (length fs)
	   assertBool "f free" (elem (UnQual . HsIdent $ "f") fs)
	   assertBool "X free" (elem (UnQual . HsIdent $ "X") fs)
	   assertBool "Y free" (elem (UnQual . HsIdent $ "Y") fs)
	   assertEqual "Nothing declared" [] ds
      ),
      labeledTest "mutually dependent function definitions" (
        do (fs,ds) <- testDecls $
	                "g x y = f x     \n" ++
			        "f z   = g X Y"
	   assertEqual "2 free" 2 (length fs)
	   assertBool "X free" (elem (UnQual . HsIdent $ "X") fs)
	   assertBool "Y free" (elem (UnQual . HsIdent $ "Y") fs)
	   assertEqual "2 declared" 2 (length ds)
	   assertBool "f declared" (elem (UnQual . HsIdent $ "f") ds)
	   assertBool "g declared" (elem (UnQual . HsIdent $ "g") ds)
      ),
      labeledTest "function definition with where clause" (
        do (fs,ds) <- testDecls "f z = g X Y where g x y = f x"
	   assertEqual "2 free" 2 (length fs)
	   assertBool "X free" (elem (UnQual . HsIdent $ "X") fs)
	   assertBool "Y free" (elem (UnQual . HsIdent $ "Y") fs)
	   assertEqual "1 declared" 1 (length ds)
	   assertBool "f declared" (elem (UnQual . HsIdent $ "f") ds)
      ),
      labeledTest "case expression" (
        do (fs,ds) <- testExpr "case a of {(C x y) -> f z where z = x y}"
	   assertEqual "3 free" 3 (length fs)
	   assertBool "C free" (elem (UnQual . HsIdent $ "C") fs)
	   assertBool "a free" (elem (UnQual . HsIdent $ "a") fs)
	   assertBool "f free" (elem (UnQual . HsIdent $ "f") fs)
	   assertEqual "Nothing declared" [] ds
      )
     ]

