-----------------------------------------------------------------------------
-- |
-- Maintainer  :  nfr@di.uminho.pt
-- Stability   :  experimental
-- Portability :  portable
--
-- Haskell Modules Graph Constructor
--
-----------------------------------------------------------------------------

module Language.Haskell.FunctionGraph where

import Language.Haskell.Parser
import Language.Haskell.Syntax
import Data.Relation.SetOfPairs
import Data.Set

-----------------------------------------------------------------------------
--  * Haskell Parsing 

-- | Create function graph for Haskell module in given file.
mkHaFunGraph :: FilePath -> IO (Rel String String)
mkHaFunGraph f = do 
		 texto <- readFile f
		 let parseRes = parseModule texto
	         case parseRes of
		   (ParseOk hsmodule)  -> putStrLn "Graph Build ok" >> 
		                          --writeFile "periscope.txt" (show (map ((showString "--NEWLINE--") . show ) (getDecls hsmodule))) >>
		                          --return emptyRel
		                          putStrLn (show (buildMinFunGraph (getFunBinds hsmodule))) >>
		                          return (buildFunGraph (getFunBinds hsmodule))
		   (ParseFailed src s) -> putStrLn ("Parsing Error in " ++ 
		                                    srcFilename src ++ 
		                                    " at line " ++ 
		                                    show (srcLine src) ++ 
		                                    " at column " ++ 
		                                    show (srcColumn src) ++ ")") >> 
		                          return emptySet

-----------------------------------------------------------------------------
--  * Graph Construction 

-- | Build function graph for a list of modules.
buildFunctionGraph :: [HsModule] -> (Rel String String)
buildFunctionGraph ms
  = foldr (\m r -> r `union` (buildFunctionGraph1 m)) emptyRel ms

-- | Build function graph for a single module.
buildFunctionGraph1 :: HsModule -> Rel String String
buildFunctionGraph1 m
  = buildFunGraph (getFunBinds m)

-- | Build minimized function graph for a list of modules.
buildMinimizedFunctionGraph :: [HsModule] -> (Rel String String)
buildMinimizedFunctionGraph ms
  = foldr (\m r -> r `union` (buildMinimizedFunctionGraph1 m)) emptyRel ms
  
-- | Build minimized function graph for a single module.
buildMinimizedFunctionGraph1 :: HsModule -> Rel String String
buildMinimizedFunctionGraph1 m
  = buildMinFunGraph (getFunBinds m)

buildMinFunGraph :: [HsMatch] -> (Rel String String)
buildMinFunGraph hsmatches = let l = getTopFuns hsmatches
                                 g = buildFunGraph hsmatches
                             in projectWith (\x -> (\y -> elem y l)) g

getTopFuns :: [HsMatch] -> [String]
getTopFuns [] = []
getTopFuns ((HsMatch _ (HsIdent fun1) _ _ _):xs) = fun1 : getTopFuns xs

buildFunGraph :: [HsMatch] -> (Rel String String)
buildFunGraph [] = emptyRel
buildFunGraph ((HsMatch _ (HsIdent fun1) _ (HsUnGuardedRhs hsexp) _):xs) = 
    union (mkRelNeighbors fun1 (getHsExpFuns hsexp)) (buildFunGraph xs)
buildFunGraph (h:t) = buildFunGraph t

-- | Find function Identifiers inside Haskell Expressions
getHsExpFuns :: HsExp -> [String]
getHsExpFuns (HsVar hsqname)                      = getHsQNameFuns hsqname
getHsExpFuns (HsCon hsqname)                      = getHsQNameFuns hsqname
getHsExpFuns (HsInfixApp exp1 hsqop exp2)         = getHsExpFuns exp1 ++ getHsExpFuns exp2 ++ getHsQOpFuns hsqop
getHsExpFuns (HsApp exp1 exp2)                    = getHsExpFuns exp1 ++ getHsExpFuns exp2
getHsExpFuns (HsNegApp exp)                       = getHsExpFuns exp
getHsExpFuns (HsLambda _ hspats exp)              = getHsExpFuns exp ++ foldr (++) [] (map getHsPatFuns hspats)
getHsExpFuns (HsLet decls exp)                    = getHsExpFuns exp ++ getHsDeclsFuns decls
getHsExpFuns (HsIf exp1 exp2 exp3)                = getHsExpFuns exp1 ++ getHsExpFuns exp2 ++ getHsExpFuns exp3
getHsExpFuns (HsCase exp hsalts)                  = getHsExpFuns exp ++ foldr (++) [] (map getHsAltFuns hsalts)
getHsExpFuns (HsDo hsstmts)                       = foldr (++) [] (map getHsStmtFuns hsstmts) 
getHsExpFuns (HsTuple exps)                       = foldr (++) [] (map getHsExpFuns exps)
getHsExpFuns (HsList exps)                        = foldr (++) [] (map getHsExpFuns exps)
getHsExpFuns (HsParen exp)                        = getHsExpFuns exp
getHsExpFuns (HsLeftSection exp hsqop)            = getHsExpFuns exp ++ getHsQOpFuns hsqop
getHsExpFuns (HsRightSection hsqop exp)           = getHsExpFuns exp ++ getHsQOpFuns hsqop
getHsExpFuns (HsRecConstr hsqname hsfieldupdates) = getHsQNameFuns hsqname ++ foldr (++) [] (map getHsFieldUpdateFuns hsfieldupdates)
getHsExpFuns (HsRecUpdate exp hsfieldupdates)     = getHsExpFuns exp ++ foldr (++) [] (map getHsFieldUpdateFuns hsfieldupdates)
getHsExpFuns (HsEnumFrom exp)                     = getHsExpFuns exp  
getHsExpFuns (HsEnumFromTo exp1 exp2)             = getHsExpFuns exp1 ++ getHsExpFuns exp2
getHsExpFuns (HsEnumFromThen exp1 exp2)           = getHsExpFuns exp1 ++ getHsExpFuns exp2
getHsExpFuns (HsEnumFromThenTo exp1 exp2 exp3)    = getHsExpFuns exp1 ++ getHsExpFuns exp2 ++ getHsExpFuns exp3
getHsExpFuns (HsListComp exp hsstmts)             = getHsExpFuns exp ++ foldr (++) [] (map getHsStmtFuns hsstmts) 
getHsExpFuns (HsExpTypeSig _ exp _HsQualType)     = getHsExpFuns exp
getHsExpFuns _                                    = []

getHsFieldUpdateFuns :: HsFieldUpdate -> [String]
getHsFieldUpdateFuns (HsFieldUpdate hsqname hsexp) = getHsQNameFuns hsqname ++ getHsExpFuns hsexp

getHsQNameFuns :: HsQName -> [String]
getHsQNameFuns (UnQual (HsIdent fun)) = [fun]
getHsQNameFuns _                      = []

getHsQOpFuns :: HsQOp -> [String]
getHsQOpFuns (HsQVarOp hsqname) = getHsQNameFuns hsqname
getHsQOpFuns (HsQConOp hsqname) = getHsQNameFuns hsqname

getHsPatFuns :: HsPat -> [String]
getHsPatFuns (HsPVar hsname)                     = getHsNameFun hsname
getHsPatFuns (HsPNeg hspat)                      = getHsPatFuns hspat
getHsPatFuns (HsPInfixApp hspat1 hsqname hspat2) = getHsPatFuns hspat1 ++ getHsQNameFuns hsqname ++ getHsPatFuns hspat2
getHsPatFuns (HsPApp hsqname hspats)             = getHsQNameFuns hsqname ++ foldr (++) [] (map getHsPatFuns hspats)
getHsPatFuns (HsPTuple hspats)                   = foldr (++) [] (map getHsPatFuns hspats)
getHsPatFuns (HsPList hspats)                    = foldr (++) [] (map getHsPatFuns hspats)
getHsPatFuns (HsPParen hspat)                    = getHsPatFuns hspat
getHsPatFuns (HsPRec hsqname hspatfields)        = getHsQNameFuns hsqname ++ foldr (++) [] (map getHsPFieldPatFuns hspatfields)
getHsPatFuns (HsPAsPat hsname hspat)             = getHsNameFun hsname ++ getHsPatFuns hspat
getHsPatFuns (HsPIrrPat hspat)                   = getHsPatFuns hspat
getHsPatFuns _ = []

getHsDeclsFuns :: [HsDecl] -> [String]
getHsDeclsFuns hsdecls = foldr (++) [] (map getHsExpFuns (getExpsFromHsDecls hsdecls))

getHsNameFun :: HsName -> [String] 
getHsNameFun (HsIdent fun)  = [fun]
getHsNameFun (HsSymbol fun) = [fun]

getHsPFieldPatFuns :: HsPatField -> [String]
getHsPFieldPatFuns (HsPFieldPat hsqname hspat) = getHsQNameFuns hsqname ++ getHsPatFuns hspat

getHsAltFuns :: HsAlt -> [String]
getHsAltFuns (HsAlt _ hspat hsguardedalts hsdecls) = getHsPatFuns hspat ++ getHsGuardedAltsFuns hsguardedalts ++ getHsDeclsFuns hsdecls

getHsGuardedAltsFuns :: HsGuardedAlts -> [String]
getHsGuardedAltsFuns (HsUnGuardedAlt hsexp)        = getHsExpFuns hsexp
getHsGuardedAltsFuns (HsGuardedAlts hsguardedalts) = foldr (++) [] (map getHsGuardedAltFuns hsguardedalts)

getHsGuardedAltFuns :: HsGuardedAlt -> [String]
getHsGuardedAltFuns (HsGuardedAlt _ hsexp1 hsexp2) = getHsExpFuns hsexp1 ++ getHsExpFuns hsexp2

getHsStmtFuns :: HsStmt -> [String]
getHsStmtFuns (HsGenerator _ hspat hsexp) = getHsPatFuns hspat ++ getHsExpFuns hsexp
getHsStmtFuns (HsQualifier hsexp)         = getHsExpFuns hsexp
getHsStmtFuns (HsLetStmt hsdecls)         = getHsDeclsFuns hsdecls 


-- Auxiliar functions to treat Haskell.Syntax

getDecls :: HsModule -> [HsDecl]
getDecls (HsModule _ _ _ _ decls ) = decls

filterFunBinds :: [HsDecl] -> [HsMatch]
filterFunBinds []                        = []
filterFunBinds ((HsFunBind hsmatches):t) = hsmatches ++ filterFunBinds t
filterFunBinds (h:t)                     = filterFunBinds t

getFunBinds :: HsModule -> [HsMatch]
getFunBinds = filterFunBinds . getDecls

getExpsFromHsMatches :: [HsMatch] -> [HsExp]
getExpsFromHsMatches [] = []
getExpsFromHsMatches ((HsMatch _ _ _ (HsUnGuardedRhs hsexp) _):t) = hsexp : getExpsFromHsMatches t
getExpsFromHsMatches _ = []

getExpsFromHsDecls :: [HsDecl] -> [HsExp]
getExpsFromHsDecls = getExpsFromHsMatches . filterFunBinds
                   
-----------------------------------------------------------------------------
