-----------------------------------------------------------------------------
-- |
-- Copyright   :  (c) Universidade do Minho, 2004
-- License     :  LGPL
-- 
-- Maintainer  :  joost.visser@di.uminho.pt
-- Stability   :  experimental
-- Portability :  portable
--
-- Extraction of conditional call graphs from Java programs.
--
-----------------------------------------------------------------------------

module Language.Java.CallGraph where

------------------------------------------------------------------------------

import Language.Java.Syntax
import Language.Java.SyntaxTermInstances
import StrategyLib
import Language.Java.Util
import MonadState
import MonadTrans
import MonadRun

import Data.Relation.MapFromPairs
import Data.Relation.GraphViz
import Monoid
import Data.Set

-------------------------------------------------------------------------------
-- * Representation

-- | The type of conditional call graphs.
type CallGraph
  = LRel CGNode CGNode CGEdgeType
  
-- | The type of call graph nodes.
type CGNode = String

-- | The type of edges types in call graphs.
type CGEdgeType = String

-- | Empty call graph.  
emptyCG :: CallGraph
emptyCG = emptyFM

-- | Append two call graphs to form a new one.
appendCG :: CallGraph -> CallGraph -> CallGraph
appendCG cg1 cg2 = plusFM cg1 cg1

-- | Call graphs are monoids.
instance Monoid CallGraph where
  mempty  = emptyCG
  mappend = appendCG

-- | Add an edge of given type to a call graph.
addEdgeToCG :: CGNode -> CGNode -> CGEdgeType
            -> CallGraph -> CallGraph
addEdgeToCG "" _ _ gr   = gr
addEdgeToCG x y tp gr   = addToFM gr (x,y) tp

-- | Add a call edge to a call graph.
addCallEdge         :: CGNode -> CGNode -> CallGraph -> CallGraph
addCallEdge s t cg  = addEdgeToCG s t "call" cg

-- | Add a nesting edge to a call graph.
addNestEdge         :: CGNode -> CGNode -> CallGraph -> CallGraph
addNestEdge s t cg  = addEdgeToCG s t "nest" cg


------------------------------------------------------------------------------
-- * CCG generation 

{-
-- | Create a call graph in graphviz format.
javaCallGraph :: Monad m => [CompilationUnit] -> m String
javaCallGraph cu
  = do edges <- localNameSupply 0 (java2ccg "" cu)
       return (mkGraph "ccg" edges)
-}

-- | Create a call graph.
-- Use a name supply to generate unique node names
-- Basically follows the Propagage design pattern, where the
--   name of the parent node is the environment.
java2ccg    :: (Term a, Monad m) 
            => String       -- ^ Name of the current parent node. 
            -> a 
            -> StateT Integer m CallGraph
java2ccg parent x
  = applyTU worker x
    where
      worker = allTU' worker
                 `adhocTU` class2ccg
        	         `adhocTU` method2ccg
                 `adhocTU` mcall2ccg
                 --`adhocTU` for2ccg1
                 --`adhocTU` for2ccg2
                 --`adhocTU` if2ccg1
                 --`adhocTU` if2ccg2
                 --`adhocTU` if2ccg3
      class2ccg (Class1 _ ident _ _ body) 
             = do cg <- java2ccg ident body
                  return (addNestEdge parent ident cg)
      method2ccg (MethodHeader_MethodBody header body) 
             = do ident <- getMethodName header
                  cg <- java2ccg ident body
                  return (addNestEdge parent ident cg)
      mcall2ccg (PrimaryMethodInvocation prim ident args)
             = do cg <- java2ccg parent (prim,args)
                  return (addCallEdge parent ident cg)
      mcall2ccg (SuperMethodInvocation ident args)
             = do cg <- java2ccg parent args
                  return (addCallEdge parent ident cg)
      mcall2ccg (NameMethodInvocation name args)
             = do ident <- return (name2str name)
                  cg <- java2ccg parent args
                  return (addCallEdge parent ident cg)
{-
      for2ccg1 (For_semicolon_semicolon init expr upd stat)
             = do ident  <- supplyName
                  edges  <- java2ccg parent (init,expr,upd)
                  edges' <- java2ccg ident stat
                  return ((mkIterEdge parent ident)++edges++edges')
      for2ccg2 (For_semicolon_semicolon1 init expr upd stat)
             = do ident  <- supplyName
                  edges  <- java2ccg parent (init,expr,upd)
                  edges' <- java2ccg ident stat
                  return ((mkIterEdge parent ident)++edges++edges') 
      if2ccg1 (If expr stat_t)
             = do ident <- supplyName
                  edges <- java2ccg parent expr
                  es_t  <- java2ccg (ident++":true") stat_t
                  return ((mkCondEdge ["true"] parent ident)
                          ++edges++es_t)
      if2ccg2 (If_else expr stat_t stat_e)
             = do ident <- supplyName
                  edges <- java2ccg parent expr
                  es_t  <- java2ccg (ident++":true") stat_t
                  es_e  <- java2ccg (ident++":false") stat_e
                  return ((mkCondEdge ["true","false"] parent ident)
                          ++edges++es_t++es_e)
      if2ccg3 (If_else1 expr stat_t stat_e)
             = do ident <- supplyName
                  edges <- java2ccg parent expr
                  es_t  <- java2ccg (ident++":true") stat_t
                  es_e  <- java2ccg (ident++":false") stat_e
                  return ((mkCondEdge ["true","false"] parent ident)
                          ++edges++es_t++es_e)
-}

--- Helpers for generating new names -----------------------------------------

supplyName :: MonadState Integer m => m String
supplyName 
  = do i <- get
       put (i+1)
       return ("__"++(show i))

localNameSupply :: Monad m => Integer -> StateT Integer m a -> m a
localNameSupply s
  = unlift (StateAlg s fst)
        
--- Extraction of identifiers from Java fragments ----------------------------

-- Extract base identifier from full name
name2str :: Name -> Identifier                  
name2str (Identifier_p idents) = last idents
name2str (Class idents) = last idents



    
--- Helpers for dot generation -----------------------------------------------

cg2dot :: String -> CallGraph -> String
cg2dot n cg
  = lrel2dot n printEdge cg
  

lrel2dot    :: String 
            -> (((NodeName,NodeName), et) -> DotStatement) 
            -> FiniteMap (NodeName,NodeName) et 
            -> String
lrel2dot n printEdge lrel
  = "digraph "++n++" {\n"++(unlines stmts)++"}\n"
    where
      stmts     = nodeStmts++edgeStmts
      nodeStmts = map printNode (setToList (entities lrel))
      edgeStmts = map printEdge (fmToList lrel)
      
printEdge (nodes,"call") = mkEdge "color=red" nodes
printEdge (nodes,"nest") = mkEdge "color=tan" nodes
printEdge (nodes,_)      = mkEdge "color=grey" nodes

printNode node = mkNode "shape=box, height=.1" node

{-
mkGraph n edges
  = "digraph "++n++" {\n"++edges++"}\n"

mkNode n    = n++" [ shape=box ]\n"
       
mkEdge "" t = ""    -- This case takes care of root
mkEdge s t  = s++" -> "++t++"\n"

mkCallEdge "" t = ""    -- This case takes care of root
mkCallEdge s t  = s++" -> "++t++" [ style=dashed ]\n"

mkIterEdge "" t = ""    -- This case takes care of root
mkIterEdge s t  = t++" [shape=circle,color=blue,label=\"\",size=.5]\n"++
                  s++" -> "++t++"\n"
                  
mkCondEdge ps "" t = ""    -- This case takes care of root
mkCondEdge ps s t  
  = t++" [shape=record,label="++label++",size=.125,color=brown]\n"++
    s++" -> "++t++"\n"
    where label  = "\"{ if |{ "++ports++" }}\""
          ports  = concat (sepWith " | " (map port ps))
          port p = "<"++p++"> "++p 
-}
------------------------------------------------------------------------------

  
