-------------------------------------------------------------------------------
--
-- | 
-- Module      :  Language.PLSql.Pretty
-- Copyright   :  (c) Tiago Alves, Paulo Silva, 2004
-- License     :  LGPL
-- 
-- Maintainer  :  joost.visser@di.uminho.pt
-- Stability   :  experimental
-- Portability :  portable
--
-- This module defines a pretty-printer for PL-Sql.
--
-------------------------------------------------------------------------------


module Language.PLSql.Pretty (prettyPrint) where


import Text.PrettyPrint
import Language.PLSql.Syntax

-----------------------------------------------------------------------------


class PLSqlPP a where
	-- | Pretty-print something in isolation.
	pretty :: a -> Doc
	-- | Pretty-print something in a precedence context.
	prettyPrec :: Int -> a -> Doc
	pretty = prettyPrec 0
	prettyPrec _ = pretty


prettyPrint :: PLSqlPP a => a -> String
prettyPrint = renderStyle style . pretty 


-- | Library
prettyList s l = cat $ punctuate s $ map pretty l

instance (PLSqlPP a) => PLSqlPP (Maybe a) where
    pretty (Nothing) = empty
    pretty (Just x) = pretty x

-- | Definition

instance PLSqlPP Alias where
    pretty (Alias x0 x1) = pretty x0 <+> pretty x1

instance PLSqlPP Aliased_select_item where
    pretty (Aliased_select_item x0 x1) = 
        pretty x0 <+> pretty x1        

instance PLSqlPP Aliased_select_items where
    pretty (Aliased_select_items x0) =
        prettyList (text ",") x0

instance PLSqlPP Aliased_select_items_or_all where
    pretty (Aliased_select_items_or_all_c1 x0) =
        pretty x0
    pretty (Aliased_select_items_or_all_c2) =
        text "*"

instance PLSqlPP Aliased_table_description where
    pretty (Aliased_table_description x0 x1) =
        pretty x0 <+> pretty x1

instance PLSqlPP Aliased_table_descriptions where
    pretty (Aliased_table_descriptions x0) =
        prettyList (text ",") x0

instance PLSqlPP Allocate_extent_clause where
    pretty (Allocate_extent_clause x1) =
        text "(" <+> pretty x1 <+> text ")"

instance PLSqlPP Allocate_extent_item where
    pretty (Allocate_extent_item_c1 x0) =
        pretty x0
    pretty (Allocate_extent_item_c2 x1) =
        text "DATAFILE" <+> pretty x1
    pretty (Allocate_extent_item_c3 x1) = 
        text "INSTANCE" <+> pretty x1

instance PLSqlPP Allocate_extent_items where
    pretty (Allocate_extent_items x0) = hsep( map pretty x0)

instance PLSqlPP Allocate_statement where
    pretty (Allocate_statement x1) =
        text "ALLOCATE" <+> pretty x1

instance PLSqlPP Alter_cluster_item where
    pretty (Alter_cluster_item_c1 x1) = 
        text "PCTUSED" <+> pretty x1
    pretty (Alter_cluster_item_c2 x1) =
        text "PCTFREE" <+> pretty x1
    pretty (Alter_cluster_item_c3 x0) = pretty x0
    pretty (Alter_cluster_item_c4 x1) =
        text "INITRANS" <+> pretty x1
    pretty (Alter_cluster_item_c5 x1) =
        text "MAXTRANS" <+> pretty x1
    pretty (Alter_cluster_item_c6 x0) = pretty x0
    pretty (Alter_cluster_item_c7 x2) = 
        text "ALLOCATE" <+> text "EXTENT" <+> pretty x2
    pretty (Alter_cluster_item_c8 x2) = 
        text "DEALLOCATE" <+> text "UNUSED" <+> pretty x2

instance PLSqlPP Alter_cluster_items where
    pretty (Alter_cluster_items x0) = hsep( map pretty x0)

instance PLSqlPP Alter_cluster_statement where
    pretty (Alter_cluster_statement x2 x3 x4)
      = text "ALTER" <+>
          text "CLUSTER" <+> pretty x2 <+> pretty x3 <+> pretty x4

instance PLSqlPP Alter_database_item where
    pretty (Alter_database_item_c1 x1 x2)
      = text "MOUNT" <+> pretty x1 <+> pretty x2
    pretty (Alter_database_item_c2) = text "CONVERT"
    pretty (Alter_database_item_c3 x1)
      = text "OPEN" <+> pretty x1
    pretty (Alter_database_item_c4 x1)
      = text "ACTIVATE" <+> pretty x1
    pretty (Alter_database_item_c5) = text "ARCHIVELOG"
    pretty (Alter_database_item_c6) = text "NOARCHIVELOG"
    pretty (Alter_database_item_c7 x1)
      = text "RECOVER" <+> pretty x1
    pretty (Alter_database_item_c8 x2 x3)
      = text "ADD" <+>
          text "LOGFILE" <+> pretty x2 <+> pretty x3
    pretty (Alter_database_item_c9 x3)
      = text "ADD" <+>
          text "LOGFILE" <+> text "MEMBER" <+> pretty x3
    pretty (Alter_database_item_c10 x2)
      = text "DROP" <+> text "LOGFILE" <+> pretty x2
    pretty (Alter_database_item_c11 x3)
      = text "DROP" <+>
          text "LOGFILE" <+> text "MEMBER" <+> pretty x3
    pretty (Alter_database_item_c12 x1 x3 x4)
      = text "CLEAR" <+>
          pretty x1 <+> text "LOGFILE" <+> pretty x3 <+> pretty x4
    pretty (Alter_database_item_c13 x0) = pretty x0
    pretty (Alter_database_item_c14 x4 x5)
      = text "CREATE" <+>
          text "STANDBY" <+>
        text "CONTROLFILE" <+>
          text "AS" <+> pretty x4 <+> pretty x5
    pretty (Alter_database_item_c15 x2)
      = text "BACKUP" <+> text "CONTROLFILE" <+> pretty x2
    pretty (Alter_database_item_c16 x3)
      = text "RENAME" <+>
          text "GLOBAL_NAME" <+> text "TO" <+> pretty x3
    pretty (Alter_database_item_c17)
      = text "RESET" <+> text "COMPATIBILITY"
    pretty (Alter_database_item_c18 x1)
      = text "SET" <+> pretty x1
    pretty (Alter_database_item_c19 x1 x2)
      = text "ENABLE" <+> pretty x1 <+> pretty x2
    pretty (Alter_database_item_c20 x1)
      = text "DISABLE" <+> pretty x1
    pretty (Alter_database_item_c21 x2 x4)
      = text "CREATE" <+>
          text "DATAFILE" <+> pretty x2 <+> text "AS" <+> pretty x4
    pretty (Alter_database_item_c22 x1 x2)
      = text "DATAFILE" <+> pretty x1 <+> pretty x2

instance PLSqlPP Alter_database_items where
    pretty (Alter_database_items x0) = hsep( map pretty x0)

instance PLSqlPP Alter_database_set_item where
    pretty (Alter_database_set_item_c1 x2)
      = text "DBLOW" <+> text "=" <+> pretty x2
    pretty (Alter_database_set_item_c2 x2)
      = text "DBHIGH" <+> text "=" <+> pretty x2
    pretty (Alter_database_set_item_c3 x1)
      = text "DBMAC" <+> pretty x1

instance PLSqlPP Alter_database_statement where
    pretty (Alter_database_statement x2 x3)
      = text "ALTER" <+>
          text "DATABASE" <+> pretty x2 <+> pretty x3

instance PLSqlPP Alter_datafile_item where
    pretty (Alter_datafile_item_c1) = text "ONLINE"
    pretty (Alter_datafile_item_c2 x1)
      = text "OFFLINE" <+> pretty x1
    pretty (Alter_datafile_item_c3 x1)
      = text "RESIZE" <+> pretty x1
    pretty (Alter_datafile_item_c4 x0) = pretty x0
    pretty (Alter_datafile_item_c5)
      = text "END" <+> text "BACKUP"

instance PLSqlPP Alter_function_statement where
    pretty (Alter_function_statement x2)
      = text "ALTER" <+>
          text "FUNCTION" <+> pretty x2 <+> text "COMPILE"

instance PLSqlPP Alter_index_statement where
    pretty (Alter_index_statement x2 x3)
      = text "ALTER" <+>
          text "INDEX" <+> pretty x2 <+> pretty x3

instance PLSqlPP Alter_index_statement_item where
    pretty (Alter_index_statement_item_c1 x1)
      = text "PCTFREE" <+> pretty x1
    pretty (Alter_index_statement_item_c2 x1)
      = text "INITRANS" <+> pretty x1
    pretty (Alter_index_statement_item_c3 x1)
      = text "MAXTRANS" <+> pretty x1
    pretty (Alter_index_statement_item_c4 x0) = pretty x0
    pretty (Alter_index_statement_item_c5 x2)
      = text "ALLOCATE" <+> text "EXTENT" <+> pretty x2
    pretty (Alter_index_statement_item_c6 x2)
      = text "DEALLOCATE" <+> text "UNUSED" <+> pretty x2
    pretty (Alter_index_statement_item_c7 x1)
      = text "REBUILD" <+> pretty x1

instance PLSqlPP Alter_index_statement_items where
    pretty (Alter_index_statement_items x0) = hsep( map pretty x0)

instance PLSqlPP Alter_package_statement where
    pretty (Alter_package_statement x2 x4)
      = text "ALTER" <+>
          text "PACKAGE" <+>
        pretty x2 <+> text "COMPILE" <+> pretty x4


instance PLSqlPP Alter_procedure_statement where
    pretty (Alter_procedure_statement x2)
      = text "ALTER" <+>
          text "PROCEDURE" <+> pretty x2 <+> text "COMPILE"

instance PLSqlPP Alter_profile_item where
    pretty (Alter_profile_item_c1 x0 x1)
      = pretty x0 <+> pretty x1
    pretty (Alter_profile_item_c2 x1)
      = text "PRIVATE_SGA" <+> pretty x1

instance PLSqlPP Alter_profile_item_1 where
    pretty (Alter_profile_item_1_c1)
      = text "SESSIONS_PER_USER"
    pretty (Alter_profile_item_1_c2) = text "CPU_PER_SESSION"
    pretty (Alter_profile_item_1_c3) = text "CPU_PER_CALL"
    pretty (Alter_profile_item_1_c4) = text "CONNECT_TIME"
    pretty (Alter_profile_item_1_c5) = text "IDLE_TIME"
    pretty (Alter_profile_item_1_c6)
      = text "LOGICAL_READS_PER_SESSION"
    pretty (Alter_profile_item_1_c7)
      = text "LOGICAL_READS_PER_CALL"
    pretty (Alter_profile_item_1_c8) = text "COMPOSITE_LIMIT"

instance PLSqlPP Alter_profile_items where
    pretty (Alter_profile_items x0) = hsep( map pretty x0)

instance PLSqlPP Alter_profile_statement where
    pretty (Alter_profile_statement x2 x4)
      = text "ALTER" <+>
          text "PROFILE" <+>
        pretty x2 <+> text "LIMIT" <+> pretty x4

instance PLSqlPP Alter_resource_cost_statement where
    pretty (Alter_resource_cost_statement x3)
      = text "ALTER" <+>
          text "RESOURCE" <+> text "COST" <+> pretty x3

instance PLSqlPP Alter_role_statement where
    pretty (Alter_role_statement x2 x3)
      = text "ALTER" <+>
          text "ROLE" <+> pretty x2 <+> pretty x3

instance PLSqlPP Alter_rollback_segment_statement where
    pretty (Alter_rollback_segment_statement x3 x4)
      = text "ALTER" <+>
          text "ROLLBACK" <+>
        text "SEGMENT" <+> pretty x3 <+> pretty x4

instance PLSqlPP Alter_sequence_statement where
    pretty (Alter_sequence_statement x2 x3)
      = text "ALTER" <+>
          text "SEQUENCE" <+> pretty x2 <+> pretty x3

instance PLSqlPP Alter_session_clause where
    pretty (Alter_session_clause_c1 x1)
      = text "SET" <+> pretty x1
    pretty (Alter_session_clause_c2 x3)
      = text "CLOSE" <+>
          text "DATABASE" <+> text "LINK" <+> pretty x3
    pretty (Alter_session_clause_c3 x1)
      = text "ADVISE" <+> pretty x1
    pretty (Alter_session_clause_c4 x0)
      = pretty x0 <+>
          text "COMMIT" <+> text "IN" <+> text "PROCEDURE"

instance PLSqlPP Alter_session_statement where
    pretty (Alter_session_statement x2)
      = text "ALTER" <+> text "SESSION" <+> pretty x2

instance PLSqlPP Alter_snapshot_log_statement where
    pretty (Alter_snapshot_log_statement x3 x4 x5)
      = text "ALTER" <+>
          text "SNAPSHOT" <+>
        text "LOG" <+> pretty x3 <+> pretty x4 <+> pretty x5

instance PLSqlPP Alter_snapshot_statement where
    pretty (Alter_snapshot_statement x2 x3 x4 x5)
      = text "ALTER" <+>
          text "SNAPSHOT" <+>
        pretty x2 <+> pretty x3 <+> pretty x4 <+> pretty x5

instance PLSqlPP Alter_system_statement where
    pretty (Alter_system_statement x2)
      = text "ALTER" <+> text "SYSTEM" <+> pretty x2

instance PLSqlPP Alter_table_item where
    pretty (Alter_table_item x0 x1) = pretty x0 <+> pretty x1

instance PLSqlPP Alter_table_item_1 where
    pretty (Alter_table_item_1_c1 x1)
      = text "ADD" <+> pretty x1
    pretty (Alter_table_item_1_c2 x2)
      = text "MODIFY" <+> text "(" <+> pretty x2 <+> text ")"
    pretty (Alter_table_item_1_c3 x1)
      = text "MODIFY" <+> pretty x1
    pretty (Alter_table_item_1_c4 x1)
      = text "PCTFREE" <+> pretty x1
    pretty (Alter_table_item_1_c5 x1)
      = text "PCTUSED" <+> pretty x1
    pretty (Alter_table_item_1_c6 x1)
      = text "INITRANS" <+> pretty x1
    pretty (Alter_table_item_1_c7 x1)
      = text "MAXTRANS" <+> pretty x1
    pretty (Alter_table_item_1_c8 x0) = pretty x0
    pretty (Alter_table_item_1_c9 x1)
      = text "DROP" <+> pretty x1
    pretty (Alter_table_item_1_c10 x2)
      = text "ALLOCATE" <+> text "EXTENT" <+> pretty x2
    pretty (Alter_table_item_1_c11 x2)
      = text "DEALLOCATE" <+> text "UNUSED" <+> pretty x2

instance PLSqlPP Alter_table_item_2 where
    pretty (Alter_table_item_2_c1 x0) = pretty x0
    pretty (Alter_table_item_2_c2 x0 x1)
      = pretty x0 <+> pretty x1

instance PLSqlPP Alter_table_items_1 where
    pretty (Alter_table_items_1 x0) = hsep( map pretty x0)

instance PLSqlPP Alter_table_statement where
    pretty (Alter_table_statement x2 x3)
      = text "ALTER" <+>
          text "TABLE" <+> pretty x2 <+> pretty x3

instance PLSqlPP Alter_tablespace_statement where
    pretty (Alter_tablespace_statement x2 x3)
      = text "ALTER" <+>
          text "TABLESPACE" <+> pretty x2 <+> pretty x3

instance PLSqlPP Alter_trigger_statement where
    pretty (Alter_trigger_statement x2 x3)
      = text "ALTER" <+>
          text "TRIGGER" <+> pretty x2 <+> pretty x3


instance PLSqlPP Alter_user_statement where
    pretty (Alter_user_statement x2 x3)
      = text "ALTER" <+>
          text "USER" <+> pretty x2 <+> pretty x3

instance PLSqlPP Alter_view_statement where
    pretty (Alter_view_statement x2)
      = text "ALTER" <+>
          text "VIEW" <+> pretty x2 <+> text "COMPILE"

instance PLSqlPP Append_item where
    pretty (Append_item x0 x1 x2 x3 x4)
      = pretty x0 <+>
          pretty x1 <+> pretty x2 <+> pretty x3 <+> pretty x4

instance PLSqlPP Append_items where
    pretty (Append_items x0) = prettyList (text ",") x0

instance PLSqlPP Append_items_brackets where
    pretty (Append_items_brackets x1)
      = text "(" <+> pretty x1 <+> text ")"

instance PLSqlPP Archive_log_clause where
    pretty (Archive_log_clause x2 x3)
      = text "ARCHIVE" <+>
          text "LOG" <+> pretty x2 <+> pretty x3

instance PLSqlPP Archive_log_item where
    pretty (Archive_log_item_c1 x0 x1)
      = pretty x0 <+> pretty x1
    pretty (Archive_log_item_c2) = text "STOP"

instance PLSqlPP Archive_log_item_located where
    pretty (Archive_log_item_located_c1 x1)
      = text "SEQUENCE" <+> pretty x1
    pretty (Archive_log_item_located_c2 x1)
      = text "CHANGE" <+> pretty x1
    pretty (Archive_log_item_located_c3) = text "CURRENT"
    pretty (Archive_log_item_located_c4 x1)
      = text "GROUP" <+> pretty x1
    pretty (Archive_log_item_located_c5 x1)
      = text "LOGFILE" <+> pretty x1
    pretty (Archive_log_item_located_c6) = text "NEXT"
    pretty (Archive_log_item_located_c7) = text "ALL"
    pretty (Archive_log_item_located_c8) = text "START"

instance PLSqlPP As_subquery where
    pretty (As_subquery x1) = text "AS" <+> pretty x1

instance PLSqlPP Assignment_statement where
    pretty (Assignment_statement x0 x1 x2)
      = pretty x0 <+> pretty x1 <+> pretty x2

instance PLSqlPP Assignment_symbol where
    pretty (Assignment_symbol) = text ":="

instance PLSqlPP Atomic_boolean_expression where
    pretty (Atomic_boolean_expression_c1 x0) = pretty x0
    pretty (Atomic_boolean_expression_c2 x0 x1 x2)
      = pretty x0 <+> pretty x1 <+> pretty x2
    pretty (Atomic_boolean_expression_c3 x0 x1 x2)
      = pretty x0 <+> pretty x1 <+> pretty x2
    pretty (Atomic_boolean_expression_c4 x0 x1 x2 x3)
      = pretty x0 <+> pretty x1 <+> pretty x2 <+> pretty x3
    pretty (Atomic_boolean_expression_c5 x0 x1 x3)
      = pretty x0 <+> pretty x1 <+> text "IN" <+> pretty x3
    pretty (Atomic_boolean_expression_c6 x0 x1 x3)
      = pretty x0 <+> pretty x1 <+> text "IN" <+> pretty x3
    pretty (Atomic_boolean_expression_c7 x0 x1 x3 x5)
      = pretty x0 <+>
          pretty x1 <+>
        text "BETWEEN" <+> pretty x3 <+> text "AND" <+> pretty x5
    pretty (Atomic_boolean_expression_c8 x0 x2)
      = pretty x0 <+> text "IS" <+> pretty x2 <+> text "NULL"
    pretty (Atomic_boolean_expression_c9 x1)
      = text "EXISTS" <+> pretty x1
    pretty (Atomic_boolean_expression_c10 x1)
      = text "(" <+> pretty x1 <+> text ")"
    pretty (Atomic_boolean_expression_c11 x0) = pretty x0
    pretty (Atomic_boolean_expression_c12 x2)
      = text "CURRENT" <+> text "OF" <+> pretty x2
    pretty (Atomic_boolean_expression_c13 x0 x1)
      = pretty x0 <+> pretty x1
    pretty (Atomic_boolean_expression_c14 x0 x1 x3)
      = pretty x0 <+> pretty x1 <+> text "LIKE" <+> pretty x3
    pretty (Atomic_boolean_expression_c15 x0 x1 x3 x4)
      = pretty x0 <+>
          pretty x1 <+> text "LIKE" <+> pretty x3 <+> hsep (map pretty x4)

instance PLSqlPP Atomic_expression where
    pretty (Atomic_expression_c1 x0) = pretty x0
    pretty (Atomic_expression_c2 x0) = pretty x0
    pretty (Atomic_expression_c3 x0 x1)
      = pretty x0 <+> pretty x1
    pretty (Atomic_expression_c4 x0) = pretty x0
    pretty (Atomic_expression_c5 x0) = pretty x0
    pretty (Atomic_expression_c6 x1)
      = text "(" <+> pretty x1 <+> text ")"
    pretty (Atomic_expression_c7 x0 x1)
      = pretty x0 <+> pretty x1
    pretty (Atomic_expression_c8 x0)
      = pretty x0 <+> text "(" <+> text "+" <+> text ")"
    pretty (Atomic_expression_c9 x1 x3)
      = text "(" <+>
          pretty x1 <+> text ":" <+> pretty x3 <+> text ")"

instance PLSqlPP Autoextend where
    pretty (Autoextend_c1) = text "OFF"
    pretty (Autoextend_c2 x1 x2)
      = text "ON" <+> pretty x1 <+> pretty x2

instance PLSqlPP Autoextend_clause where
    pretty (Autoextend_clause x1)
      = text "AUTOEXTEND" <+> pretty x1

instance PLSqlPP Backup_controlfile_clause where
    pretty (Backup_controlfile_clause_c1 x1 x2)
      = text "TO" <+> pretty x1 <+> pretty x2
    pretty (Backup_controlfile_clause_c2 x2)
      = text "TO" <+> text "TRACE" <+> pretty x2

instance PLSqlPP Basic_loop_statement where
    pretty (Basic_loop_statement x1 x4)
      = text "LOOP" <+>
          pretty x1 <+> text "END" <+> text "LOOP" <+> pretty x4

instance PLSqlPP Begin_declare_section_statement where
    pretty (Begin_declare_section_statement)
      = text "BEGIN" <+> text "DECLARE" <+> text "SECTION"

instance PLSqlPP Binary_or_name where
    pretty (Binary_or_name_c1) = text "BINARY"
    pretty (Binary_or_name_c2 x0) = pretty x0

instance PLSqlPP Boolean_expression where
    pretty (Boolean_expression_c1 x0) = pretty x0
    pretty (Boolean_expression_c2 x1)
      = text "NOT" <+> pretty x1
    pretty (Boolean_expression_c3 x0 x2)
      = pretty x0 <+> text "AND" <+> pretty x2
    pretty (Boolean_expression_c4 x0 x2)
      = pretty x0 <+> text "OR" <+> pretty x2

instance PLSqlPP Break_item where
    pretty (Break_item x1) = text "ON" <+> hsep( map pretty x1)

instance PLSqlPP Bulk_collect where
    pretty (Bulk_collect) = text "BULK" <+> text "COLLECT"

instance PLSqlPP Call where
    pretty (Call_c1 x0) = pretty x0
    pretty (Call_c2 x0) = pretty x0

instance PLSqlPP Checkpoint where
    pretty (Checkpoint_c1) = text "CHECKPOINT"
    pretty (Checkpoint_c2)
      = text "CHECK" <+> text "DATAFILES"

instance PLSqlPP Close_statement where
    pretty (Close_statement x1) = text "CLOSE" <+> pretty x1

instance PLSqlPP Col_item where
    pretty (Col_item_c1 x1) = text "FORMAT" <+> pretty x1
    pretty (Col_item_c2 x1) = text "JUST" <+> pretty x1
    pretty (Col_item_c3) = text "NEWLINE"
    pretty (Col_item_c4) = text "NOPRINT"
    pretty (Col_item_c5) = text "PRINT"
    pretty (Col_item_c6 x1) = text "HEADING" <+> pretty x1
    pretty (Col_item_c7 x1) = text "HEADING" <+> pretty x1
    pretty (Col_item_c8 x1) = text "NEW_VALUE" <+> pretty x1

instance PLSqlPP Col_items where
    pretty (Col_items x0) = hsep( map pretty x0)

instance PLSqlPP Column_name where
    pretty (Column_name x0) = pretty x0

instance PLSqlPP Column_names where
    pretty (Column_names x0) = prettyList (text ",") x0

instance PLSqlPP Columns_brackets where
    pretty (Columns_brackets x1)
      = text "(" <+> pretty x1 <+> text ")"

instance PLSqlPP Comment where
    pretty (Comment_c1 x1) = text "COMMENT" <+> pretty x1
    pretty (Comment_c2) = text "RELEASE"

instance PLSqlPP Comment_item where
    pretty (Comment_item_c1) = text "TABLE"
    pretty (Comment_item_c2) = text "COLUMN"

instance PLSqlPP Comment_statement where
    pretty (Comment_statement x2 x3 x5)
      = text "COMMENT" <+>
          text "ON" <+>
        pretty x2 <+> pretty x3 <+> text "IS" <+> pretty x5

instance PLSqlPP Commit_statement where
    pretty (Commit_statement x1 x2)
      = text "COMMIT" <+> pretty x1 <+> pretty x2

instance PLSqlPP Connect_by_clause where
    pretty (Connect_by_clause x0 x3 x4)
      = pretty x0 <+>
          text "CONNECT" <+> text "BY" <+> pretty x3 <+> pretty x4

instance PLSqlPP Connect_by_clause_prefix where
    pretty (Connect_by_clause_prefix x2)
      = text "START" <+> text "WITH" <+> pretty x2

instance PLSqlPP Connect_to_clause where
    pretty (Connect_to_clause x2 x5)
      = text "CONNECT" <+>
          text "TO" <+>
        pretty x2 <+>
          text "IDENTIFIED" <+> text "BY" <+> pretty x5

instance PLSqlPP Connect_using_clause where
    pretty (Connect_using_clause x1)
      = text "USING" <+> pretty x1

instance PLSqlPP Constant_declaration where
    pretty (Constant_declaration x0 x1)
      = pretty x0 <+> pretty x1

instance PLSqlPP Constant_declaration_name where
    pretty (Constant_declaration_name x0 x2)
      = pretty x0 <+> text "CONSTANT" <+> pretty x2

instance PLSqlPP Constraint where
    pretty (Constraint_c1 x0 x1 x2)
      = pretty x0 <+> pretty x1 <+> pretty x2
    pretty (Constraint_c2 x0 x1) = pretty x0 <+> pretty x1

instance PLSqlPP Constraint_clause where
    pretty (Constraint_clause x1)
      = text "CONSTRAINT" <+> pretty x1

instance PLSqlPP Constraint_header where
    pretty (Constraint_header_c1 x1)
      = text "CONSTRAINT" <+> pretty x1
    pretty (Constraint_header_c2 x1)
      = text "CONSTRAINT" <+> pretty x1

instance PLSqlPP Constraint_item_1 where
    pretty (Constraint_item_1_c1 x0 x1)
      = pretty x0 <+> pretty x1
    pretty (Constraint_item_1_c2 x2 x4 x5 x6)
      = text "FOREIGN" <+>
          text "KEY" <+>
        pretty x2 <+>
          text "REFERENCES" <+>
            pretty x4 <+> pretty x5 <+> pretty x6
    pretty (Constraint_item_1_c3 x2)
      = text "CHECK" <+> text "(" <+> pretty x2 <+> text ")"
    pretty (Constraint_item_1_c4 x0)
      = pretty x0 <+> text "NULL"
    pretty (Constraint_item_1_c5 x1 x2 x3)
      = text "REFERENCES" <+>
          pretty x1 <+> pretty x2 <+> pretty x3
    pretty (Constraint_item_1_c6) = text "CASCADE"

instance PLSqlPP Constraint_item_2 where
    pretty (Constraint_item_2_c1 x2)
      = text "USING" <+> text "INDEX" <+> pretty x2
    pretty (Constraint_item_2_c2) = text "DISABLE"

instance PLSqlPP Constraints where
    pretty (Constraints x0) = hsep( map pretty x0)

instance PLSqlPP Create_database_link_statement where
    pretty (Create_database_link_statement x1 x4 x5 x6)
      = text "CREATE" <+>
          pretty x1 <+>
        text "DATABASE" <+>
          text "LINK" <+> pretty x4 <+> pretty x5 <+> pretty x6

instance PLSqlPP Create_function_statement where
    pretty (Create_function_statement x1 x2)
      = text "CREATE" <+> pretty x1 <+> pretty x2

instance PLSqlPP Create_index_statement where
    pretty (Create_index_statement x1 x2 x4 x6 x7 x8)
      = text "CREATE" <+>
          pretty x1 <+>
        pretty x2 <+>
          text "INDEX" <+>
            pretty x4 <+>
              text "ON" <+> pretty x6 <+> pretty x7 <+> pretty x8

instance PLSqlPP Create_or_replace where
    pretty (Create_or_replace x1)
      = text "CREATE" <+> pretty x1

instance PLSqlPP Create_procedure_statement where
    pretty (Create_procedure_statement x1 x2)
      = text "CREATE" <+> pretty x1 <+> pretty x2

instance PLSqlPP Create_role_statement where
    pretty (Create_role_statement x2 x3)
      = text "CREATE" <+>
          text "ROLE" <+> pretty x2 <+> pretty x3

instance PLSqlPP Create_rollback_segment_item where
    pretty (Create_rollback_segment_item_c1 x0) = pretty x0
    pretty (Create_rollback_segment_item_c2 x0) = pretty x0
    pretty (Create_rollback_segment_item_c3 x2)
      = text "OPTIMAL" <+> text "TO" <+> pretty x2
    pretty (Create_rollback_segment_item_c4)
      = text "OPTIMAL" <+> text "NULL"

instance PLSqlPP Create_rollback_segment_items where
    pretty (Create_rollback_segment_items x0) = hsep( map pretty x0)

instance PLSqlPP Create_rollback_segment_statement where
    pretty (Create_rollback_segment_statement x1 x4 x5)
      = text "CREATE" <+>
          pretty x1 <+>
        text "ROLLBACK" <+>
          text "SEGMENT" <+> pretty x4 <+> pretty x5

instance PLSqlPP Create_sequence_statement where
    pretty (Create_sequence_statement x2 x3)
      = text "CREATE" <+>
          text "SEQUENCE" <+> pretty x2 <+> hsep( map pretty x3)

instance PLSqlPP Create_synonym_statement where
    pretty (Create_synonym_statement x1 x3 x5)
      = text "CREATE" <+>
          pretty x1 <+>
        text "SYNONYM" <+> pretty x3 <+> text "FOR" <+> pretty x5

instance PLSqlPP Create_table_clause_1 where
    pretty (Create_table_clause_1_c1 x0) = pretty x0
    pretty (Create_table_clause_1_c2 x1 x2)
      = text "CLUSTER" <+> pretty x1 <+> pretty x2

instance PLSqlPP Create_table_clause_2 where
    pretty (Create_table_clause_2 x0 x1 x2)
      = pretty x0 <+> pretty x1 <+> pretty x2

instance PLSqlPP Create_table_statement where
    pretty (Create_table_statement x2 x3 x4 x5 x6)
      = text "CREATE" <+>
          text "TABLE" <+>
        pretty x2 <+>
          pretty x3 <+> pretty x4 <+> pretty x5 <+> pretty x6

instance PLSqlPP Create_tablespace_statement where
    pretty (Create_tablespace_statement x2 x4 x5)
      = text "CREATE" <+>
          text "TABLESPACE" <+>
        pretty x2 <+> text "DATAFILE" <+> pretty x4 <+> pretty x5

instance PLSqlPP Create_trigger_statement where
    pretty (Create_trigger_statement x1 x3 x4 x5 x7 x8 x9)
      = text "CREATE" <+>
          pretty x1 <+>
        text "TRIGGER" <+>
          pretty x3 <+>
            pretty x4 <+>
              pretty x5 <+>
            text "ON" <+> pretty x7 <+> pretty x8 <+> pretty x9

instance PLSqlPP Create_type_statement where
    pretty (Create_type_statement x1 x3 x4 x7)
      = text "CREATE" <+>
          pretty x1 <+>
        text "TYPE" <+>
          pretty x3 <+>
            pretty x4 <+> text "TABLE" <+> text "OF" <+> pretty x7

instance PLSqlPP Create_user_statement where
    pretty (Create_user_statement x2 x3)
      = text "CREATE" <+>
          text "USER" <+> pretty x2 <+> pretty x3

instance PLSqlPP Create_view_statement where
    pretty (Create_view_statement x1 x2 x4 x5 x7 x8)
      = text "CREATE" <+>
          pretty x1 <+>
        pretty x2 <+>
          text "VIEW" <+>
            pretty x4 <+>
              pretty x5 <+> text "AS" <+> pretty x7 <+> pretty x8

instance PLSqlPP Cursor where
    pretty (Cursor_c1 x0) = pretty x0
    pretty (Cursor_c2 x1) = text ":" <+> pretty x1

instance PLSqlPP Cursor_attribute_attr where
    pretty (Cursor_attribute_attr_c1 x0) = pretty x0
    pretty (Cursor_attribute_attr_c2 x0) = pretty x0
    pretty (Cursor_attribute_attr_c3 x0) = pretty x0
    pretty (Cursor_attribute_attr_c4 x0) = pretty x0

instance PLSqlPP Cursor_declaration where
    pretty (Cursor_declaration x0 x2)
      = pretty x0 <+> text "IS" <+> pretty x2

instance PLSqlPP Cursor_declaration_kernel where
    pretty (Cursor_declaration_kernel x1 x2)
      = text "CURSOR" <+> pretty x1 <+> pretty x2

instance PLSqlPP Cursor_name where
    pretty (Cursor_name x0) = pretty x0

instance PLSqlPP Cursor_ref where
    pretty (Cursor_ref x0 x1) = pretty x0 <+> pretty x1

instance PLSqlPP Cursor_return where
    pretty (Cursor_return x1) = text "RETURN" <+> pretty x1

instance PLSqlPP Datafile_description where
    pretty (Datafile_description x0 x1)
      = pretty x0 <+> pretty x1

instance PLSqlPP Datafile_descriptions where
    pretty (Datafile_descriptions x0)
      = prettyList (text ",") x0

instance PLSqlPP Datatype where
    pretty (Datatype_c1) = text "BINARY_INTEGER"
    pretty (Datatype_c2) = text "INTEGER"
    pretty (Datatype_c3) = text "NUMBER"
    pretty (Datatype_c4 x2)
      = text "NUMBER" <+> text "(" <+> pretty x2 <+> text ")"
    pretty (Datatype_c5 x2 x4)
      = text "NUMBER" <+>
          text "(" <+>
        pretty x2 <+> text "," <+> pretty x4 <+> text ")"
    pretty (Datatype_c6) = text "CHAR"
    pretty (Datatype_c7 x2)
      = text "CHAR" <+> text "(" <+> pretty x2 <+> text ")"
    pretty (Datatype_c8) = text "DATE"
    pretty (Datatype_c9) = text "VARCHAR"
    pretty (Datatype_c10 x2)
      = text "VARCHAR" <+> text "(" <+> pretty x2 <+> text ")"
    pretty (Datatype_c11) = text "VARCHAR2"
    pretty (Datatype_c12 x2)
      = text "VARCHAR2" <+> text "(" <+> pretty x2 <+> text ")"
    pretty (Datatype_c13) = text "LONG"
    pretty (Datatype_c14 x2)
      = text "INTEGER" <+> text "(" <+> pretty x2 <+> text ")"

instance PLSqlPP Date where
    pretty (Date x0) = pretty x0

instance PLSqlPP Dblink where
    pretty (Dblink x0) = pretty x0

instance PLSqlPP Declaration where
    pretty (Declaration_c1 x0) = pretty x0
    pretty (Declaration_c2 x0) = pretty x0
    pretty (Declaration_c3 x0) = pretty x0
    pretty (Declaration_c4 x0) = pretty x0
    pretty (Declaration_c5 x0) = pretty x0
    pretty (Declaration_c6 x0) = pretty x0
    pretty (Declaration_c7 x0) = pretty x0
    pretty (Declaration_c8 x0) = pretty x0
    pretty (Declaration_c9 x0) = pretty x0
    pretty (Declaration_c10 x0) = pretty x0
    pretty (Declaration_c11 x0) = pretty x0
    pretty (Declaration_c12 x0) = pretty x0

instance PLSqlPP Declaration_init_value where
    pretty (Declaration_init_value x0 x1)
      = pretty x0 <+> pretty x1

instance PLSqlPP Declarations where
    pretty (Declarations x0) = prettyList (text ";") x0

instance PLSqlPP Declarations_semicolon where
    pretty (Declarations_semicolon x0 x1)
      = pretty x0 <+> pretty x1

instance PLSqlPP Declare_cursor_statement where
    pretty (Declare_cursor_statement x1 x4)
      = text "DECLARE" <+>
          pretty x1 <+> text "CURSOR" <+> text "FOR" <+> pretty x4

instance PLSqlPP Declare_section where
    pretty (Declare_section x1)
      = text "DECLARE" <+> pretty x1

instance PLSqlPP Default_expression where
    pretty (Default_expression x1)
      = text "DEFAULT" <+> pretty x1

instance PLSqlPP Delete_item where
    pretty (Delete_item_c1 x0) = pretty x0
    pretty (Delete_item_c2 x0 x1) = pretty x0 <+> pretty x1
    pretty (Delete_item_c3 x0) = pretty x0

instance PLSqlPP Delete_statement where
    pretty (Delete_statement x1 x2 x3)
      = text "DELETE" <+> pretty x1 <+> pretty x2 <+> pretty x3

instance PLSqlPP Disable_clause where
    pretty (Disable_clause x1) = text "DISABLE" <+> pretty x1

instance PLSqlPP Disable_clause_item where
    pretty (Disable_clause_item_c1 x1 x2)
      = text "UNIQUE" <+> pretty x1 <+> pretty x2
    pretty (Disable_clause_item_c2 x2)
      = text "PRIMARY" <+> text "KEY" <+> pretty x2
    pretty (Disable_clause_item_c3 x0 x1)
      = pretty x0 <+> pretty x1
    pretty (Disable_clause_item_c4 x0) = pretty x0
    pretty (Disable_clause_item_c5)
      = text "TABLE" <+> text "LOCK"

instance PLSqlPP Drop_database_link_statement where
    pretty (Drop_database_link_statement x1 x4)
      = text "DROP" <+>
          pretty x1 <+>
        text "DATABASE" <+> text "LINK" <+> pretty x4

instance PLSqlPP Drop_function_statement where
    pretty (Drop_function_statement x2)
      = text "DROP" <+> text "FUNCTION" <+> pretty x2

instance PLSqlPP Drop_index_statement where
    pretty (Drop_index_statement x2)
      = text "DROP" <+> text "INDEX" <+> pretty x2

instance PLSqlPP Drop_procedure_statement where
    pretty (Drop_procedure_statement x2)
      = text "DROP" <+> text "PROCEDURE" <+> pretty x2

instance PLSqlPP Drop_sequence_statement where
    pretty (Drop_sequence_statement x2)
      = text "DROP" <+> text "SEQUENCE" <+> pretty x2

instance PLSqlPP Drop_synonym_statement where
    pretty (Drop_synonym_statement x1 x3)
      = text "DROP" <+>
          pretty x1 <+> text "SYNONYM" <+> pretty x3

instance PLSqlPP Drop_table_statement where
    pretty (Drop_table_statement x2 x3)
      = text "DROP" <+>
          text "TABLE" <+> pretty x2 <+> pretty x3

instance PLSqlPP Drop_trigger_statement where
    pretty (Drop_trigger_statement x2)
      = text "DROP" <+> text "TRIGGER" <+> pretty x2

instance PLSqlPP Drop_view_statement where
    pretty (Drop_view_statement x2)
      = text "DROP" <+> text "VIEW" <+> pretty x2

instance PLSqlPP Dummy_top_sort where
    pretty (Dummy_top_sort_c1 x0) = pretty x0
    pretty (Dummy_top_sort_c2 x0) = text x0

instance PLSqlPP Else_branch where
    pretty (Else_branch x1) = text "ELSE" <+> pretty x1

instance PLSqlPP Elsif_branch where
    pretty (Elsif_branch x1 x3)
      = text "ELSIF" <+>
          pretty x1 <+> text "THEN" <+> pretty x3

instance PLSqlPP Emb_name where
    pretty (Emb_name x0) = text x0

instance PLSqlPP Embedded_sql_statement where
    pretty (Embedded_sql_statement_c1 x0) = pretty x0
    pretty (Embedded_sql_statement_c2 x0) = pretty x0
    pretty (Embedded_sql_statement_c3 x0) = pretty x0
    pretty (Embedded_sql_statement_c4 x0) = pretty x0
    pretty (Embedded_sql_statement_c5 x0) = pretty x0
    pretty (Embedded_sql_statement_c6 x0) = pretty x0
    pretty (Embedded_sql_statement_c7 x0) = pretty x0
    pretty (Embedded_sql_statement_c8 x0) = pretty x0

instance PLSqlPP Embedded_sql_statements where
    pretty (Embedded_sql_statements x0)
      = prettyList (text ";") x0

instance PLSqlPP En_dis_able_clauses where
    pretty (En_dis_able_clauses_c1 x0) = pretty x0
    pretty (En_dis_able_clauses_c2 x0) = pretty x0
    pretty (En_dis_able_clauses_c3 x0 x1)
      = pretty x0 <+> pretty x1
    pretty (En_dis_able_clauses_c4 x0 x1)
      = pretty x0 <+> pretty x1

instance PLSqlPP Enable_clause where
    pretty (Enable_clause x1) = text "ENABLE" <+> pretty x1

instance PLSqlPP Enable_clause_item where
    pretty (Enable_clause_item_c1 x0 x1 x2)
      = pretty x0 <+> pretty x1 <+> pretty x2
    pretty (Enable_clause_item_c2 x0) = pretty x0
    pretty (Enable_clause_item_c3)
      = text "TABLE" <+> text "LOCK"

instance PLSqlPP Enable_clause_item_header where
    pretty (Enable_clause_item_header_c1 x1)
      = text "UNIQUE" <+> pretty x1
    pretty (Enable_clause_item_header_c2)
      = text "PRIMARY" <+> text "KEY"
    pretty (Enable_clause_item_header_c3 x0) = pretty x0


instance PLSqlPP End_declare_section_statement where
    pretty (End_declare_section_statement)
      = text "END" <+> text "DECLARE" <+> text "SECTION"

instance PLSqlPP End_of_function where
    pretty (End_of_function x1) = text "END" <+> pretty x1

instance PLSqlPP End_of_package where
    pretty (End_of_package x1) = text "END" <+> pretty x1

instance PLSqlPP End_of_procedure where
    pretty (End_of_procedure x1) = text "END" <+> pretty x1

instance PLSqlPP Exception_declaration where
    pretty (Exception_declaration x0)
      = pretty x0 <+> text "EXCEPTION"

instance PLSqlPP Exception_handler where
    pretty (Exception_handler x1 x3)
      = text "WHEN" <+> pretty x1 <+> text "THEN" <+> pretty x3

instance PLSqlPP Exception_handler_others where
    pretty (Exception_handler_others x3)
      = text "WHEN" <+>
          text "OTHERS" <+> text "THEN" <+> pretty x3

instance PLSqlPP Exception_handlers where
    pretty (Exception_handlers x0 x1)
      = hsep( map pretty x0) <+> pretty x1

instance PLSqlPP Exception_name where
    pretty (Exception_name_c1 x0) = pretty x0
    pretty (Exception_name_c2) = text "OTHERS"

instance PLSqlPP Exception_section where
    pretty (Exception_section x1)
      = text "EXCEPTION" <+> pretty x1

instance PLSqlPP Exception_when_clause where
    pretty (Exception_when_clause x0 x1)
      = pretty x0 <+> hsep( map pretty x1)

instance PLSqlPP Exceptions_into where
    pretty (Exceptions_into x2)
      = text "EXCEPTIONS" <+> text "INTO" <+> pretty x2

instance PLSqlPP Execute_parameter where
    pretty (Execute_parameter x0 x1)
      = pretty x0 <+> pretty x1

instance PLSqlPP Execute_statement where
    pretty (Execute_statement x1 x2 x3 x4)
      = text "EXECUTE" <+>
          pretty x1 <+> pretty x2 <+> pretty x3 <+> pretty x4

instance PLSqlPP Execute_using where
    pretty (Execute_using x1)
      = text "USING" <+> prettyList (text ",") x1

instance PLSqlPP Exit_statement where
    pretty (Exit_statement x1 x2)
      = text "EXIT" <+> pretty x1 <+> pretty x2

instance PLSqlPP Exit_when_clause where
    pretty (Exit_when_clause x1) = text "WHEN" <+> pretty x1

instance PLSqlPP Explain_plan_statement where
    pretty (Explain_plan_statement x2 x3 x5)
      = text "EXPLAIN" <+>
          text "PLAN" <+>
        pretty x2 <+> pretty x3 <+> text "FOR" <+> pretty x5

instance PLSqlPP Expression where
    pretty (Expression_c1 x0) = pretty x0
    pretty (Expression_c2 x1) = text "_" <+> pretty x1
    pretty (Expression_c3 x1) = text "+" <+> pretty x1
    pretty (Expression_c4 x0 x2)
      = pretty x0 <+> text "**" <+> pretty x2
    pretty (Expression_Expression1 x0 x2)
      = pretty x0 <+> text "*" <+> pretty x2
    pretty (Expression_Expression2 x0 x2)
      = pretty x0 <+> text "/" <+> pretty x2
    pretty (Expression_Expression3 x0 x2)
      = pretty x0 <+> text "+" <+> pretty x2
    pretty (Expression_c8 x0 x2)
      = pretty x0 <+> text "_" <+> pretty x2
    pretty (Expression_c9 x0 x2)
      = pretty x0 <+> text "||" <+> pretty x2

instance PLSqlPP Expression_or_subquery_brackets where
    pretty (Expression_or_subquery_brackets_c1 x0)
      = pretty x0
    pretty (Expression_or_subquery_brackets_c2 x0)
      = pretty x0

instance PLSqlPP Expressions where
    pretty (Expressions x0) = prettyList (text ",") x0

instance PLSqlPP Expressions_brackets where
    pretty (Expressions_brackets x1)
      = text "(" <+> pretty x1 <+> text ")"

instance PLSqlPP Expressions_or_subquery_brackets where
    pretty (Expressions_or_subquery_brackets_c1 x0)
      = pretty x0
    pretty (Expressions_or_subquery_brackets_c2 x0)
      = pretty x0

instance PLSqlPP Fetch_name where
    pretty (Fetch_name x0) = pretty x0

instance PLSqlPP Fetch_names where
    pretty (Fetch_names x0) = prettyList (text ",") x0

instance PLSqlPP Fetch_statement where
    pretty (Fetch_statement x1 x3)
      = text "FETCH" <+>
          pretty x1 <+> text "INTO" <+> pretty x3

instance PLSqlPP Field_declaration where
    pretty (Field_declaration x0 x1 x2)
      = pretty x0 <+> pretty x1 <+> pretty x2

instance PLSqlPP Field_declarations where
    pretty (Field_declarations x0) = prettyList (text ",") x0

instance PLSqlPP File_rename_clause where
    pretty (File_rename_clause x1 x2 x4)
      = text "RENAME" <+>
          pretty x1 <+> pretty x2 <+> text "TO" <+> pretty x4

instance PLSqlPP Filename where
    pretty (Filename x0) = text x0

instance PLSqlPP Filename_with_extension where
    pretty (Filename_with_extension x0) = text x0

instance PLSqlPP Filenames where
    pretty (Filenames x0) = prettyList (text ",") x0

instance PLSqlPP Filenames_opt_brackets where
    pretty (Filenames_opt_brackets_c1 x0) = pretty x0
    pretty (Filenames_opt_brackets_c2 x1)
      = text "(" <+> pretty x1 <+> text ")"

instance PLSqlPP Filespec where
    pretty (Filespec x0 x1 x2)
      = pretty x0 <+> pretty x1 <+> pretty x2

instance PLSqlPP Filespecs where
    pretty (Filespecs x0) = prettyList (text ",") x0

instance PLSqlPP For_loop_statement where
    pretty (For_loop_statement x1 x3 x4 x5)
      = text "FOR" <+>
          pretty x1 <+>
        text "IN" <+> pretty x3 <+> pretty x4 <+> pretty x5

instance PLSqlPP For_loop_statement_range where
    pretty (For_loop_statement_range_c1 x0) = pretty x0
    pretty (For_loop_statement_range_c2 x0 x2)
      = pretty x0 <+> text ".." <+> pretty x2
    pretty (For_loop_statement_range_c3 x0) = pretty x0

instance PLSqlPP For_sql_statement where
    pretty (For_sql_statement x1 x2)
      = text "FOR" <+> pretty x1 <+> pretty x2

instance PLSqlPP For_update_clause where
    pretty (For_update_clause x2 x3)
      = text "FOR" <+>
          text "UPDATE" <+> pretty x2 <+> pretty x3

instance PLSqlPP For_update_item where
    pretty (For_update_item x1) = text "OF" <+> pretty x1

instance PLSqlPP For_update_items where
    pretty (For_update_items x0) = prettyList (text ",") x0

instance PLSqlPP Format where
    pretty (Format x0) = text x0

instance PLSqlPP From_location where
    pretty (From_location x1) = text "FROM" <+> pretty x1

instance PLSqlPP Function_body_declaration where
    pretty (Function_body_declaration_c1 x0 x1 x2 x3 x4)
      = pretty x0 <+>
          pretty x1 <+> pretty x2 <+> pretty x3 <+> pretty x4
    pretty (Function_body_declaration_c2 x0 x1 x3 x4 x5)
      = pretty x0 <+>
          pretty x1 <+>
        text "BEGIN" <+> pretty x3 <+> pretty x4 <+> pretty x5
    pretty (Function_body_declaration_c3 x0 x1 x5)
      = pretty x0 <+>
          pretty x1 <+>
        text "LANGUAGE" <+>
          text "JAVA" <+> text "NAME" <+> pretty x5

instance PLSqlPP Function_call where
    pretty (Function_call_c1 x0 x1) = pretty x0 <+> pretty x1
    pretty (Function_call_c2)
      = text "COUNT" <+> text "(" <+> text "*" <+> text ")"
    pretty (Function_call_c3 x1) = text "COUNT" <+> pretty x1

instance PLSqlPP Function_header_declaration where
    pretty (Function_header_declaration x1 x2 x4)
      = text "FUNCTION" <+>
          pretty x1 <+> pretty x2 <+> text "RETURN" <+> pretty x4

instance PLSqlPP Function_name where
    pretty (Function_name x0) = pretty x0

instance PLSqlPP Goto_statement where
    pretty (Goto_statement x1) = text "GOTO" <+> pretty x1

instance PLSqlPP Grant_item where
    pretty (Grant_item_c1 x0) = pretty x0
    pretty (Grant_item_c2) = text ","
    pretty (Grant_item_c3 x0) = pretty x0

instance PLSqlPP Grant_statement where
    pretty (Grant_statement x1 x2 x4 x5)
      = text "GRANT" <+>
          hsep( map pretty x1) <+>
        pretty x2 <+> text "TO" <+> pretty x4 <+> pretty x5

instance PLSqlPP Grantee where
    pretty (Grantee_c1 x0) = pretty x0
    pretty (Grantee_c2 x0) = pretty x0

instance PLSqlPP Grantees where
    pretty (Grantees x0) = prettyList (text ",") x0

instance PLSqlPP Group_by_clause where
    pretty (Group_by_clause x2 x3)
      = text "GROUP" <+> text "BY" <+> pretty x2 <+> pretty x3

instance PLSqlPP Group_integer where
    pretty (Group_integer x1) = text "GROUP" <+> pretty x1

instance PLSqlPP Group_or_filenames where
    pretty (Group_or_filenames_c1 x0) = pretty x0
    pretty (Group_or_filenames_c2 x0) = pretty x0

instance PLSqlPP Group_or_filenames_list where
    pretty (Group_or_filenames_list x0)
      = prettyList (text ",") x0

instance PLSqlPP Grouped_filespec where
    pretty (Grouped_filespec x0 x1) = pretty x0 <+> pretty x1

instance PLSqlPP Grouped_filespecs where
    pretty (Grouped_filespecs x0) = prettyList (text ",") x0

instance PLSqlPP Having_clause where
    pretty (Having_clause x1) = text "HAVING" <+> pretty x1

instance PLSqlPP Host_language_statement where
    pretty (Host_language_statement x2)
      = text "EXEC" <+>
          text "SQL" <+> pretty x2 <+> text "END_EXEC"

instance PLSqlPP Host_name where
    pretty (Host_name x0) = pretty x0

instance PLSqlPP If_statement where
    pretty (If_statement x1 x2 x3 x4)
      = text "IF" <+>
          pretty x1 <+>
        pretty x2 <+>
          hsep( map pretty x3) <+> pretty x4 <+> text "END" <+> text "IF"

instance PLSqlPP Include_statement where
    pretty (Include_statement x1)
      = text "INCLUDE" <+> pretty x1

instance PLSqlPP Index_on_clause where
    pretty (Index_on_clause_c1 x0 x2)
      = pretty x0 <+> text "(" <+> pretty x2 <+> text ")"
    pretty (Index_on_clause_c2 x1)
      = text "CLUSTER" <+> pretty x1

instance PLSqlPP Index_parameter where
    pretty (Index_parameter_c1 x1)
      = text "PCTFREE" <+> pretty x1
    pretty (Index_parameter_c2 x1)
      = text "INITRANS" <+> pretty x1
    pretty (Index_parameter_c3 x1)
      = text "MAXTRANS" <+> pretty x1
    pretty (Index_parameter_c4 x0) = pretty x0
    pretty (Index_parameter_c5 x0) = pretty x0
    pretty (Index_parameter_c6) = text "NOSORT"
    pretty (Index_parameter_c7) = text "RECOVERABLE"
    pretty (Index_parameter_c8) = text "UNRECOVERABLE"
    pretty (Index_parameter_c9 x1)
      = text "PCTUSED" <+> pretty x1
    pretty (Index_parameter_c10)
      = text "ORGANIZATION" <+>
          text "INDEX" <+> text "NOCOMPRESS"

instance PLSqlPP Index_parameters where
    pretty (Index_parameters x0) = hsep( map pretty x0)

instance PLSqlPP Init_value where
    pretty (Init_value x0 x1) = pretty x0 <+> pretty x1

instance PLSqlPP Insert_statement where
    pretty (Insert_statement_c1 x1 x2 x3)
      = text "INSERT" <+> pretty x1 <+> pretty x2 <+> pretty x3
    pretty (Insert_statement_c2 x1 x2 x3 x4)
      = text "INSERT" <+>
          pretty x1 <+> pretty x2 <+> pretty x3 <+> pretty x4

instance PLSqlPP SqlInteger where
    pretty (Integer x0) = text x0

instance PLSqlPP Integer_or_default where
    pretty (Integer_or_default_c1 x0) = pretty x0
    pretty (Integer_or_default_c2) = text "DEFAULT"

instance PLSqlPP Integer_or_other where
    pretty (Integer_or_other_c1 x0) = pretty x0
    pretty (Integer_or_other_c2) = text "UNLIMITED"
    pretty (Integer_or_other_c3) = text "DEFAULT"

instance PLSqlPP Integer_or_unlimited where
    pretty (Integer_or_unlimited_c1 x0) = pretty x0
    pretty (Integer_or_unlimited_c2) = text "UNLIMITED"

instance PLSqlPP Integer_text_var where
    pretty (Integer_text_var_c1 x0) = pretty x0
    pretty (Integer_text_var_c2 x0) = pretty x0
    pretty (Integer_text_var_c3 x0) = pretty x0
    pretty (Integer_text_var_c4 x0) = pretty x0

instance PLSqlPP Into where
    pretty (Into x0) = pretty x0 <+> text "INTO"

instance PLSqlPP Into_qualified_name where
    pretty (Into_qualified_name x0 x1)
      = pretty x0 <+> pretty x1

instance PLSqlPP Into_qualified_names where
    pretty (Into_qualified_names x0 x1)
      = pretty x0 <+> pretty x1

instance PLSqlPP Isolation_level where
    pretty (Isolation_level_c1)
      = text "READ" <+> text "COMMITTED"
    pretty (Isolation_level_c2) = text "SERIALIZABLE"

instance PLSqlPP Isolation_level_clause where
    pretty (Isolation_level_clause x2)
      = text "ISOLATION" <+> text "LEVEL" <+> pretty x2

instance PLSqlPP Keep_clause where
    pretty (Keep_clause x1) = text "KEEP" <+> pretty x1

instance PLSqlPP Keyword_except_to_on_from where
    pretty (Keyword_except_to_on_from x0) = text x0

instance PLSqlPP Kw_all_triggers where
    pretty (Kw_all_triggers) = text "ALL" <+> text "TRIGGERS"

instance PLSqlPP Kw_any_some_all where
    pretty (Kw_any_some_all_c1) = text "ANY"
    pretty (Kw_any_some_all_c2) = text "SOME"
    pretty (Kw_any_some_all_c3) = text "ALL"

instance PLSqlPP Kw_append where
    pretty (Kw_append) = text "APPEND"

instance PLSqlPP Kw_as where
    pretty (Kw_as) = text "AS"

instance PLSqlPP Kw_assignment_default where
    pretty (Kw_assignment_default_c1 x0) = pretty x0
    pretty (Kw_assignment_default_c2) = text "DEFAULT"

instance PLSqlPP Kw_automatic where
    pretty (Kw_automatic) = text "AUTOMATIC"

instance PLSqlPP Kw_before_after_instead_of where
    pretty (Kw_before_after_instead_of_c1) = text "BEFORE"
    pretty (Kw_before_after_instead_of_c2) = text "AFTER"
    pretty (Kw_before_after_instead_of_c3)
      = text "INSTEAD" <+> text "OF"

instance PLSqlPP Kw_bitmap where
    pretty (Kw_bitmap) = text "BITMAP"

instance PLSqlPP Kw_cache_nocache where
    pretty (Kw_cache_nocache_c1) = text "CACHE"
    pretty (Kw_cache_nocache_c2) = text "NOCACHE"

instance PLSqlPP Kw_cascade where
    pretty (Kw_cascade) = text "CASCADE"

instance PLSqlPP Kw_cascade_constraints where
    pretty (Kw_cascade_constraints)
      = text "CASCADE" <+> text "CONSTRAINTS"

instance PLSqlPP Kw_commit_rollback_nothing where
    pretty (Kw_commit_rollback_nothing_c1) = text "COMMIT"
    pretty (Kw_commit_rollback_nothing_c2) = text "ROLLBACK"
    pretty (Kw_commit_rollback_nothing_c3) = text "NOTHING"

instance PLSqlPP Kw_default where
    pretty (Kw_default) = text "DEFAULT"

instance PLSqlPP Kw_desc_asc where
    pretty (Kw_desc_asc_c1) = text "DESC"
    pretty (Kw_desc_asc_c2) = text "ASC"

instance PLSqlPP Kw_distinct_all where
    pretty (Kw_distinct_all_c1) = text "DISTINCT"
    pretty (Kw_distinct_all_c2) = text "ALL"

instance PLSqlPP Kw_drop where
    pretty (Kw_drop) = text "DROP"

instance PLSqlPP Kw_enable_disable where
    pretty (Kw_enable_disable_c1) = text "ENABLE"
    pretty (Kw_enable_disable_c2) = text "DISABLE"

instance PLSqlPP Kw_enable_disable_compile where
    pretty (Kw_enable_disable_compile_c1) = text "ENABLE"
    pretty (Kw_enable_disable_compile_c2) = text "DISABLE"
    pretty (Kw_enable_disable_compile_c3) = text "COMPILE"

instance PLSqlPP Kw_exclusive_parallel where
    pretty (Kw_exclusive_parallel_c1) = text "EXCLUSIVE"
    pretty (Kw_exclusive_parallel_c2) = text "PARALLEL"

instance PLSqlPP Kw_external where
    pretty (Kw_external) = text "EXTERNAL"

instance PLSqlPP Kw_fast_complete_force where
    pretty (Kw_fast_complete_force_c1) = text "FAST"
    pretty (Kw_fast_complete_force_c2) = text "COMPLETE"
    pretty (Kw_fast_complete_force_c3) = text "FORCE"

instance PLSqlPP Kw_file_datafile where
    pretty (Kw_file_datafile_c1) = text "FILE"
    pretty (Kw_file_datafile_c2) = text "DATAFILE"

instance PLSqlPP Kw_force_no_force where
    pretty (Kw_force_no_force_c1) = text "FORCE"
    pretty (Kw_force_no_force_c2)
      = text "NO" <+> text "FORCE"

instance PLSqlPP Kw_from where
    pretty (Kw_from) = text "FROM"

instance PLSqlPP Kw_global_local where
    pretty (Kw_global_local_c1) = text "GLOBAL"
    pretty (Kw_global_local_c2) = text "LOCAL"

instance PLSqlPP Kw_immediate where
    pretty (Kw_immediate) = text "IMMEDIATE"

instance PLSqlPP Kw_in_out_nocopy where
    pretty (Kw_in_out_nocopy_c1) = text "IN"
    pretty (Kw_in_out_nocopy_c2) = text "OUT"
    pretty (Kw_in_out_nocopy_c3) = text "NOCOPY"

instance PLSqlPP Kw_into where
    pretty (Kw_into) = text "INTO"

instance PLSqlPP Kw_is_as where
    pretty (Kw_is_as_c1) = text "IS"
    pretty (Kw_is_as_c2) = text "AS"

instance PLSqlPP Kw_local where
    pretty (Kw_local) = text "LOCAL"

instance PLSqlPP Kw_no_resetlogs where
    pretty (Kw_no_resetlogs_c1) = text "RESETLOGS"
    pretty (Kw_no_resetlogs_c2) = text "NORESETLOGS"

instance PLSqlPP Kw_nocache_cache where
    pretty (Kw_nocache_cache_c1) = text "NOCACHE"
    pretty (Kw_nocache_cache_c2) = text "CACHE"

instance PLSqlPP Kw_nologging where
    pretty (Kw_nologging) = text "NOLOGGING"

instance PLSqlPP Kw_normal_temporary_immediate where
    pretty (Kw_normal_temporary_immediate_c1) = text "NORMAL"
    pretty (Kw_normal_temporary_immediate_c2)
      = text "TEMPORARY"
    pretty (Kw_normal_temporary_immediate_c3)
      = text "IMMEDIATE"

instance PLSqlPP Kw_not where
    pretty (Kw_not) = text "NOT"

instance PLSqlPP Kw_not_null where
    pretty (Kw_not_null) = text "NOT" <+> text "NULL"

instance PLSqlPP Kw_nowait where
    pretty (Kw_nowait) = text "NOWAIT"

instance PLSqlPP Kw_old_new where
    pretty (Kw_old_new_c1) = text "OLD"
    pretty (Kw_old_new_c2) = text "NEW"

instance PLSqlPP Kw_on where
    pretty (Kw_on) = text "ON"

instance PLSqlPP Kw_on_delete_cascade where
    pretty (Kw_on_delete_cascade)
      = text "ON" <+> text "DELETE" <+> text "CASCADE"

instance PLSqlPP Kw_on_off where
    pretty (Kw_on_off_c1) = text "ON"
    pretty (Kw_on_off_c2) = text "OFF"

instance PLSqlPP Kw_or_replace where
    pretty (Kw_or_replace) = text "OR" <+> text "REPLACE"

instance PLSqlPP Kw_package_body where
    pretty (Kw_package_body_c1) = text "PACKAGE"
    pretty (Kw_package_body_c2) = text "BODY"

instance PLSqlPP Kw_public where
    pretty (Kw_public) = text "PUBLIC"

instance PLSqlPP Kw_reuse where
    pretty (Kw_reuse) = text "REUSE"

instance PLSqlPP Kw_reverse where
    pretty (Kw_reverse) = text "REVERSE"

instance PLSqlPP Kw_savepoint where
    pretty (Kw_savepoint) = text "SAVEPOINT"

instance PLSqlPP Kw_semicolon where
    pretty (Kw_semicolon) = text ";"

instance PLSqlPP Kw_sql_trace_global_names where
    pretty (Kw_sql_trace_global_names_c1) = text "SQL_TRACE"
    pretty (Kw_sql_trace_global_names_c2)
      = text "GLOBAL_NAMES"

instance PLSqlPP Kw_standby_database where
    pretty (Kw_standby_database Nothing)
      = text "DATABASE"
    pretty (Kw_standby_database (Just _))
      = text "STANDBY" <+> text "DATABASE"

instance PLSqlPP Kw_table_cluster where
    pretty (Kw_table_cluster_c1) = text "TABLE"
    pretty (Kw_table_cluster_c2) = text "CLUSTER"

instance PLSqlPP Kw_timestamp_signature where
    pretty (Kw_timestamp_signature_c1) = text "TIMESTAMP"
    pretty (Kw_timestamp_signature_c2) = text "SIGNATURE"

instance PLSqlPP Kw_to where
    pretty (Kw_to) = text "TO"

instance PLSqlPP Kw_true_false where
    pretty (Kw_true_false_c1) = text "TRUE"
    pretty (Kw_true_false_c2) = text "FALSE"

instance PLSqlPP Kw_unarchived where
    pretty (Kw_unarchived) = text "UNARCHIVED"

instance PLSqlPP Kw_unique where
    pretty (Kw_unique) = text "UNIQUE"

instance PLSqlPP Kw_unique_primary_key where
    pretty (Kw_unique_primary_key_c1) = text "UNIQUE"
    pretty (Kw_unique_primary_key_c2)
      = text "PRIMARY" <+> text "KEY"

instance PLSqlPP Kw_unrecoverable_datafile where
    pretty (Kw_unrecoverable_datafile)
      = text "UNRECOVERABLE" <+> text "DATAFILE"

instance PLSqlPP Kw_with_admin_option where
    pretty (Kw_with_admin_option)
      = text "WITH" <+> text "ADMIN" <+> text "OPTION"

instance PLSqlPP Kw_work where
    pretty (Kw_work) = text "WORK"

instance PLSqlPP Label_header where
    pretty (Label_header x1)
      = text "<<" <+> pretty x1 <+> text ">>"

instance PLSqlPP Label_name where
    pretty (Label_name x0) = pretty x0

instance PLSqlPP Language where
    pretty (Language x0) = pretty x0

instance PLSqlPP Left_right where
    pretty (Left_right_c1) = text "RIGHT"
    pretty (Left_right_c2) = text "R"
    pretty (Left_right_c3) = text "LEFT"
    pretty (Left_right_c4) = text "L"

instance PLSqlPP Lhs where
    pretty (Lhs_c1 x0) = pretty x0
    pretty (Lhs_c2 x0 x2)
      = pretty x0 <+> text "(" <+> pretty x2 <+> text ")"

instance PLSqlPP Load_data_clause where
    pretty (Load_data_clause x0 x1 x4 x5 x6 x7 x8)
      = hsep( map pretty x0) <+>
          pretty x1 <+>
        text "INTO" <+>
          text "TABLE" <+>
            pretty x4 <+>
              pretty x5 <+> pretty x6 <+> pretty x7 <+> pretty x8

instance PLSqlPP Load_data_option where
    pretty (Load_data_option x0 x2)
      = pretty x0 <+> text "=" <+> pretty x2

instance PLSqlPP Load_data_options where
    pretty (Load_data_options x0) = prettyList (text ",") x0

instance PLSqlPP Load_data_parameter where
    pretty (Load_data_parameter_c1) = text "INFILE"
    pretty (Load_data_parameter_c2) = text "BADFILE"
    pretty (Load_data_parameter_c3) = text "DISCARDFILE"
    pretty (Load_data_parameter_c4)
      = text "APPEND" <+> text "RECLEN"
    pretty (Load_data_parameter_c5)
      = text "REPLACE" <+> text "RECLEN"

instance PLSqlPP Load_data_parameter_value where
    pretty (Load_data_parameter_value x0 x1)
      = pretty x0 <+> pretty x1

instance PLSqlPP Load_data_statement where
    pretty (Load_data_statement x2)
      = text "LOAD" <+> text "DATA" <+> hsep( map pretty x2)

instance PLSqlPP Load_data_value where
    pretty (Load_data_value_c1 x0) = pretty x0
    pretty (Load_data_value_c2 x0) = pretty x0

instance PLSqlPP Lock_mode where
    pretty (Lock_mode_c1) = text "ROW" <+> text "SHARE"
    pretty (Lock_mode_c2) = text "ROW" <+> text "EXCLUSIVE"
    pretty (Lock_mode_c3) = text "SHARE" <+> text "UPDATE"
    pretty (Lock_mode_c4) = text "SHARE"
    pretty (Lock_mode_c5)
      = text "SHARE" <+> text "ROW" <+> text "EXCLUSIVE"
    pretty (Lock_mode_c6) = text "EXCLUSIVE"

instance PLSqlPP Lock_table_statement where
    pretty (Lock_table_statement x2 x4 x6)
      = text "LOCK" <+>
          text "TABLE" <+>
        pretty x2 <+>
          text "IN" <+> pretty x4 <+> text "MODE" <+> pretty x6

instance PLSqlPP Logfile_member where
    pretty (Logfile_member x0 x2)
      = pretty x0 <+> text "TO" <+> pretty x2

instance PLSqlPP Logfile_members where
    pretty (Logfile_members x0) = prettyList (text ",") x0

instance PLSqlPP Loop_statement where
    pretty (Loop_statement_c1 x0) = pretty x0
    pretty (Loop_statement_c2 x0) = pretty x0
    pretty (Loop_statement_c3 x0) = pretty x0

instance PLSqlPP Max_memory_size where
    pretty (Max_memory_size_c1)
      = text "MAXSIZE" <+> text "UNLIMITED"
    pretty (Max_memory_size_c2 x1)
      = text "MAXSIZE" <+> pretty x1

instance PLSqlPP Memory_size where
    pretty (Memory_size x0) = pretty x0

instance PLSqlPP Memory_size_clause where
    pretty (Memory_size_clause x1)
      = text "SIZE" <+> pretty x1

instance PLSqlPP Memory_size_or_null where
    pretty (Memory_size_or_null_c1 x0) = pretty x0
    pretty (Memory_size_or_null_c2) = text "NULL"

instance PLSqlPP Memory_size_or_other where
    pretty (Memory_size_or_other_c1 x0) = pretty x0
    pretty (Memory_size_or_other_c2) = text "UNLIMITED"
    pretty (Memory_size_or_other_c3) = text "DEFAULT"

instance PLSqlPP Memory_size_or_unlimited where
    pretty (Memory_size_or_unlimited_c1 x0) = pretty x0
    pretty (Memory_size_or_unlimited_c2) = text "UNLIMITED"

instance PLSqlPP Meta_parameter where
    pretty (Meta_parameter_c1) = text "ARRAYSIZE"
    pretty (Meta_parameter_c2) = text "ECHO"
    pretty (Meta_parameter_c3) = text "ESCAPE"
    pretty (Meta_parameter_c4) = text "EMBEDDED"
    pretty (Meta_parameter_c5) = text "FEEDBACK"
    pretty (Meta_parameter_c6) = text "HEADING"
    pretty (Meta_parameter_c7) = text "LINES"
    pretty (Meta_parameter_c8) = text "LINESIZE"
    pretty (Meta_parameter_c9) = text "NEWPAGE"
    pretty (Meta_parameter_c10) = text "NULL"
    pretty (Meta_parameter_c11) = text "NUMW"
    pretty (Meta_parameter_c12) = text "NUMWIDTH"
    pretty (Meta_parameter_c13) = text "PAGESIZE"
    pretty (Meta_parameter_c14) = text "PAUSE"
    pretty (Meta_parameter_c15) = text "RECSEP"
    pretty (Meta_parameter_c16) = text "SERVEROUTPUT"
    pretty (Meta_parameter_c17) = text "SPACE"
    pretty (Meta_parameter_c18) = text "TERM"
    pretty (Meta_parameter_c19) = text "TERMOUT"
    pretty (Meta_parameter_c20) = text "TIME"
    pretty (Meta_parameter_c21) = text "TIMING"
    pretty (Meta_parameter_c22) = text "TRIMSPOOL"
    pretty (Meta_parameter_c23) = text "VERIFY"
    pretty (Meta_parameter_c24) = text "WRAP"

instance PLSqlPP Meta_parameter_value where
    pretty (Meta_parameter_value x0 x1)
      = pretty x0 <+> pretty x1

instance PLSqlPP Meta_parameter_values where
    pretty (Meta_parameter_values x0) = hsep( map pretty x0)

instance PLSqlPP Meta_statement where
    pretty (Meta_statement_c1 x0) = pretty x0
    pretty (Meta_statement_c2 x0) = pretty x0
    pretty (Meta_statement_c3 x0) = pretty x0
    pretty (Meta_statement_c4 x0) = pretty x0
    pretty (Meta_statement_c5 x0) = pretty x0
    pretty (Meta_statement_c6 x0) = pretty x0
    pretty (Meta_statement_c7 x0) = pretty x0
    pretty (Meta_statement_c8 x0) = pretty x0

instance PLSqlPP Meta_statement_semicolon where
    pretty (Meta_statement_semicolon x0 x1)
      = pretty x0 <+> pretty x1

instance PLSqlPP Name where
    pretty (Name_c1 x0) = pretty x0
    pretty (Name_c2) = text "CONVERT"
    pretty (Name_c3) = text "PROFILE"
    pretty (Name_c4) = text "LANGUAGE"
    pretty (Name_c5) = text "NAME"

instance PLSqlPP Name_or_keyword where
    pretty (Name_or_keyword x0) = text x0

instance PLSqlPP Names where
    pretty (Names x0) = prettyList (text ",") x0

instance PLSqlPP Names_brackets where
    pretty (Names_brackets x1)
      = text "(" <+> pretty x1 <+> text ")"

instance PLSqlPP Next_date where
    pretty (Next_date x1) = text "NEXT" <+> pretty x1

instance PLSqlPP Next_memory_size where
    pretty (Next_memory_size x1) = text "NEXT" <+> pretty x1

instance PLSqlPP Null_statement where
    pretty (Null_statement) = text "NULL"

instance PLSqlPP Number where
    pretty (Number x0) = text x0

instance PLSqlPP Of_columns where
    pretty (Of_columns x1) = text "OF" <+> pretty x1

instance PLSqlPP On_clause where
    pretty (On_clause x1) = text "ON" <+> pretty x1

instance PLSqlPP Open_for_clause where
    pretty (Open_for_clause_c1 x0) = pretty x0
    pretty (Open_for_clause_c2 x0 x2)
      = pretty x0 <+> text "USING" <+> pretty x2

instance PLSqlPP Open_for_cursor where
    pretty (Open_for_cursor_c1 x0) = pretty x0
    pretty (Open_for_cursor_c2 x1) = text ":" <+> pretty x1

instance PLSqlPP Open_for_statement where
    pretty (Open_for_statement x1 x3)
      = text "OPEN" <+> pretty x1 <+> text "FOR" <+> pretty x3

instance PLSqlPP Open_statement where
    pretty (Open_statement x1) = text "OPEN" <+> pretty x1

instance PLSqlPP Or_exception_name where
    pretty (Or_exception_name x1) = text "OR" <+> pretty x1

instance PLSqlPP Order_by_clause where
    pretty (Order_by_clause x2)
      = text "ORDER" <+> text "BY" <+> pretty x2

instance PLSqlPP Order_item where
    pretty (Order_item x0 x1) = pretty x0 <+> pretty x1

instance PLSqlPP Order_items where
    pretty (Order_items x0) = prettyList (text ",") x0

instance PLSqlPP Ordered_column where
    pretty (Ordered_column x0 x1) = pretty x0 <+> pretty x1

instance PLSqlPP Ordered_columns where
    pretty (Ordered_columns x0) = prettyList (text ",") x0

instance PLSqlPP Package_body where
    pretty (Package_body_c1 x0 x1 x2 x3)
      = pretty x0 <+> pretty x1 <+> pretty x2 <+> pretty x3
    pretty (Package_body_c2 x0 x1 x3 x4 x5 x6 x7 x8)
      = pretty x0 <+>
          pretty x1 <+>
        text "BEGIN" <+>
          pretty x3 <+>
            pretty x4 <+>
              pretty x5 <+> pretty x6 <+> pretty x7 <+> pretty x8

instance PLSqlPP Package_body_prefix where
    pretty (Package_body_prefix x0 x3 x4)
      = pretty x0 <+>
          text "PACKAGE" <+>
        text "BODY" <+> pretty x3 <+> pretty x4

instance PLSqlPP Package_declaration where
    pretty (Package_declaration_c1 x0) = pretty x0
    pretty (Package_declaration_c2 x0) = pretty x0

instance PLSqlPP Package_header where
    pretty (Package_header x0 x1 x2)
      = pretty x0 <+> pretty x1 <+> pretty x2

instance PLSqlPP Package_header_prefix where
    pretty (Package_header_prefix x0 x2 x3)
      = pretty x0 <+>
          text "PACKAGE" <+> pretty x2 <+> pretty x3

instance PLSqlPP Package_name where
    pretty (Package_name x0) = pretty x0

instance PLSqlPP Parallel_clause where
    pretty (Parallel_clause_c1) = text "NOPARALLEL"
    pretty (Parallel_clause_c2 x2)
      = text "PARALLEL" <+> text "(" <+> pretty x2 <+> text ")"

instance PLSqlPP Parallel_items where
    pretty (Parallel_items_c1 x1)
      = text "DEGREE" <+> pretty x1
    pretty (Parallel_items_c2 x1)
      = text "INSTANCES" <+> pretty x1
    pretty (Parallel_items_c3 x1 x2)
      = text "DEGREE" <+> pretty x1 <+> pretty x2

instance PLSqlPP Parameter where
    pretty (Parameter_c1 x0) = pretty x0
    pretty (Parameter_c2 x0 x2)
      = pretty x0 <+> text "=>" <+> pretty x2

instance PLSqlPP Parameter_declaration where
    pretty (Parameter_declaration x0 x1 x2 x3)
      = pretty x0 <+> hsep (map pretty x1) <+> pretty x2 <+> pretty x3

instance PLSqlPP Parameter_declarations where
    pretty (Parameter_declarations x0)
      = prettyList (text ",") x0

instance PLSqlPP Parameter_declarations_brackets where
    pretty (Parameter_declarations_brackets x1)
      = text "(" <+> pretty x1 <+> text ")"

instance PLSqlPP Parameters where
    pretty (Parameters x0) = prettyList (text ",") x0

instance PLSqlPP Parameters_brackets where
    pretty (Parameters_brackets x1)
      = text "(" <+> pretty x1 <+> text ")"

instance PLSqlPP Password where
    pretty (Password x0) = pretty x0

instance PLSqlPP Perc_found where
    pretty (Perc_found) = text "%FOUND"

instance PLSqlPP Perc_isopen where
    pretty (Perc_isopen) = text "%ISOPEN"

instance PLSqlPP Perc_notfound where
    pretty (Perc_notfound) = text "%NOTFOUND"

instance PLSqlPP Perc_rowcount where
    pretty (Perc_rowcount) = text "%ROWCOUNT"

instance PLSqlPP Perc_rowtype where
    pretty (Perc_rowtype) = text "%ROWTYPE"

instance PLSqlPP Perc_type where
    pretty (Perc_type) = text "%TYPE"

instance PLSqlPP Perc_typed_name where
    pretty (Perc_typed_name_c1 x0) = pretty x0
    pretty (Perc_typed_name_c2 x0 x2)
      = pretty x0 <+> text "." <+> pretty x2
    pretty (Perc_typed_name_c3 x0 x1)
      = pretty x0 <+> pretty x1
    pretty (Perc_typed_name_c4 x0 x2 x3)
      = pretty x0 <+> text "." <+> pretty x2 <+> pretty x3
    pretty (Perc_typed_name_c5 x0 x1)
      = pretty x0 <+> pretty x1
    pretty (Perc_typed_name_c6 x0 x2 x3)
      = pretty x0 <+> text "." <+> pretty x2 <+> pretty x3
    pretty (Perc_typed_name_c7 x0) = pretty x0

instance PLSqlPP Permission where
    pretty (Permission_c1) = text "READ" <+> text "ONLY"
    pretty (Permission_c2) = text "READ" <+> text "WRITE"

instance PLSqlPP Pl_statement where
    pretty (Pl_statement x0 x1) = pretty x0 <+> pretty x1

instance PLSqlPP Pl_statements where
    pretty (Pl_statements_c1 x0 x1) = pretty x0 <+> pretty x1
    pretty (Pl_statements_c2 x0 x1 x2)
      = pretty x0 <+> pretty x1 <+> pretty x2

instance PLSqlPP Plsql_block where
    pretty (Plsql_block x0 x1 x2)
      = pretty x0 <+> pretty x1 <+> pretty x2

instance PLSqlPP Position where
    pretty (Position_c1 x2 x4)
      = text "POSITION" <+>
          text "(" <+>
        pretty x2 <+> text ":" <+> pretty x4 <+> text ")"
    pretty (Position_c2 x1) = text "CONSTANT" <+> pretty x1

instance PLSqlPP Pragma_declaration where
    pretty (Pragma_declaration_c1 x3 x5)
      = text "PRAGMA" <+>
          text "EXCEPTION_INIT" <+>
        text "(" <+>
          pretty x3 <+> text "," <+> pretty x5 <+> text ")"
    pretty (Pragma_declaration_c2 x3 x5)
      = text "PRAGMA" <+>
          text "RESTRICT_REFERENCES" <+>
        text "(" <+>
          pretty x3 <+> text "," <+> pretty x5 <+> text ")"
    pretty (Pragma_declaration_c3)
      = text "PRAGMA" <+> text "AUTONOMOUS_TRANSACTION"

instance PLSqlPP Procedure_body_declaration where
    pretty (Procedure_body_declaration_c1 x0 x1 x2 x3 x4)
      = pretty x0 <+>
          pretty x1 <+> pretty x2 <+> pretty x3 <+> pretty x4
    pretty (Procedure_body_declaration_c2 x0 x1 x3 x4 x5)
      = pretty x0 <+>
          pretty x1 <+>
        text "BEGIN" <+> pretty x3 <+> pretty x4 <+> pretty x5
    pretty (Procedure_body_declaration_c3 x0 x1 x5)
      = pretty x0 <+>
          pretty x1 <+>
        text "LANGUAGE" <+>
          text "JAVA" <+> text "NAME" <+> pretty x5

instance PLSqlPP Procedure_header_declaration where
    pretty (Procedure_header_declaration x1 x2)
      = text "PROCEDURE" <+> pretty x1 <+> pretty x2

instance PLSqlPP Profile where
    pretty (Profile x0) = pretty x0

instance PLSqlPP Qualified_name where
    pretty (Qualified_name_c1 x0) = pretty x0
    pretty (Qualified_name_c2 x0) = pretty x0
    pretty (Qualified_name_c3 x0 x2)
      = pretty x0 <+> text "." <+> pretty x2
    pretty (Qualified_name_c4) = text "CONVERT"
    pretty (Qualified_name_c5) = text "PROFILE"
    pretty (Qualified_name_c6) = text "LANGUAGE"
    pretty (Qualified_name_c7) = text "NAME"
    pretty (Qualified_name_c8 x0)
      = pretty x0 <+> text "." <+> text "EXECUTE"

instance PLSqlPP Qualified_names where
    pretty (Qualified_names x0) = prettyList (text ",") x0

instance PLSqlPP Quota where
    pretty (Quota x1 x3)
      = text "QUOTA" <+> pretty x1 <+> text "ON" <+> pretty x3

instance PLSqlPP Quotas where
    pretty (Quotas x0) = hsep( map pretty x0)

instance PLSqlPP Raise_statement where
    pretty (Raise_statement x1) = text "RAISE" <+> pretty x1

instance PLSqlPP Rebuild_item where
    pretty (Rebuild_item_c1 x1)
      = text "PARALLEL" <+> pretty x1
    pretty (Rebuild_item_c2) = text "NOPARALLEL"
    pretty (Rebuild_item_c3) = text "RECOVERABLE"
    pretty (Rebuild_item_c4) = text "UNRECOVERABLE"
    pretty (Rebuild_item_c5 x0) = pretty x0

instance PLSqlPP Rebuild_items where
    pretty (Rebuild_items x0) = hsep( map pretty x0)

instance PLSqlPP Record_type_definition where
    pretty (Record_type_definition x1 x5)
      = text "TYPE" <+>
          pretty x1 <+>
        text "IS" <+>
          text "RECORD" <+> text "(" <+> pretty x5 <+> text ")"

instance PLSqlPP Recover_clause where
    pretty (Recover_clause x1 x2 x3 x4)
      = text "RECOVER" <+>
          pretty x1 <+> pretty x2 <+> pretty x3 <+> pretty x4

instance PLSqlPP Recover_database_item where
    pretty (Recover_database_item_c1)
      = text "UNTIL" <+> text "CANCEL"
    pretty (Recover_database_item_c2 x2)
      = text "UNTIL" <+> text "TIME" <+> pretty x2
    pretty (Recover_database_item_c3 x2)
      = text "UNTIL" <+> text "CHANGE" <+> pretty x2
    pretty (Recover_database_item_c4)
      = text "USING" <+> text "BACKUP" <+> text "CONTROLFILE"

instance PLSqlPP Recover_item where
    pretty (Recover_item_c1 x0 x1) = pretty x0 <+> pretty x1
    pretty (Recover_item_c2 x1)
      = text "TABLESPACE" <+> pretty x1
    pretty (Recover_item_c3 x1)
      = text "DATABASE" <+> pretty x1
    pretty (Recover_item_c4 x1)
      = text "LOGFILE" <+> pretty x1
    pretty (Recover_item_c5 x1)
      = text "CONTINUE" <+> pretty x1
    pretty (Recover_item_c6) = text "CANCEL"

instance PLSqlPP Ref_type_declaration where
    pretty (Ref_type_declaration x1 x5)
      = text "TYPE" <+>
          pretty x1 <+>
        text "IS" <+> text "REF" <+> text "CURSOR" <+> pretty x5

instance PLSqlPP Referencing where
    pretty (Referencing x1)
      = text "REFERENCING" <+> pretty x1

instance PLSqlPP Referencing_item where
    pretty (Referencing_item x0 x2)
      = pretty x0 <+> text "AS" <+> pretty x2

instance PLSqlPP Referencing_items where
    pretty (Referencing_items x0) = hsep( map pretty x0)

instance PLSqlPP Relational_operator where
    pretty (Relational_operator_c1) = text "<"
    pretty (Relational_operator_c2) = text "<="
    pretty (Relational_operator_c3) = text ">"
    pretty (Relational_operator_c4) = text ">="
    pretty (Relational_operator_c5) = text "="
    pretty (Relational_operator_c6) = text "<>"
    pretty (Relational_operator_c7) = text "!="

instance PLSqlPP Rename_statement where
    pretty (Rename_statement x1 x3)
      = text "RENAME" <+> pretty x1 <+> text "TO" <+> pretty x3

instance PLSqlPP Resource_cost_item where
    pretty (Resource_cost_item_c1 x1)
      = text "CPU_PER_SESSION" <+> pretty x1
    pretty (Resource_cost_item_c2 x1)
      = text "CONNECT_TIME" <+> pretty x1
    pretty (Resource_cost_item_c3 x1)
      = text "LOGICAL_READS_PER_SECOND" <+> pretty x1
    pretty (Resource_cost_item_c4 x1)
      = text "PRIVATE_SGA" <+> pretty x1

instance PLSqlPP Resource_cost_items where
    pretty (Resource_cost_items x0) = hsep( map pretty x0)

instance PLSqlPP Return_statement where
    pretty (Return_statement x1)
      = text "RETURN" <+> pretty x1

instance PLSqlPP Return_type where
    pretty (Return_type x0) = pretty x0

instance PLSqlPP Reused_filename where
    pretty (Reused_filename x0 x1) = pretty x0 <+> pretty x1

instance PLSqlPP Reused_filenames where
    pretty (Reused_filenames x0) = prettyList (text ",") x0

instance PLSqlPP Revoke_statement where
    pretty (Revoke_statement x1 x2 x4 x5)
      = text "REVOKE" <+>
          hsep( map pretty x1) <+>
        pretty x2 <+> text "FROM" <+> pretty x4 <+> pretty x5

instance PLSqlPP Role where
    pretty (Role x0) = pretty x0

instance PLSqlPP Role_description where
    pretty (Role_description_c1 x0) = pretty x0
    pretty (Role_description_c2) = text "ALL"
    pretty (Role_description_c3 x2)
      = text "ALL" <+> text "EXCEPT" <+> pretty x2
    pretty (Role_description_c4) = text "NONE"

instance PLSqlPP Role_item where
    pretty (Role_item_c1) = text "NOT" <+> text "IDENTIFIED"
    pretty (Role_item_c2 x1)
      = text "IDENTIFIED" <+> pretty x1

instance PLSqlPP Roles where
    pretty (Roles x0) = hsep( map pretty x0)

instance PLSqlPP Rollback_segment_item where
    pretty (Rollback_segment_item_c1) = text "ONLINE"
    pretty (Rollback_segment_item_c2) = text "OFFLINE"
    pretty (Rollback_segment_item_c3 x0) = pretty x0
    pretty (Rollback_segment_item_c4 x1 x2)
      = text "SHRINK" <+> pretty x1 <+> pretty x2

instance PLSqlPP Rollback_statement where
    pretty (Rollback_statement x1 x2)
      = text "ROLLBACK" <+> pretty x1 <+> pretty x2

instance PLSqlPP Row_clause where
    pretty (Row_clause x0 x4)
      = pretty x0 <+>
          text "FOR" <+> text "EACH" <+> text "ROW" <+> pretty x4

instance PLSqlPP Savepoint where
    pretty (Savepoint_c1 x1 x2)
      = text "TO" <+> pretty x1 <+> pretty x2
    pretty (Savepoint_c2 x1) = text "FORCE" <+> pretty x1
    pretty (Savepoint_c3) = text "RELEASE"

instance PLSqlPP Savepoint_statement where
    pretty (Savepoint_statement x1)
      = text "SAVEPOINT" <+> pretty x1

instance PLSqlPP Schema_qualifier where
    pretty (Schema_qualifier x0) = pretty x0

instance PLSqlPP Select_clause where
    pretty (Select_clause_c1 x0) = pretty x0
    pretty (Select_clause_c2 x0) = pretty x0
    pretty (Select_clause_c3 x0) = pretty x0
    pretty (Select_clause_c4 x0) = pretty x0
    pretty (Select_clause_c5 x0) = pretty x0

instance PLSqlPP Select_clause_operator where
    pretty (Select_clause_operator_c1) = text "UNION"
    pretty (Select_clause_operator_c2)
      = text "UNION" <+> text "ALL"
    pretty (Select_clause_operator_c3) = text "INTERSECT"
    pretty (Select_clause_operator_c4) = text "MINUS"

instance PLSqlPP Select_item where
    pretty (Select_item_c1 x0 x1) = pretty x0 <+> pretty x1
    pretty (Select_item_c2 x0 x1) = pretty x0 <+> pretty x1
    pretty (Select_item_c3 x0) = pretty x0

instance PLSqlPP Select_statement where
    pretty (Select_statement x1 x2 x3 x5 x6)
      = text "SELECT" <+>
          pretty x1 <+>
        pretty x2 <+>
          pretty x3 <+> text "FROM" <+> pretty x5 <+> hsep( map pretty x6)

instance PLSqlPP Select_statement_without_into where
    pretty (Select_statement_without_into_c1 x1 x2 x4 x5)
      = text "SELECT" <+>
          pretty x1 <+>
        pretty x2 <+> text "FROM" <+> pretty x4 <+> hsep( map pretty x5)
    pretty (Select_statement_without_into_c2 x0 x1 x2)
      = pretty x0 <+> pretty x1 <+> pretty x2

instance PLSqlPP Sequence_item where
    pretty (Sequence_item_c1 x2)
      = text "INCREMENT" <+> text "BY" <+> pretty x2
    pretty (Sequence_item_c2 x2)
      = text "START" <+> text "WITH" <+> pretty x2
    pretty (Sequence_item_c3 x1)
      = text "MAXVALUE" <+> pretty x1
    pretty (Sequence_item_c4) = text "NOMAXVALUE"
    pretty (Sequence_item_c5 x1)
      = text "MINVALUE" <+> pretty x1
    pretty (Sequence_item_c6) = text "NOMINVALUE"
    pretty (Sequence_item_c7) = text "CYCLE"
    pretty (Sequence_item_c8) = text "NOCYCLE"
    pretty (Sequence_item_c9 x1) = text "CACHE" <+> pretty x1
    pretty (Sequence_item_c10) = text "NOCACHE"
    pretty (Sequence_item_c11) = text "ORDER"
    pretty (Sequence_item_c12) = text "NOORDER"

instance PLSqlPP Sequence_items where
    pretty (Sequence_items x0) = hsep( map pretty x0)

instance PLSqlPP Session_set_item where
    pretty (Session_set_item_c1 x0 x2)
      = pretty x0 <+> text "=" <+> pretty x2
    pretty (Session_set_item_c2 x2)
      = text "NLS_LANGUAGE" <+> text "=" <+> pretty x2
    pretty (Session_set_item_c3 x2)
      = text "NLS_TERRITORY" <+> text "=" <+> pretty x2
    pretty (Session_set_item_c4 x2)
      = text "NLS_DATE_FORMAT" <+> text "=" <+> pretty x2
    pretty (Session_set_item_c5 x2)
      = text "NLS_DATE_LANGUAGE" <+> text "=" <+> pretty x2
    pretty (Session_set_item_c6 x2)
      = text "NLS_NUMERIC_CHARACTERS" <+>
          text "=" <+> pretty x2
    pretty (Session_set_item_c7 x2)
      = text "NLS_ISO_CURRENCY" <+> text "=" <+> pretty x2
    pretty (Session_set_item_c8 x2)
      = text "NLS_CURRENCY" <+> text "=" <+> pretty x2
    pretty (Session_set_item_c9 x2)
      = text "NLS_SORT" <+> text "=" <+> pretty x2
    pretty (Session_set_item_c10 x2)
      = text "NLS_CALENDAR" <+> text "=" <+> pretty x2
    pretty (Session_set_item_c11 x2)
      = text "LABEL" <+> text "=" <+> pretty x2
    pretty (Session_set_item_c12 x2)
      = text "MLS_LABEL_FORMAT" <+> text "=" <+> pretty x2
    pretty (Session_set_item_c13 x2)
      = text "OPTIMIZER_GOAL" <+> text "=" <+> pretty x2
    pretty (Session_set_item_c14 x2)
      = text "FLAGGER" <+> text "=" <+> pretty x2
    pretty (Session_set_item_c15 x2)
      = text "SESSION_CACHED_OPEN_CURSORS" <+>
          text "=" <+> pretty x2
    pretty (Session_set_item_c16 x2)
      = text "INSTANCE" <+> text "=" <+> pretty x2
    pretty (Session_set_item_c17 x2)
      = text "HASH_JOIN_ENABLED" <+> text "=" <+> pretty x2
    pretty (Session_set_item_c18 x2)
      = text "HASH_AREA_SIZE" <+> text "=" <+> pretty x2
    pretty (Session_set_item_c19 x2)
      = text "HASH_MULTIBLOCK_IO_COUNT" <+>
          text "=" <+> pretty x2
    pretty (Session_set_item_c20 x2)
      = text "ISOLATION_LEVEL" <+> text "=" <+> pretty x2
    pretty (Session_set_item_c21 x1)
      = text "EVENTS" <+> pretty x1

instance PLSqlPP Session_set_items where
    pretty (Session_set_items x0) = hsep( map pretty x0)

instance PLSqlPP Set_flagger_item where
    pretty (Set_flagger_item_c1) = text "ENTRY"
    pretty (Set_flagger_item_c2) = text "INTERMEDIATE"
    pretty (Set_flagger_item_c3) = text "FULL"
    pretty (Set_flagger_item_c4) = text "OFF"

instance PLSqlPP Set_item where
    pretty (Set_item_c1 x0 x2)
      = pretty x0 <+> text "=" <+> pretty x2
    pretty (Set_item_c2 x0 x2)
      = pretty x0 <+> text "=" <+> pretty x2

instance PLSqlPP Set_items where
    pretty (Set_items x0) = prettyList (text ",") x0

instance PLSqlPP Set_label_item where
    pretty (Set_label_item_c1 x0) = pretty x0
    pretty (Set_label_item_c2) = text "DBHIGH"
    pretty (Set_label_item_c3) = text "DBLOW"

instance PLSqlPP Set_optimizer_goal_item where
    pretty (Set_optimizer_goal_item_c1) = text "ALL_ROWS"
    pretty (Set_optimizer_goal_item_c2) = text "FIRST_ROWS"
    pretty (Set_optimizer_goal_item_c3) = text "RULE"
    pretty (Set_optimizer_goal_item_c4) = text "CHOOSE"

instance PLSqlPP Set_statement_id where
    pretty (Set_statement_id x2)
      = text "SET" <+> text "STATEMENT_ID" <+> pretty x2

instance PLSqlPP Set_transaction_clause where
    pretty (Set_transaction_clause_c1 x0) = pretty x0
    pretty (Set_transaction_clause_c2 x0) = pretty x0
    pretty (Set_transaction_clause_c3 x3)
      = text "USE" <+>
          text "ROLLBACK" <+> text "SEGMENT" <+> pretty x3

instance PLSqlPP Set_transaction_statement where
    pretty (Set_transaction_statement x1 x3)
      = text "SET" <+>
          pretty x1 <+> text "TRANSACTION" <+> pretty x3

instance PLSqlPP Snap_shot_item where
    pretty (Snap_shot_item_c1 x1)
      = text "PCTFREE" <+> pretty x1
    pretty (Snap_shot_item_c2 x1)
      = text "PCTUSED" <+> pretty x1
    pretty (Snap_shot_item_c3 x1)
      = text "INITRANS" <+> pretty x1
    pretty (Snap_shot_item_c4 x1)
      = text "MAXTRANS" <+> pretty x1
    pretty (Snap_shot_item_c5 x0) = pretty x0
    pretty (Snap_shot_item_c6 x0) = pretty x0

instance PLSqlPP Snap_shot_items where
    pretty (Snap_shot_items x0) = hsep( map pretty x0)

instance PLSqlPP Snapshot_refresh where
    pretty (Snapshot_refresh x1 x2 x3)
      = text "REFRESH" <+>
          pretty x1 <+> pretty x2 <+> pretty x3

instance PLSqlPP Sql_plus_identifier where
    pretty (Sql_plus_identifier x0) = text x0

instance PLSqlPP Sql_plus_macro where
    pretty (Sql_plus_macro x0) = pretty x0

instance PLSqlPP Sql_plus_statement where
    pretty (Sql_plus_statement_c1 x1)
      = text "SPOOL" <+> pretty x1
    pretty (Sql_plus_statement_c2)
      = text "SPOOL" <+> text "OFF"
    pretty (Sql_plus_statement_c3 x1 x2)
      = text "TTITLE" <+> pretty x1 <+> pretty x2
    pretty (Sql_plus_statement_c4 x1)
      = text "BTITLE" <+> pretty x1
    pretty (Sql_plus_statement_c5 x1 x3)
      = text "DEFINE" <+> pretty x1 <+> text "=" <+> pretty x3
    pretty (Sql_plus_statement_c6 x1 x3)
      = text "DEFINE" <+> pretty x1 <+> text "=" <+> pretty x3
    pretty (Sql_plus_statement_c7 x1 x3)
      = text "DEFINE" <+> pretty x1 <+> text "=" <+> pretty x3
    pretty (Sql_plus_statement_c8 x1)
      = text "UNDEFINE" <+> pretty x1
    pretty (Sql_plus_statement_c9 x1)
      = text "UNDEFINE" <+> pretty x1
    pretty (Sql_plus_statement_c10 x1)
      = text "SET" <+> pretty x1
    pretty (Sql_plus_statement_c11 x1)
      = text "CLEAR" <+> pretty x1
    pretty (Sql_plus_statement_c12 x2)
      = text "WHENEVER" <+> text "SQLERROR" <+> pretty x2
    pretty (Sql_plus_statement_c13 x1)
      = text "START" <+> hsep( map pretty x1)
    pretty (Sql_plus_statement_c14 x1 x2)
      = text "COL" <+> pretty x1 <+> pretty x2
    pretty (Sql_plus_statement_c15 x1 x2)
      = text "COLUMN" <+> pretty x1 <+> pretty x2
    pretty (Sql_plus_statement_c16 x1)
      = text "BREAK" <+> hsep( map pretty x1)
    pretty (Sql_plus_statement_c17 x1 x3)
      = text "ACCEPT" <+>
          pretty x1 <+> text "PROMPT" <+> pretty x3
    pretty (Sql_plus_statement_c18 x1)
      = text "COMPUTE" <+> hsep( map pretty x1)
    pretty (Sql_plus_statement_c19 x2)
      = text "OPTIONS" <+> text "(" <+> pretty x2 <+> text ")"
    pretty (Sql_plus_statement_c20) = text "SQL" <+> text ">"
    pretty (Sql_plus_statement_c21)
      = text "SQL" <+> text ">" <+> text "/"
    pretty (Sql_plus_statement_c22) = text "TEXT"
    pretty (Sql_plus_statement_c23 x0)
      = pretty x0 <+>
          text "ROWS" <+> text "SELECTED" <+> text "."

instance PLSqlPP Sql_plus_value where
    pretty (Sql_plus_value x0) = text x0

instance PLSqlPP Sql_plus_value_or_text where
    pretty (Sql_plus_value_or_text_c1 x0) = pretty x0
    pretty (Sql_plus_value_or_text_c2 x0) = pretty x0

instance PLSqlPP Sql_statement where
    pretty (Sql_statement_c1 x0) = pretty x0
    pretty (Sql_statement_c2 x0) = pretty x0
    pretty (Sql_statement_c3 x0) = pretty x0
    pretty (Sql_statement_c4 x0) = pretty x0
    pretty (Sql_statement_c5 x0) = pretty x0
    pretty (Sql_statement_c6 x0) = pretty x0
    pretty (Sql_statement_c7 x0) = pretty x0
    pretty (Sql_statement_c8 x0) = pretty x0
    pretty (Sql_statement_c9 x0) = pretty x0
    pretty (Sql_statement_c10 x0) = pretty x0
    pretty (Sql_statement_c11 x0) = pretty x0
    pretty (Sql_statement_c12 x0) = pretty x0
    pretty (Sql_statement_c13 x0) = pretty x0
    pretty (Sql_statement_c14 x0) = pretty x0
    pretty (Sql_statement_c15 x0) = pretty x0
    pretty (Sql_statement_c16 x0) = pretty x0
    pretty (Sql_statement_c17 x0) = pretty x0
    pretty (Sql_statement_c18 x0) = pretty x0
    pretty (Sql_statement_c19 x0) = pretty x0
    pretty (Sql_statement_c20 x0) = pretty x0
    pretty (Sql_statement_c21 x0) = pretty x0
    pretty (Sql_statement_c22 x0) = pretty x0
    pretty (Sql_statement_c23 x0) = pretty x0
    pretty (Sql_statement_c24 x0) = pretty x0
    pretty (Sql_statement_c25 x0) = pretty x0
    pretty (Sql_statement_c26 x0) = pretty x0
    pretty (Sql_statement_c27 x0) = pretty x0
    pretty (Sql_statement_c28 x0) = pretty x0
    pretty (Sql_statement_c29 x0) = pretty x0
    pretty (Sql_statement_c30 x0) = pretty x0
    pretty (Sql_statement_c31 x0) = pretty x0
    pretty (Sql_statement_c32 x0) = pretty x0
    pretty (Sql_statement_c33 x0) = pretty x0
    pretty (Sql_statement_c34 x0) = pretty x0
    pretty (Sql_statement_c35 x0) = pretty x0
    pretty (Sql_statement_c36 x0) = pretty x0
    pretty (Sql_statement_c37 x0) = pretty x0
    pretty (Sql_statement_c38 x0) = pretty x0
    pretty (Sql_statement_c39 x0) = pretty x0
    pretty (Sql_statement_c40 x0) = pretty x0
    pretty (Sql_statement_c41 x0) = pretty x0
    pretty (Sql_statement_c42 x0) = pretty x0
    pretty (Sql_statement_c43 x0) = pretty x0
    pretty (Sql_statement_c44 x0) = pretty x0
    pretty (Sql_statement_c45 x0) = pretty x0
    pretty (Sql_statement_c46 x0) = pretty x0
    pretty (Sql_statement_c47 x0) = pretty x0
    pretty (Sql_statement_c48 x0) = pretty x0
    pretty (Sql_statement_c49 x0) = pretty x0
    pretty (Sql_statement_c50 x0) = pretty x0
    pretty (Sql_statement_c51 x0) = pretty x0
    pretty (Sql_statement_c52 x0) = pretty x0
    pretty (Sql_statement_c53 x0) = pretty x0
    pretty (Sql_statement_c54 x0) = pretty x0
    pretty (Sql_statement_c55 x0) = pretty x0
    pretty (Sql_statement_c56 x0) = pretty x0
    pretty (Sql_statement_c57 x0) = pretty x0
    pretty (Sql_statement_c58 x0) = pretty x0
    pretty (Sql_statement_c59 x0) = pretty x0
    pretty (Sql_statement_c60 x0) = pretty x0
    pretty (Sql_statement_c61 x0) = pretty x0

instance PLSqlPP Start_date where
    pretty (Start_date x2)
      = text "START" <+> text "WITH" <+> pretty x2

instance PLSqlPP Statement_block where
    pretty (Statement_block x0 x2)
      = pretty x0 <+> text "END" <+> pretty x2

instance PLSqlPP Statement_block_no_end where
    pretty (Statement_block_no_end x1 x2)
      = text "BEGIN" <+> pretty x1 <+> pretty x2

instance PLSqlPP Storage_clause where
    pretty (Storage_clause x1) = text "STORAGE" <+> pretty x1

instance PLSqlPP Storage_item where
    pretty (Storage_item_c1 x1)
      = text "INITIAL" <+> pretty x1
    pretty (Storage_item_c2 x0) = pretty x0
    pretty (Storage_item_c3 x1)
      = text "MINEXTENTS" <+> pretty x1
    pretty (Storage_item_c4 x1)
      = text "MAXEXTENTS" <+> pretty x1
    pretty (Storage_item_c5 x1)
      = text "PCTINCREASE" <+> pretty x1
    pretty (Storage_item_c6 x1)
      = text "FREELISTS" <+> pretty x1
    pretty (Storage_item_c7 x2)
      = text "FREELIST" <+> text "GROUPS" <+> pretty x2
    pretty (Storage_item_c8 x1)
      = text "OPTIMAL" <+> pretty x1

instance PLSqlPP Storage_items where
    pretty (Storage_items x0) = hsep( map pretty x0)

instance PLSqlPP Storage_items_opt_brackets where
    pretty (Storage_items_opt_brackets_c1 x0) = pretty x0
    pretty (Storage_items_opt_brackets_c2 x1)
      = text "(" <+> pretty x1 <+> text ")"

instance PLSqlPP Subquery where
    pretty (Subquery x0) = pretty x0

instance PLSqlPP Subquery_brackets where
    pretty (Subquery_brackets x1)
      = text "(" <+> pretty x1 <+> text ")"

instance PLSqlPP Subquery_opt_brackets where
    pretty (Subquery_opt_brackets_c1 x1)
      = text "(" <+> pretty x1 <+> text ")"
    pretty (Subquery_opt_brackets_c2 x0) = pretty x0

instance PLSqlPP Symbol_dot where
    pretty (Symbol_dot) = text "."

instance PLSqlPP System_item where
    pretty (System_item_c1 x0)
      = pretty x0 <+> text "RESTRICTED" <+> text "SESSION"
    pretty (System_item_c2)
      = text "FLUSH" <+> text "SHARED_POOL"
    pretty (System_item_c3 x0 x1) = pretty x0 <+> pretty x1
    pretty (System_item_c4 x1) = text "SET" <+> pretty x1
    pretty (System_item_c5)
      = text "SWITCH" <+> text "LOGFILE"
    pretty (System_item_c6 x0)
      = pretty x0 <+> text "DISTRIBUTED" <+> text "RECOVERY"
    pretty (System_item_c7 x0) = pretty x0
    pretty (System_item_c8 x3 x5)
      = text "KILL" <+>
          text "SESSION" <+>
        text "'" <+>
          pretty x3 <+> text "," <+> pretty x5 <+> text "'"

instance PLSqlPP System_items where
    pretty (System_items x0) = hsep( map pretty x0)

instance PLSqlPP System_set_item where
    pretty (System_set_item_c1 x2)
      = text "RESOURCE_LIMIT" <+> text "=" <+> pretty x2
    pretty (System_set_item_c2 x2)
      = text "GLOBAL_NAMES" <+> text "=" <+> pretty x2
    pretty (System_set_item_c3 x2)
      = text "SCAN_INSTANCES" <+> text "=" <+> pretty x2
    pretty (System_set_item_c4 x2)
      = text "CACHE_INSTANCES" <+> text "=" <+> pretty x2
    pretty (System_set_item_c5 x2)
      = text "MTS_SERVERS" <+> text "=" <+> pretty x2
    pretty (System_set_item_c6 x3 x5)
      = text "MTS_DISPATCHERS" <+>
          text "=" <+>
        text "'" <+>
          pretty x3 <+> text "," <+> pretty x5 <+> text "'"
    pretty (System_set_item_c7 x2)
      = text "LICENSE_MAX_SESSIONS" <+> text "=" <+> pretty x2
    pretty (System_set_item_c8 x2)
      = text "LICENSE_SESSIONS_WARNING" <+>
          text "=" <+> pretty x2
    pretty (System_set_item_c9 x2)
      = text "LICENSE_MAX_USERS" <+> text "=" <+> pretty x2
    pretty (System_set_item_c10 x2)
      = text "REMOTE_DEPENDENCIES_MODE" <+>
          text "=" <+> pretty x2

instance PLSqlPP System_set_items where
    pretty (System_set_items x0) = hsep( map pretty x0)

instance PLSqlPP Table_description where
    pretty (Table_description_c1 x0) = pretty x0
    pretty (Table_description_c2 x0) = pretty x0

instance PLSqlPP Table_item where
    pretty (Table_item_c1 x0 x1 x2 x3)
      = pretty x0 <+> pretty x1 <+> pretty x2 <+> pretty x3
    pretty (Table_item_c2 x0) = pretty x0

instance PLSqlPP Table_items where
    pretty (Table_items x0) = prettyList (text ",") x0

instance PLSqlPP Table_items_opt_brackets where
    pretty (Table_items_opt_brackets_c1 x1)
      = text "(\n\t\t" <+> pretty x1 <+> text "\n)"
    pretty (Table_items_opt_brackets_c2 x0) = pretty x0

instance PLSqlPP Table_modify_item where
    pretty (Table_modify_item x0 x1 x2 x3)
      = pretty x0 <+> pretty x1 <+> pretty x2 <+> pretty x3

instance PLSqlPP Table_modify_items where
    pretty (Table_modify_items x0) = hsep( map pretty x0)

instance PLSqlPP Table_type_declaration where
    pretty (Table_type_declaration x1 x5 x6)
      = text "TYPE" <+>
          pretty x1 <+>
        text "IS" <+>
          text "TABLE" <+>
            text "OF" <+>
              pretty x5 <+>
            pretty x6 <+>
              text "INDEX" <+> text "BY" <+> text "BINARY_INTEGER"

instance PLSqlPP Tablespace where
    pretty (Tablespace x0) = pretty x0

instance PLSqlPP Tablespace_clause where
    pretty (Tablespace_clause x1 x2)
      = text "TABLESPACE" <+> pretty x1 <+> pretty x2

instance PLSqlPP Tablespace_item where
    pretty (Tablespace_item_c1 x2)
      = text "ADD" <+> text "DATAFILE" <+> pretty x2
    pretty (Tablespace_item_c2 x0) = pretty x0
    pretty (Tablespace_item_c3) = text "COALESCE"
    pretty (Tablespace_item_c4 x1)
      = text "DEFAULT" <+> pretty x1
    pretty (Tablespace_item_c5) = text "ONLINE"
    pretty (Tablespace_item_c6 x1)
      = text "OFFLINE" <+> pretty x1
    pretty (Tablespace_item_c7)
      = text "BEGIN" <+> text "BACKUP"
    pretty (Tablespace_item_c8)
      = text "END" <+> text "BACKUP"
    pretty (Tablespace_item_c9) = text "READ" <+> text "ONLY"
    pretty (Tablespace_item_c10)
      = text "READ" <+> text "WRITE"
    pretty (Tablespace_item_c11) = text "PERMANENT"
    pretty (Tablespace_item_c12) = text "TEMPORARY"

instance PLSqlPP Tablespace_items where
    pretty (Tablespace_items x0) = hsep( map pretty x0)

instance PLSqlPP Territory where
    pretty (Territory x0) = pretty x0

instance PLSqlPP Text where
    pretty (Text_c1 x0) = text x0
    pretty (Text_c2) = text "ESCAPE"

instance PLSqlPP Then_branch where
    pretty (Then_branch x1) = text "THEN" <+> pretty x1

instance PLSqlPP Thread_integer where
    pretty (Thread_integer x1) = text "THREAD" <+> pretty x1

instance PLSqlPP To_location where
    pretty (To_location x1) = text "TO" <+> pretty x1

instance PLSqlPP Top_expression where
    pretty (Top_expression_c1 x0) = pretty x0
    pretty (Top_expression_c2 x0) = pretty x0

instance PLSqlPP Top_sort where
    pretty (Top_sort_c1 x0) = pretty x0
    pretty (Top_sort_c2 x0) = pretty x0

instance PLSqlPP Top_sort_cobol_embedded_sql where
    pretty (Top_sort_cobol_embedded_sql x0 x1)
      = prettyList (text ".") x0 <+> pretty x1

instance PLSqlPP Top_sort_pl_sql where
    pretty (Top_sort_pl_sql x0) = hsep( map pretty x0)

instance PLSqlPP Trailing_clause where
    pretty (Trailing_clause)
      = text "TRAILING" <+> text "NULLCOLS"

instance PLSqlPP Trailing_declarations where
    pretty (Trailing_declarations x0 x2)
      = pretty x0 <+> text "END" <+> pretty x2

instance PLSqlPP Trigger_item where
    pretty (Trigger_item_c1) = text "DELETE"
    pretty (Trigger_item_c2) = text "INSERT"
    pretty (Trigger_item_c3 x1) = text "UPDATE" <+> pretty x1

instance PLSqlPP Trigger_items where
    pretty (Trigger_items x0) = prettyList (text "OR") x0

instance PLSqlPP Truncate_clause where
    pretty (Truncate_clause_c1)
      = text "DROP" <+> text "STORAGE"
    pretty (Truncate_clause_c2) = text "REUSE"

instance PLSqlPP Truncate_statement where
    pretty (Truncate_statement x1 x2 x3)
      = text "TRUNCATE" <+>
          pretty x1 <+> pretty x2 <+> pretty x3

instance PLSqlPP Ttitle_date where
    pretty (Ttitle_date_c1) = text "DATE"
    pretty (Ttitle_date_c2) = text "DATEVAR"

instance PLSqlPP Ttitle_item where
    pretty (Ttitle_item_c1) = text "ON"
    pretty (Ttitle_item_c2) = text "OFF"
    pretty (Ttitle_item_c3 x1)
      = text "CENTER" <+> hsep( map pretty x1)
    pretty (Ttitle_item_c4 x1)
      = text "SKIP" <+> hsep( map pretty x1)
    pretty (Ttitle_item_c5 x1 x2)
      = text "LEFT" <+> pretty x1 <+> hsep( map pretty x2)
    pretty (Ttitle_item_c6 x1 x2)
      = text "RIGHT" <+> pretty x1 <+> hsep( map pretty x2)

instance PLSqlPP Ttitle_items where
    pretty (Ttitle_items x0) = hsep( map pretty x0)

instance PLSqlPP Type_name where
    pretty (Type_name x0) = pretty x0

instance PLSqlPP Unlabelled_pl_statement where
    pretty (Unlabelled_pl_statement_c1 x0) = pretty x0
    pretty (Unlabelled_pl_statement_c2 x0) = pretty x0
    pretty (Unlabelled_pl_statement_c3 x0) = pretty x0
    pretty (Unlabelled_pl_statement_c4 x0) = pretty x0
    pretty (Unlabelled_pl_statement_c5 x0) = pretty x0
    pretty (Unlabelled_pl_statement_c6 x0) = pretty x0
    pretty (Unlabelled_pl_statement_c7 x0) = pretty x0
    pretty (Unlabelled_pl_statement_c8 x0) = pretty x0
    pretty (Unlabelled_pl_statement_c9 x0) = pretty x0
    pretty (Unlabelled_pl_statement_c10 x0) = pretty x0
    pretty (Unlabelled_pl_statement_c11 x0) = pretty x0
    pretty (Unlabelled_pl_statement_c12 x0) = pretty x0
    pretty (Unlabelled_pl_statement_c13 x0) = pretty x0
    pretty (Unlabelled_pl_statement_c14 x0) = pretty x0

instance PLSqlPP Update_statement where
    pretty (Update_statement x1 x3 x4)
      = text "UPDATE" <+>
          pretty x1 <+> text "SET" <+> pretty x3 <+> pretty x4

instance PLSqlPP User where
    pretty (User x0) = pretty x0

instance PLSqlPP User_identification where
    pretty (User_identification_c1 x1)
      = text "BY" <+> pretty x1
    pretty (User_identification_c2) = text "EXTERNALLY"

instance PLSqlPP User_item where
    pretty (User_item_c1 x1)
      = text "IDENTIFIED" <+> pretty x1
    pretty (User_item_c2 x2)
      = text "DEFAULT" <+> text "TABLESPACE" <+> pretty x2
    pretty (User_item_c3 x2)
      = text "TEMPORARY" <+> text "TABLESPACE" <+> pretty x2
    pretty (User_item_c4 x0) = pretty x0
    pretty (User_item_c5 x1) = text "PROFILE" <+> pretty x1
    pretty (User_item_c6 x2)
      = text "DEFAULT" <+> text "ROLE" <+> pretty x2

instance PLSqlPP User_items where
    pretty (User_items x0) = hsep( map pretty x0)

instance PLSqlPP Using_index_clause where
    pretty (Using_index_clause x2)
      = text "USING" <+> text "INDEX" <+> pretty x2

instance PLSqlPP Using_index_item where
    pretty (Using_index_item_c1 x1)
      = text "INITRANS" <+> pretty x1
    pretty (Using_index_item_c2 x1)
      = text "MAXTRANS" <+> pretty x1
    pretty (Using_index_item_c3 x0) = pretty x0
    pretty (Using_index_item_c4 x0) = pretty x0
    pretty (Using_index_item_c5 x1)
      = text "PCTFREE" <+> pretty x1
    pretty (Using_index_item_c6 x1)
      = text "PCTUSED" <+> pretty x1

instance PLSqlPP Values_description where
    pretty (Values_description_c1 x2)
      = text "VALUES" <+> text "(" <+> pretty x2 <+> text ")"
    pretty (Values_description_c2 x0) = pretty x0
    pretty (Values_description_c3 x0) = pretty x0

instance PLSqlPP Var_declaration where
    pretty (Var_declaration x0 x1 x2)
      = pretty x0 <+> pretty x1 <+> pretty x2

instance PLSqlPP View_with_clause where
    pretty (View_with_clause_c1) = text "WITH"
    pretty (View_with_clause_c2)
      = text "WITH" <+> text "READ" <+> text "ONLY"
    pretty (View_with_clause_c3 x3)
      = text "WITH" <+>
          text "CHECK" <+> text "OPTION" <+> pretty x3

instance PLSqlPP When_clause where
    pretty (When_clause x1) = text "WHEN" <+> pretty x1

instance PLSqlPP Whenever_sqlerror_clause where
    pretty (Whenever_sqlerror_clause_c1) = text "CONTINUE"
    pretty (Whenever_sqlerror_clause_c2)
      = text "EXIT" <+> text "ROLLBACK"
    pretty (Whenever_sqlerror_clause_c3)
      = text "EXIT" <+> text "FAILURE"

instance PLSqlPP Whenever_sqlerror_statement where
    pretty (Whenever_sqlerror_statement_c1 x3)
      = text "WHENEVER" <+>
          text "SQLERROR" <+> text "GOTO" <+> pretty x3
    pretty (Whenever_sqlerror_statement_c2)
      = text "WHENEVER" <+> text "SQLERROR" <+> text "CONTINUE"

instance PLSqlPP Where_clause where
    pretty (Where_clause x1) = text "WHERE" <+> pretty x1

instance PLSqlPP While_loop_statement where
    pretty (While_loop_statement x1 x2)
      = text "WHILE" <+> pretty x1 <+> pretty x2

instance PLSqlPP Lex_name where
    pretty (Lex_identifier x0) = text x0
    pretty (Lex_quoted_name x0) = text x0
    pretty (Lex_ampersand_identifier x0) = text x0
    pretty (REF) = text "REF"
    pretty (Lex_name_kw x0) = text x0

instance PLSqlPP Lex_qualified_name where
    pretty (Lex_name x0) = pretty x0
    pretty (Lex_name_Lex_dot_Lex_name x0 x1 x2)
      = pretty x0 <+> text x1 <+> pretty x2
    pretty (Lex_name_Lex_dot_Lex_name_Lex_dot_Lex_name x0 x1 x2 x3 x4)
      = pretty x0 <+>
          text x1 <+> pretty x2 <+> text x3 <+> pretty x4
    pretty (Lex_name_Lex_at_Lex_name x0 x1 x2)
      = pretty x0 <+> text x1 <+> pretty x2
    pretty (Lex_name_Lex_dot_Lex_name_Lex_at_Lex_name x0 x1 x2 x3 x4)
      = pretty x0 <+>
          text x1 <+> pretty x2 <+> text x3 <+> pretty x4

instance PLSqlPP Lex_host_name where
    pretty (Lex_host_identifier x1) = text ":" <+> text x1

instance PLSqlPP Lex_qualified_host_name where
    pretty (Lex_host_name_p x0) = hsep( map pretty x0)
    pretty (Lex_host_name_Lex_dot_Lex_host_identifier x0 x1 x2)
      = pretty x0 <+> text x1 <+> text x2
    pretty (Lex_host_name_Lex_host_identifier x0 x3)
      = pretty x0 <+>
          text "[" <+> text ":" <+> text x3 <+> text "]"


instance PLSqlPP Lex_schema_qualifier where
    pretty (Lex_name_Lex_dot_star x0 x1)
      = pretty x0 <+> text x1
    pretty (Lex_name_Lex_dot_Lex_name_Lex_dot_star x0 x1 x2 x3)
      = pretty x0 <+> text x1 <+> pretty x2 <+> text x3

instance PLSqlPP Kw_k_m where
    pretty (K) = text "K"
    pretty (M) = text "M"

instance PLSqlPP Lex_memory_size where
    pretty (Lex_memory_size_int_Kw_k_m_opt x0 x1)
      = text x0 <+> pretty x1

instance PLSqlPP Lex_sql_plus_macro where
    pretty (Lex_at_Lex_at_Lex_sql_plus_identifier x0 x1 x2)
      = text x0 <+> text x1 <+> text x2
