-------------------------------------------------------------------------------
--
-- | 
-- Module      :  Language.VDM_SL.Optimize
-- Copyright   :  (c) Tiago Alves, Paulo Silva, 2004
-- License     :  LGPL
-- 
-- Maintainer  :  joost.visser@di.uminho.pt
-- Stability   :  experimental
-- Portability :  portable
--
-- This module defines optimizing transformations for VDM_SL.
--
-------------------------------------------------------------------------------

module Language.VDM_SL.Optimize where

import Language.VDM_SL.Syntax
import Language.VDM_SL.SyntaxTermInstances
import Control.Monad.Identity
import Language.VDM_SL.HLStratLib
import Language.VDM_SL.FuncLib
import Language.VDM_SL.AuxFuncLib
import TermRep

-- | ??
optimizeTables :: [Field] -> [Field]
optimizeTables lst =
   let uniqueKeys = collectUniqueKeys lst
       keys = [ (name, pk) | (name, pk) <- uniqueKeys]
       tables = map (\(x,y) -> (x, (y, func y))) keys
       func y = collectRng y lst
   in [ FIELD_NAME name (MAP_TYPE pk (unifyCompose dom)) 
        | (name, (pk, dom)) <- tables ]

-- | ??
unifyCompose :: [Type] -> Type
unifyCompose tlst = foldr composeFusion (head tlst) (tail tlst)

-- | Function that makes the fusion of two composes
--
-- Case 1: @ (A1 * A2 * ...) nil = (A1 * A2 * ...) @
--
-- Case 2: @ nil (B1 * B2 * ...) = (B1 * B2 * ...) @
-- 
-- Case 3: @ (A1 * A2 * ...) (B1 * B2 * ...) = (A1 * A2 * ... * B1 * B2 * ...) @
composeFusion :: Type -> Type -> Type
composeFusion (NIL_TYPE) x = x
composeFusion x (NIL_TYPE) = x
composeFusion (COMPOSE name1 (FIELD_LIST lst1)) (COMPOSE name2 (FIELD_LIST lst2)) =
   COMPOSE (mergeName name1 name2) (FIELD_LIST (lst1 ++ lst2))

-- | Function that compare two composes.
-- Two composes are equal if their FIELD_LIST are equal. This ignores
-- the compose identifier, that haves no meaning here.
compareCompose :: Type -> Type -> Bool
compareCompose (COMPOSE _ fldLst1) (COMPOSE _ fldLst2) = fldLst1 == fldLst2
compareCompose x y = x == y

-- | ??
collectUniqueKeys :: [Field] -> [(Name, Type)]
collectUniqueKeys = filterDuplicates compareTuples . findKeys
   where 
   	compareTuples = (\x y -> compareCompose (snd x) (snd y))

-- | ??
findKeys :: [Field] -> [(Name, Type)]
findKeys = runIdentity . strat_allTU (return . step)
   where
	step (FIELD_NAME ident (MAP_TYPE x _)) = [(ident, x)]
	step _ = []

-- | ??
collectRng :: (Term r) => Type -> r -> [Type]
collectRng t = runIdentity . strat_allTU (return . step t)
   where
	step t (FIELD_NAME _ (MAP_TYPE key dom)) = 
		if compareCompose t key then [dom] else []
	step _ _ = []


-- | Optmize nested composes.
optimizeComposes :: [Field] -> [Field]
optimizeComposes = applyIter (runIdentity . stratTP aux)
   where aux (COMPOSE name (FIELD_LIST lst)) = COMPOSE name (FIELD_LIST (applyCompose lst))
         aux a = a
         applyCompose lst = concat $ map composeMerge lst

-- | Merge the outer compose with the inner one if they exist.
composeMerge :: Field -> [Field]
composeMerge (FIELD_NAME name1 (COMPOSE _ (FIELD_LIST lst))) =
   [FIELD_NAME (mergeName name1 name2) vdmType
      | (FIELD_NAME name2 vdmType) <- lst]
composeMerge a = [a]

-------------------------------------------------------------------------------

