-------------------------------------------------------------------------------
--
-- | 
-- Module      :  Language.VDM_SL.Pretty
-- Copyright   :  (c) Tiago Alves, Paulo Silva, 2004
-- License     :  LGPL
-- 
-- Maintainer  :  joost.visser@di.uminho.pt
-- Stability   :  experimental
-- Portability :  portable
--
-- This defines a pretty printer for VDM-SL.
--
-------------------------------------------------------------------------------

module Language.VDM_SL.Pretty (
		-- * Pretty printing
		prettyPrint
		-- * Pretty-printing styles (from @Text.PrettyPrint.HughesPJ@)
	       ) where

import Language.VDM_SL.Syntax 

import Text.PrettyPrint

-----------------------------------------------------------------------------

-- | Things that can be pretty-printed, including all the syntactic objects
-- in "Language.Haskell.Syntax".
class Pretty a where
	-- | Pretty-print something in isolation.
	pretty :: a -> Doc
	-- | Pretty-print something in a precedence context.
	prettyPrec :: Int -> a -> Doc
	pretty = prettyPrec 0
	prettyPrec _ = pretty


prettyPrint :: Pretty a => a -> String
prettyPrint = renderStyle style . pretty 

------------------------ Pretty-Print -------------------------------


instance Pretty Document where
	pretty (DEF_BLOCK_S block) = 
		pretty block $$ blankline empty
	pretty (DEF_BLOCK_L doc block) = 
		(pretty doc) $$ pretty block $$ blankline empty

instance Pretty Definition_block where
	pretty (TYPES t) = 
		text "types" $$ pretty t 
	pretty (STATES s) = 
		blankline $ pretty s 
	pretty (VALUES v) = 
		text "values" $$ pretty v
	pretty (FUNCTIONS f) = 
		text "functions" $$ pretty f 
	pretty (OPERATIONS o) = 
		text "operations" $$ pretty o 
		

instance Pretty Type_definition_list where
	pretty (TYPE_DEF lst) = prettyLst lst
		where prettyLst [t] = blankline $ pretty t
		      prettyLst (t:tls) = pretty t <> semi $$ blankline (prettyLst tls)
		      
instance Pretty Type_definition where
	pretty (TYPE_TYPE name t) = 
		pretty name <+> equals <+> pretty t
	pretty (TYPE_TYPE_INV name t inv) = 
		pretty name <+> equals <+> pretty t $$ pretty inv
	pretty (TYPE_FIELD_L name fls) = 
		pretty name <+> colon <> colon <+> pretty fls
	pretty (TYPE_FIELD_L_INV name fls inv) = 
		pretty name <+> colon <> colon <+> pretty fls $$ pretty inv
	
instance Pretty Type_list where
	pretty (TYPE_S t) = 
		pretty t
	pretty (TYPE_L lst t) = 
		pretty lst <> comma <+> pretty t
	
instance Pretty Type where
	pretty (BRACK_TYPE t) = 
		parens $ pretty t
	pretty (BOOL) = 
		text "bool"
	pretty (NAT) = 
		text "nat"
	pretty (NAT1) = 
		text "nat1"
	pretty (INT) = 
		text "int"
	pretty (RAT) = 
		text "rat"
	pretty (REAL) = 
		text "real"
	pretty (CHAR) = 
		text "char"
	pretty (TOKEN) = 
		text "token"
	pretty (QUOTE_TYPE q) = 
		char '<' <> text q <> char '>'
	pretty (COMPOSE name flst) = 
		text "compose" <+> pretty name <+> text "of" <+> pretty flst <+> 
		text "end"
	pretty (UNION_TYPE t1 t2) = 
		pretty t1 <+> char '|' <+> pretty t2
	pretty (PROD_TYPE t1 t2) = 
		pretty t1 <+> char '*' <+> pretty t2
	pretty (OPT_TYPE t) = 
		brackets $ pretty t 
	pretty (SET_OF t) = 
		text "set of" <+> pretty t
	pretty (SEQ0_OF t) = 
		text "seq of" <+> pretty t
	pretty (SEQ1_OF t) = 
		text "seq1 of" <+> pretty t
	pretty (MAP_TYPE t1 t2) = 
		text "map" <+> pretty t1 <+> text "to" <+> pretty t2
	pretty (INMAP_TYPE t1 t2) = 
		text "inmap" <+> pretty t1 <+> text "to" <+> pretty t2
	pretty (FUNC_TYPE ft) = 
		pretty ft
	pretty (TYPE_VAR tv) = 
		char '@' <> text tv
	pretty (NAME_TYPE nt) = 
		pretty nt
	pretty (NIL_TYPE) =
		empty

instance Pretty Function_type where
	pretty (TOT_FUNC t1 t2) = 
		pretty t1 <+> text "->" <+> pretty t2
	pretty (TOT_FUNC0 t) = 
		lparen <> rparen <+> text "->" <+> pretty t
	pretty (PART_FUNC t1 t2) = 
		pretty t1 <+> text "+>" <+> pretty t2
	pretty (PART_FUNC0 t) = 
		lparen <> rparen <+> text "+>" <+> pretty t
	
instance Pretty Discretionary_type where
	pretty (SOME t) = 
		pretty t
	pretty (NOTH) = 
		lparen <> rparen
	
instance Pretty Field_list where
	pretty (FIELD_LIST lst) = prettyLst lst 
		where prettyLst [] = empty
		      prettyLst (f:flst) = pretty f $$ prettyLst flst

instance Pretty Field where
	pretty (FIELD_NAME name t) = 
		pretty name <> colon <+> pretty t
	pretty (FIELD_NAME_IGNORE name t) =
		pretty name <> colon <> char '-' <+> pretty t
	pretty (FIELD_TYPE t) = 
		pretty t
	
instance Pretty State_definition where
	pretty (STATE name flst) = 
		text "state" <+> pretty name <+> text "of" <+> 
		pretty flst $$ text "end"
	pretty (STATE_INV name flst inv) = 
		text "state" <+> pretty name <+> text "of" <+> 
		pretty flst $$ pretty inv $$ text "end"
	pretty (STATE_INI name flst ini) = 
		text "state" <+> pretty name <+> text "of" <+> 
		pretty flst $$ pretty ini $$ text "end"
	pretty (STATE_INV_INI name flst inv ini) = 
		text "state" <+> pretty name <+> text "of" <+> 
		pretty flst $$ pretty inv $$ pretty ini $$ text "end"
	
instance Pretty Invariant where
	pretty (INVARIANT pat expr) = 
		text "inv" <+> pretty pat <+> dequals <+> pretty expr
	
instance Pretty Initialization where
	pretty (INITIALIZATION pat expr) = 
		text "init" <+> pretty pat <+> dequals <+> pretty expr
	
instance Pretty Value_definition_list where
	pretty (VAL_DEF_S vdef) = 
		blankline $ pretty vdef
	pretty (VAL_DEF_L vlst vdef) = 
		pretty vlst <> semi $$ blankline ( pretty vdef )
	
instance Pretty Value_definition where
	pretty (PATTERN pat expr) = 
		pretty pat <+> equals <+> pretty expr
	pretty (PATTERN_TYPE pat t expr) = 
		pretty pat <> colon <+> pretty t <+> equals <+> pretty expr
	
instance Pretty Function_definition_list where
	pretty (FUNC_DEF_S fdef) = 
		blankline $ pretty fdef
	pretty (FUNC_DEF_L flst fdef) = 
		pretty flst <> semi $$ blankline ( pretty fdef )
	
instance Pretty Function_definition where
	pretty (EXPL_FUNC_DEF fdef) = 
		pretty fdef
	pretty (IMPL_FUNC_DEF fdef) = 
		pretty fdef
	
instance Pretty Explicit_function_definition where
	pretty (EXPL_FUNC n1 ft n2 plst expr mprec) = 
		pretty n1 <> colon <+> pretty ft $$ pretty n2 <+> pretty plst <+> 
		dequals <+> pretty expr $$ pretty mprec
	pretty (EXPL_FUNC_TYPE_VAR n1 tlst ft n2 plst expr mprec) = 
		pretty n1 <+> brackets ( pretty tlst ) <> colon <+> 
		pretty ft $$ pretty n2 <+> pretty plst <+> dequals <+> 
		pretty expr $$ pretty mprec
	
instance Pretty Implicit_function_definition where
	pretty (IMPL_FUNC n1 plst n2 t mprec expr) = 
		pretty n1 <+> pretty plst <+> pretty n2 <> colon <+> 
		pretty t $$ pretty mprec $$ text "post" <+> pretty expr
	pretty (IMPL_FUNC_TYPE n1 tlst plst n2 t mprec expr) = 
		pretty n1 <+> brackets( pretty tlst ) <+> pretty plst <+> 
		pretty n2 <> colon <+> pretty t $$ pretty mprec $$ text "post" <+> 
		pretty expr
	
instance Pretty Type_variable_list where
	pretty (TYPE_VAR_S tvar) = 
		char '@' <> text tvar 
	pretty (TYPE_VAR_L tlst tvar) = 
		pretty tlst <> comma <+> char '@' <> text tvar
	
instance Pretty Parameter_type_list where 
	pretty (PARAM_TYPE_E) = 
		lparen <> rparen 
	pretty (PARAM_TYPE_L p) = 
		parens $ pretty p
	
instance Pretty Pattern_type_pair_list where
	pretty (PAT_TYPE_S plst t) = 
		pretty plst <> colon <+> pretty t
	pretty (PAT_TYPE_L ptplst plst t) = 
		pretty ptplst <> comma <+> pretty plst <> colon <+> pretty t
	
instance Pretty Parameters_list where 
	pretty (PARAM_S blst) = 
		pretty blst
	pretty (PARAM_L plst blst) = 
		pretty plst <+> pretty blst
	
instance Pretty Brack_maybe_pattern_list where
	pretty (BRACK_MAY_NOTH) = 
		lparen <> rparen
	pretty (BRACK_MAY_JUST plst) = 
		parens $ pretty plst
	
instance Pretty Maybe_precondition where
	pretty (MAY_PRECOND_NOTH) = 
	 	empty	
	pretty (MAY_PRECOND_JUST expr) = 
		text "pre" <+> pretty expr 
	
instance Pretty Operation_definition_list where
	pretty (OPER_DEF_S opdef) = 
		blankline $ pretty opdef
	pretty (OPER_DEF_L oplst opdef) = 
		pretty oplst <> semi $$ blankline ( pretty opdef )
	
instance Pretty Operation_definition where 
	pretty (EXPL_OPER_DEF eodef) = 
		pretty eodef
	pretty (IMPL_OPER_DEF iodef) = 
		pretty iodef
	
instance Pretty Explicit_operation_definition where
	pretty (EXPL_OPER n1 dt1 dt2 n2 bplst stat mprec) =
		pretty n1 <> colon <+> pretty dt1 <+> dequals <> char '>' <+>
		pretty dt2 $$ pretty n2 <+> pretty bplst <+> dequals <+> 
		pretty stat $$ pretty mprec
	
instance Pretty Implicit_operation_definition where
	pretty (IMPL_OPER name plst mext mprec expr) =
		pretty name <+> pretty plst <+> pretty mext $$ pretty mprec $$
		text "post" <+> pretty expr
	pretty (IMPL_OPER_EXCEP name plst mext mprec expr elst) =
		pretty name <+> pretty plst <+> pretty mext $$ pretty mprec $$
		text "post" <+> pretty expr $$ text "errs" <+> pretty elst
	pretty (IMPL_OPER_TYPE n1 plst n2 t mext mprec expr) =
		pretty n1 <+> pretty plst $$ pretty n2 <> colon <+> 
		pretty t <+> pretty mext $$ pretty mprec $$ text "post" <+> 
		pretty expr
	pretty (IMPL_OPER_TYPE_EXCEP n1 plst n2 t mext mprec expr elst) =
		pretty n1 <+> pretty plst $$ pretty n2 <> colon <+> 
		pretty t <+> pretty mext $$ pretty mprec $$ text "post" <+>
		pretty expr $$ text "errs" <+> pretty elst
	
instance Pretty Maybe_externals where
	pretty (MAY_EXTERN_NOTH) = 
		empty	
	pretty (MAY_EXTERN_JUST elst) = 
		pretty elst
	
instance Pretty External_list where
	pretty (EXTERNAL_S vinfo) = 
		text "ext" <+> pretty vinfo
	pretty (EXTERNAL_L elst vinfo) =
		pretty elst <+> pretty vinfo
	
instance Pretty Var_information where 
	pretty (RD nlst) = 
		text "rd" <+> pretty nlst
	pretty (WR nlst) =
		text "wr" <+> pretty nlst
	pretty (RD_TYPE nlst t) =
		text "rd" <+> pretty nlst <> colon <+> pretty t
	pretty (WR_TYPE nlst t) =
		text "wr" <+> pretty nlst <> colon <+> pretty t
	
instance Pretty Exception_list where
	pretty (EXCEPT_S name expr1 expr2) =
		pretty name <> colon <+> pretty expr1 <+> text "->" <+> pretty expr2
	pretty (EXCEPT_L elst name expr1 expr2) =
		pretty elst <+> pretty name <> colon <+> pretty expr1 <+> 
		text "->" <+> pretty expr2
	
instance Pretty Expression_list where
	pretty (EXPR_S expr) =
		pretty expr
	pretty (EXPR_L elst expr) =
		pretty elst <> comma <+> pretty expr
	
instance Pretty Expression where 
	pretty (BRACK_EXPR expr) = 
		parens $ pretty expr
	pretty (LET lst expr) =
		text "let" <+> pretty lst <+> text "in" <+> pretty expr
	pretty (LET_INSET pat expr1 expr2) =
		text "let" <+> pretty pat <+> text "in set" <+> pretty expr1 <+>
		text "in" <+> pretty expr2
	pretty (LET_VAR name t expr) =
		text "let" <+> pretty name <> colon <+> pretty t <+> text "in" <+>
		pretty expr
	pretty (LET_PAT pat t expr) =
		text "let" <+> pretty pat <> colon <+> pretty t <+> text "in" <+> 
		pretty expr
	pretty (LETBE_INSET pat expr1 expr2 expr3) =
		text "let" <+> pretty pat <+> text "in set" <+> pretty expr1 <+>
		text "be st" <+> pretty expr2 <+> text "in" <+> pretty expr3
	pretty (LETBE_VAR name t expr1 expr2) =
		text "let" <+> pretty name <> colon <+> pretty t <+> text "be st" <+>
		pretty expr1 <+> text "in" <+> pretty expr2
	pretty (LETBE_PAT pat t expr1 expr2) =
		text "let" <+> pretty pat <> colon <+> pretty t <+> text "be st" <+>
		pretty expr1 <+> text "in" <+> pretty expr2
	pretty (DEF plst expr) =
		text "def" <+> pretty plst <+> text "in" <+> pretty expr
	pretty (IF_EXPR ifexpr) =
		pretty ifexpr
	pretty (CASE expr alst) =
		text "cases" <+> pretty expr <> colon $$ pretty alst $$ text "end"
	pretty (CASE_OTHER expr1 clst expr2) =
		text "cases" <+> pretty expr1 <> colon $$ pretty clst <> comma $$
		text "other" <+> text "->" <+> pretty expr2 $$ text "end"
	pretty (PLUS expr) =
		char '+' <+> pretty expr
	pretty (MINUS expr) =
		char '-' <+> pretty expr
	pretty (ABS expr) =
		text "abs" <+> pretty expr
	pretty (FLOOR expr) =
		text "floor" <+> pretty expr
	pretty (NOT expr) =
		text "not" <+> pretty expr
	pretty (CARD expr) =
		text "card" <+> pretty expr
	pretty (POWER expr) =
		text "power" <+> pretty expr
	pretty (DUNION expr) =
		text "dunion" <+> pretty expr
        pretty (DINTER expr) =
		text "dinter" <+> pretty expr
	pretty (HD expr) =
		text "hd" <+> pretty expr
	pretty (TL expr) =
		text "tl" <+> pretty expr
	pretty (LEN expr) =	
		text "len" <+> pretty expr
	pretty (ELEMS expr) =
		text "elems" <+> pretty expr
	pretty (INDS expr) =
		text "inds" <+> pretty expr
	pretty (CONC expr) =
		text "conc" <+> pretty expr
	pretty (DOM expr) =
		text "dom" <+> pretty expr
	pretty (RANGE expr) =
		text "rng" <+> pretty expr
	pretty (MERGE1 expr) =
		text "merge" <+> pretty expr
	pretty (INVERSE expr) =
		text "inverse" <+> pretty expr
	pretty (ADD expr1 expr2) =
		pretty expr1 <+> char '+' <+> pretty expr2
	pretty (SUBTR expr1 expr2) =
		pretty expr1 <+> char '-' <+> pretty expr2
	pretty (MULT expr1 expr2) =
		pretty expr1 <+> char '*' <+> pretty expr2
	pretty (DIVIDE expr1 expr2) =
		pretty expr1 <+> char '/' <+> pretty expr2
	pretty (DIV expr1 expr2) =
		pretty expr1 <+> text "div" <+> pretty expr2
	pretty (REM expr1 expr2) =
		pretty expr1 <+> text "rem" <+> pretty expr2
	pretty (MOD expr1 expr2) =
		pretty expr1 <+> text "mod" <+> pretty expr2
	pretty (LESS expr1 expr2) =
		pretty expr1 <+> char '<' <+> pretty expr2
	pretty (LESS_EQ expr1 expr2) =
		pretty expr1 <+> text "<=" <+> pretty expr2
	pretty (GREAT expr1 expr2) =
		pretty expr1 <+> char '>' <+> pretty expr2
	pretty (GREAT_EQ expr1 expr2) =
		pretty expr1 <+> text ">=" <+> pretty expr2 
	pretty (EQUAL expr1 expr2) =
		pretty expr1 <+> equals <+> pretty expr2
	pretty (NEQ expr1 expr2) =
		pretty expr1 <+> text "<>" <+> pretty expr2
	pretty (OR expr1 expr2) =
		pretty expr1 <+> text "or" <+> pretty expr2
	pretty (AND expr1 expr2) =
		pretty expr1 <+> text "and" <+> pretty expr2
	pretty (IMPLY expr1 expr2) =
		pretty expr1 <+> text "=>" <+> pretty expr2
	pretty (EQUIV expr1 expr2) =
		pretty expr1 <+> text "<=>" <+> pretty expr2
	pretty (IN_SET expr1 expr2) =
		pretty expr1 <+> text "in set" <+> pretty expr2
	pretty (NOT_IN_SET expr1 expr2) =
		pretty expr1 <+> text "not in set" <+> pretty expr2
	pretty (SUBSET expr1 expr2) =
		pretty expr1 <+> text "subset" <+> pretty expr2
	pretty (PSUBSET expr1 expr2) =
		pretty expr1 <+> text "psubset" <+> pretty expr2
	pretty (UNION expr1 expr2) =
		pretty expr1 <+> text "union" <+> pretty expr2
	pretty (MERGE2 expr1 expr2) =
		pretty expr1 <+> text "merge" <+> pretty expr2
	pretty (SET_DIFF expr1 expr2) =
		pretty expr1 <+> char '\\' <+> pretty expr2
	pretty (INTER expr1 expr2) =
		pretty expr1 <+> text "inter" <+> pretty expr2
	pretty (CONCAT expr1 expr2) =
		pretty expr1 <+> char '^' <+> pretty expr2
	pretty (MODIFY expr1 expr2) =
		pretty expr1 <+> text "++" <+> pretty expr2
	pretty (MUNION expr1 expr2) =
		pretty expr1 <+> text "munion" <+> pretty expr2
	pretty (DOM_RESTR_TO expr1 expr2) =
		pretty expr1 <+> text "<:" <+> pretty expr2
	pretty (DOM_RESTR_BY expr1 expr2) =
		pretty expr1 <+> text "<-:" <+> pretty expr2
	pretty (RNG_RESTR_TO expr1 expr2) =
		pretty expr1 <+> text ":>" <+> pretty expr2
	pretty (RNG_RESTR_BY expr1 expr2) =
		pretty expr1 <+> text ":->" <+> pretty expr2
	pretty (COMP expr1 expr2) =
		pretty expr1 <+> text "comp" <+> pretty expr2
	pretty (ITERATE expr1 expr2) =
		pretty expr1 <+> text "**" <+> pretty expr2
	pretty (FORALL blst expr) = 
		text "forall" <+> pretty blst <+> char '&' <+> pretty expr
	pretty (EXISTS blst expr) = 
		text "exists" <+> pretty blst <+> char '&' <+> pretty expr
	pretty (EXISTS1 bind expr) = 
		text "exists1" <+> pretty bind <+> char '&' <+> pretty expr
	pretty (IOTA bind expr) = 
		text "iota" <+> pretty bind <+> char '&' <+> pretty expr
	pretty (EMPTY_SET) = 
		lbrace <> rbrace
	pretty (SET expr) =
		braces $ pretty expr
	pretty (SET2 expr1 expr2) =
		braces $ pretty expr1 <> comma <+> pretty expr2
	pretty (SET_ENUM expr1 expr2 elst) =
		braces $ pretty expr1 <> comma <+> pretty expr2 <> comma <+> 
		pretty elst
	pretty (COMPR_SET expr blst) = 
		braces $ pretty expr <+> char '|' <+> pretty blst
	pretty (COMPR_SET_COND expr1 blst expr2) = 
		braces $ pretty expr1 <+> char '|' <+> pretty blst <+> char '&' <+>
		pretty expr2
	pretty (RANGE_SET expr1 expr2) =
		braces $ pretty expr1 <> comma <> text "..." <> comma <+> 
		pretty expr2
	pretty (EMPTY_SEQ) =
		lbrack <> rbrack
	pretty (SEQ elst) =
		brackets $ pretty elst
	pretty (COMPR_SEQ expr blst) =
		brackets $ pretty expr <+> char '|' <+> pretty blst 
	pretty (COMPR_SEQ_COND expr1 blst expr2) = 
		brackets $ pretty expr1 <+> char '|' <+> pretty blst <+> char '&' <+>
		pretty expr2
	pretty (SUBSEQ expr1 expr2 expr3) =
		pretty expr1 <+> parens ( pretty expr2 <> comma <> text "..." <> 
		comma <+> pretty expr3 )
	pretty (EMPTY_MAP) =
		braces $ text "|->" 
	pretty (MAP mlst) =
		braces $ pretty mlst
	pretty (COMPR_MAP expr1 expr2 blst) =
		braces $ pretty expr1 <+> text "|->" <+> pretty expr2 <+> 
		char '|' <+> pretty blst 
	pretty (COMPR_MAP_COND expr1 expr2 blst expr3) = 
		braces $ pretty expr1 <+> text "|->" <+> pretty expr2 <+>
		char '|' <+> pretty blst <+> char '&' <+> pretty expr3
	pretty (TUPLE elst) =
		text "mk_" <> parens ( pretty elst )
	pretty (RECORD_E str) =
		text "mk_" <> text str <> lparen <> rparen
	pretty (RECORD_S str elst) =
		text "mk_" <> text str <> parens ( pretty elst )
	pretty (MU expr rlst) =
		text "mu" <+> parens ( pretty expr <> comma <+> pretty rlst )
	pretty (APPLY0 expr) =
		pretty expr <> lparen <> rparen
	pretty (APPLY1 expr1 expr2) =
		pretty expr1 <> parens ( pretty expr2 )
	pretty (APPLYN expr1 expr2 elst) =
		pretty expr1 <> parens ( pretty expr2 <> comma <+> pretty elst )
	pretty (FIELD_SEL expr name) =
		pretty expr <> char '.' <> pretty name
	pretty (TUPLE_SEL expr lnum) =
		pretty expr <> text ".#" <> int lnum
	pretty (FUNCT_TYPE_INST name tlst) =
		pretty name <> brackets ( pretty tlst )
	pretty (LAMBDA tlst expr) =
		text "lambda" <+> pretty tlst <+> char '&' <+> pretty expr
	pretty (IS_IDENT str expr) =
		text "is_" <> text str <> parens ( pretty expr )
	pretty (IDENT str) = 
		text str
	pretty (OLD_NAME name) =
		pretty name <> char '~'
	pretty (SYMB_LIT slit) =
		pretty slit

instance Pretty Symbolic_literal where
	pretty (LIT_NUM lnum) = 
		double lnum
	pretty (NUMERAL num) =
		int num
	pretty (TRUE) = 
		text "true"
	pretty (FALSE) = 
		text "false"
	pretty (NIL) = 
		text "nil" 
	pretty (LIT_CHAR lchar) =
		char '\'' <> text lchar <> char '\''
	pretty (LIT_STR lstr) =
		doubleQuotes $ text lstr
	pretty (LIT_QUOTE lquote) = 
		char '<' <> text lquote <> char '>'
	
instance Pretty Patternbind_expr_list where
	pretty (PATT_BIND_EXPR_S pbind expr) = 
		pretty pbind <+> equals <+> pretty expr
	pretty (PATT_BIND_EXPR_L plst pbind expr) =
		pretty plst <> semi <> pretty pbind <+> equals <+> pretty expr
	
instance Pretty Local_definition_list where
	pretty (LOCAL_DEF_S ldef) = 
		pretty ldef
	pretty (LOCAL_DEF_L llst ldef) = 
		pretty llst <> comma <+> pretty ldef
	
instance Pretty If_expression where
	pretty (IF_THEN_ELSE expr1 expr2 expr3) =
		text "if" <+> pretty expr1 $$
		text "then" <+> pretty expr2 $$
		text "else" <+> pretty expr3
	pretty (IF_THEN_ELSEIF expr1 expr2 elifexpr expr3) =
		text "if" <+> pretty expr1 $$
		text "then" <+> pretty expr2 $$
		pretty elifexpr $$
		text "else" <+> pretty expr3
	
instance Pretty Elseif_expression where
	pretty (ELSEIF expr1 expr2) =
		text "elseif" <+> pretty expr1 $$
		text "then" <+> pretty expr2
	pretty (EXP_ELSEIF elifexpr expr1 expr2) =
		pretty elifexpr $$
		text "elseif" <+> pretty expr1 <+>
		text "then" <+> pretty expr2
	
instance Pretty Cases_alternative_list where
	pretty (CASE_ALT_S plst expr) =
		pretty plst <+> text "->" <+> pretty expr
	pretty (CASE_ALT_L caltlst plst expr) =
		pretty caltlst <> comma <+> pretty plst <+> text "->" <+> 
		pretty expr
		
instance Pretty Name_list where
	pretty (NAME_S name) = 
		pretty name
	pretty (NAME_L nlst name) = 
		pretty nlst <> comma <+> pretty name
	
instance Pretty Name where
	pretty (IDENTIFIER str) = 
		text str	
	pretty (MK_ID str) = 
		text "mk_" <> text str
	pretty (IS_ID str) = 
		text "is_" <> text str
	
instance Pretty Map_enumeration_list where
	pretty (MAP_ENUM_S expr1 expr2) = 
		pretty expr1 <+> text "|->" <+> pretty expr2
	pretty (MAP_ENUM_L mlst expr1 expr2) =
		pretty mlst <> comma <+> pretty expr1 <+> text "|->" <+> 
		pretty expr2
	
instance Pretty Record_modification_list where
	pretty (RECORD_MOD_S name expr) =
		pretty name <+> text "|->" <+> pretty expr
	pretty (RECORD_MOD_L rlst name expr) =
		pretty rlst <> comma <+> pretty name <+> text "|->" <+> pretty expr
	
instance Pretty State_designator where
	pretty (STATE_S name) = 
		pretty name 
	pretty (STATE_L sdesig name) = 
		pretty sdesig <> char '.' <> pretty name
	pretty (STATE_EXPR_L sdesig expr) =
		pretty sdesig <> parens ( pretty expr )
	
instance Pretty Statement_list where
	pretty (STATEMENT_S stat) =
		pretty stat
	pretty (STATEMENT_L slst stat) =
		pretty slst <> semi $$ pretty stat
	
instance Pretty Statement where
	pretty (LET_STAT ldlst stat) =
		text "let" <+> pretty ldlst <+> text "in" <+> pretty stat
	pretty (LET_INSET_STAT pat expr stat) =
		text "let" <+> pretty pat <+> text "in set" <+> pretty expr <+>
		text "in" <+> pretty stat
	pretty (LET_VAR_STAT name t stat) =
		text "let" <+> pretty name <> colon <+> pretty t <+> text "in" <+>
		pretty stat
	pretty (LET_PAT_STAT pat t stat) =
		text "let" <+> pretty pat <> colon <+> pretty t <+> text "in" <+> 
		pretty stat
	pretty (LETBE_INSET_STAT pat expr1 expr2 stat) =
		text "let" <+> pretty pat <+> text "in set" <+> pretty expr1 <+> 
		text "be st" <+> pretty expr2 <+> text "in" <+> pretty stat
	pretty (LETBE_VAR_STAT name t expr stat) = 
		text "let" <+> pretty name <> colon <+> pretty t <+> text "be st" <+>
		pretty expr <+> text "in" <+> pretty stat
	pretty (LETBE_PAT_STAT pat t expr stat) =
		text "let" <+> pretty pat <> colon <+> pretty t <+> text "be st" <+>
		pretty expr <+> text "in" <+> pretty stat
	pretty (DEF_STAT elst stat) =
		text "def" <+> pretty elst <+> text "in" <+> pretty stat
	pretty (MAYBE_DCL_STAT mlst slst) =
		parens $ pretty mlst <+> pretty slst
	pretty (CALL_STAT cstat) =
		pretty cstat
	pretty (SKIP) =
		text "skip"
	
instance Pretty Call_statement where 
	pretty (CALL_ID_STAT name) =
		pretty name <> lparen <> rparen
	pretty (CALL_EXPR_STAT name elst) =
		pretty name <> parens ( pretty elst )
	pretty (CALL_STATE_STAT name sdesig) =
		pretty name <> lparen <> rparen <+> text "using" <+> pretty sdesig
	pretty (CALL_EXPR_STATE_STAT name elst sdesig) =
		pretty name <> parens ( pretty elst ) <+> text "using" <+> 
		pretty sdesig
	
instance Pretty Equals_definition_list where 
	pretty (EQUALS_DEF_S pbind expr) =
		pretty pbind <+> equals <+> pretty expr
	pretty (EQUALS_DEF_L elst pbind expr) =
		pretty elst <> semi $$ pretty pbind <+> equals <+> pretty expr
	
instance Pretty Maybe_dcl_statement_list where
	pretty (MAYBE_DCL_STAT_E) =
		empty
	pretty (MAYBE_DCL_STAT_S mlst name t) =
		pretty mlst <+> pretty name <> colon <+> pretty t
	pretty (MAYBE_DCL_STAT_L mlst name t expr) =
		pretty mlst <+> pretty name <> colon <+> pretty t <+> text ":=" <+>
		pretty expr
	
instance Pretty Pattern_list where
	pretty (PATTERN_S pat) =
		pretty pat
	pretty (PATTERN_L plst pat) =
		pretty plst <> comma <+> pretty pat
	
instance Pretty Pattern where 
	pretty (PAT_NAME name) = pretty name
	pretty (PAT_PAT pat) = pretty pat
	
instance Pretty Pattern2 where
	pretty (IDENT_PAT) =
		char '-'
	pretty (BRACK_PAT expr) =
		parens $ pretty expr
	pretty (SYMB_LIT_PAT slit) =
		pretty slit
	pretty (SET_ENUM_PAT plst) =
		braces $ pretty plst
	pretty (SET_UNION_PAT pat1 pat2) =
		pretty pat1 <+> text "union" <+> pretty pat2
	pretty (SEQ_ENUM_PAT plst) =
		brackets $ pretty plst
	pretty (SEQ_CONC_PAT pat1 pat2) =
		pretty pat1 <> char '^' <> pretty pat2
	pretty (TUPLE_PAT plst) =
		text "mk_" <> parens ( pretty plst )
	pretty (LIST_PAT name blst) =
		pretty name <+> pretty blst
	
instance Pretty Pattern_bind where
	pretty (PAT_BIND_PAT pat) = 
		pretty pat
	pretty (PAT_BIND_BIND bind) =
		pretty bind
	
instance Pretty Bind where
	pretty (BIND_EXPR pat expr) =
		pretty pat <+> text "in set" <+> pretty expr
	pretty (BIND_TYPE pat t) =
		pretty pat <> colon <+> pretty t
	
instance Pretty Bind_list where
	pretty (BIND_S1 msbind) =
		pretty msbind
	pretty (BIND_S2 mtbind) =
		pretty mtbind
	pretty (BIND_L1 blst msbind) =
		pretty blst <> comma <+> pretty msbind
	pretty (BIND_L2 blst mtbind) =
		pretty blst <> comma <+> pretty mtbind
	
instance Pretty Multiple_set_bind where
	pretty (MULT_SET_BIND plst expr) =
		pretty plst <+> text "in set" <+> pretty expr
	
instance Pretty Multiple_type_bind where
	pretty (MULT_TYPE_BIND plst t) =
		pretty plst <> colon <+> pretty t
	
instance Pretty Type_bind_list where
	pretty (TYPE_BIND_S pat t) =
		pretty pat <> colon <+> pretty t
	pretty (TYPE_BIND_L tlst pat t) =
		pretty tlst <> comma <+> pretty pat <> colon <+> pretty t
	
instance Pretty Local_definition where
	pretty (L_PAT pat expr) =
		pretty pat <+> equals <+> pretty expr
	pretty (L_NAME name t expr) = 
		pretty name <> colon <+> pretty t <+> equals <+> pretty expr
	pretty (L_PAT_TYPE pat t expr) =
		pretty pat <> colon <+> pretty t <+> equals <+> pretty expr
	pretty (L_EXPL_FUNC n1 ft n2 plst expr mprec) =
		pretty n1 <> colon <+> pretty ft $$
		pretty n2 <+> pretty plst <+> dequals <+> pretty expr $$
		pretty mprec
	pretty (L_EXPL_FUNC_TYPE n1 tlst ft n2 plst expr mprec) =
		pretty n1 <+> brackets ( pretty tlst ) <> colon <+> pretty ft $$
		pretty n2 <+> pretty plst <+> dequals <+> pretty expr $$
		pretty mprec
	pretty (L_IMPL_FUNC n1 plst n2 t mprec expr) =
		pretty n1 <+> pretty plst <+> pretty n2 <> colon <+> pretty t $$
		pretty mprec $$ text "post" <+> pretty expr
	pretty (L_IMPL_FUNC_TYPE n1 tlst plst n2 t mprec expr) =
		pretty n1 <+> brackets ( pretty tlst ) <+> pretty plst <+> 
		pretty n2 <> colon <+> pretty t $$
		pretty mprec $$ text "post" <+> pretty expr

blankline :: Doc -> Doc
blankline dl = space $$ dl 

dequals :: Doc
dequals = equals <> equals
