-------------------------------------------------------------------------------
--
-- | 
-- Module      :  Language.VDM_SL.Syntax
-- Copyright   :  (c) Tiago Alves, Paulo Silva, 2004
-- License     :  LGPL
-- 
-- Maintainer  :  joost.visser@di.uminho.pt
-- Stability   :  experimental
-- Portability :  portable
--
-- This module defines the Abstract Syntax Tree (AST) for the
-- VDM-SL language, accondingly with the ISO standard. Since the ISO standard
-- differs in some points from the IFAD implementation of VDM, some of the
-- features supported by the VDM-Tools might not have been implemented.
-- 
-- This AST was manually derived from the concrete BNF grammar. The concrete
-- grammar is based on the work of Richard Atterer (<atterer@in.tum.de>, 
-- <http://www.in.tum.de/~atterer/>) in a dissertation submitted at 
-- The Queen's University of Belfast: Automatic Test Data Generation From
-- VDM-SL Specifications. This can be found in the author's web site.
--
-------------------------------------------------------------------------------

module Language.VDM_SL.Syntax where


-------------------------------------------------------------------------------
-- * Primitive Datatypes 
-------------------------------------------------------------------------------

type Type_variable_identifier = String
type Lit_char = String
type Lit_string = String
type Lit_quote = String
type Lit_num = Double
type Numeral = Int

-------------------------------------------------------------------------------
--
-- * Document Datatype 
--
-------------------------------------------------------------------------------

{-|  A VDM-SL Document is a list of /Definitions Blocks/.
-}

data Document = 
	  DEF_BLOCK_S Definition_block
	| DEF_BLOCK_L Document Definition_block
	deriving (Eq, Show)

-------------------------------------------------------------------------------
--
-- ** Definition Blocks 
--
-- | These are the main blocks in a VDM-SL document:
-- 
-------------------------------------------------------------------------------

data Definition_block = 
	  -- | A list of datatype definitions
	  TYPES Type_definition_list
	  -- | A state definition
	| STATES State_definition
	  -- | A list of value definitions
	| VALUES Value_definition_list
	  -- | A list of function definitions
	| FUNCTIONS Function_definition_list
	  -- | A list of operation definition
	| OPERATIONS Operation_definition_list
	deriving (Eq, Show)

-------------------------------------------------------------------------------
--
-- *** Datatype Definition
--
-- | In VDM-SL the types are declared in type sections. Thus, there can be
-- a list of type defintions. 
--
-------------------------------------------------------------------------------

data Type_definition_list = 
	TYPE_DEF [Type_definition]
	deriving (Eq, Show)

-- | There are four ways of defining a type: 
data Type_definition = 
	  -- | Using a name and a type description
	  TYPE_TYPE Name Type
	  -- | Using a name, a type description and an invariant
	| TYPE_TYPE_INV Name Type Invariant
	  -- | Using the record way, i.e. a name and a named field list
	| TYPE_FIELD_L Name Field_list
	  -- | Using the record way with an invariant
	| TYPE_FIELD_L_INV Name Field_list Invariant
	deriving (Eq, Show)

data Type_list = 
	  TYPE_S Type
	| TYPE_L Type_list Type
	deriving (Eq, Show)
	
-- | The types that VDM-SL allows are:
data Type = 
	  -- | The enclosed (...) datatype. 
	  BRACK_TYPE Type
	  -- | Booleans
	| BOOL
	  -- | Natural numbers with 0
	| NAT
	  -- | Natural numbers without 0
	| NAT1
	  -- | Integer numbers
	| INT
	  -- | Rational numbers
	| RAT
	  -- | Real numbers
	| REAL
	  -- | Characters
	| CHAR
	  -- | Tokens
	| TOKEN
	  -- | Quote type (corresponds to enumerated types)
	| QUOTE_TYPE Lit_quote
	  -- | Compostie types (records)
	| COMPOSE Name Field_list
	  -- | Unions of types
	| UNION_TYPE Type Type
	  -- | Products of types (tuples)
	| PROD_TYPE Type Type
	  -- | Optional type ([A] means 1 + A)
	| OPT_TYPE Type
	  -- | Sets of elements of a type
	| SET_OF Type
	  -- | Sequences of elements of a type
	| SEQ0_OF Type
	  -- | Non-empty sequences of elements of a type
	| SEQ1_OF Type
	  -- | Maps (finite functions) from a type to another
	| MAP_TYPE Type Type
	  -- | Injective maps from a type to another
	| INMAP_TYPE Type Type
	  -- | Functions
	| FUNC_TYPE Function_type
	  -- | A type variable
	| TYPE_VAR Type_variable_identifier
	  -- | An user defined type
	| NAME_TYPE Name
	  -- | 
	| NIL_TYPE
	deriving (Eq, Show)

data Function_type = 
	  TOT_FUNC Type Type
	| TOT_FUNC0 Type
	| PART_FUNC Type Type
	| PART_FUNC0 Type
	deriving (Eq, Show)

data Discretionary_type = 
	  SOME Type
	| NOTH
	deriving (Eq, Show)
			
data Field_list = 
	FIELD_LIST [Field]
	deriving (Eq, Show)

data Field = 
	  FIELD_NAME Name Type
	| FIELD_NAME_IGNORE Name Type
	| FIELD_TYPE Type
	deriving (Eq, Show)

-------------------------------------------------------------------------------	   
-- State Definition 
-------------------------------------------------------------------------------

data State_definition = 
	  STATE Name Field_list
	| STATE_INV Name Field_list Invariant
	| STATE_INI Name Field_list Initialization
	| STATE_INV_INI Name Field_list Invariant Initialization 
	deriving (Eq, Show)
		      
data Invariant = 
	  INVARIANT Pattern Expression
	  deriving (Eq, Show)

data Initialization = 
	  INITIALIZATION Pattern Expression
	  deriving (Eq, Show)

-------------------------------------------------------------------------------
-- *** Values Definition
-------------------------------------------------------------------------------

data Value_definition_list = 
	  VAL_DEF_S Value_definition
	| VAL_DEF_L Value_definition_list Value_definition
	deriving (Eq, Show)

data Value_definition = 
	  PATTERN Pattern Expression
	| PATTERN_TYPE Pattern Type Expression
	deriving (Eq, Show)
		      
-------------------------------------------------------------------------------
-- *** Function Definition
-------------------------------------------------------------------------------

data Function_definition_list = 
	  FUNC_DEF_S Function_definition
	| FUNC_DEF_L Function_definition_list Function_definition
	deriving (Eq, Show)

data Function_definition = 
	  EXPL_FUNC_DEF Explicit_function_definition
	| IMPL_FUNC_DEF Implicit_function_definition
	deriving (Eq, Show)

data Explicit_function_definition = 
	  EXPL_FUNC Name Function_type Name Parameters_list Expression Maybe_precondition
	| EXPL_FUNC_TYPE_VAR Name Type_variable_list Function_type Name Parameters_list Expression Maybe_precondition
	deriving (Eq, Show)
				  
data Implicit_function_definition = 
	  IMPL_FUNC Name Parameter_type_list Name Type Maybe_precondition Expression
	| IMPL_FUNC_TYPE Name Type_variable_list Parameter_type_list Name Type Maybe_precondition Expression
	deriving (Eq, Show)

data Type_variable_list = 
	  TYPE_VAR_S Type_variable_identifier
	| TYPE_VAR_L Type_variable_list Type_variable_identifier
	deriving (Eq, Show)

data Parameter_type_list = 
	  PARAM_TYPE_E
	| PARAM_TYPE_L Pattern_type_pair_list
	deriving (Eq, Show)

data Pattern_type_pair_list = 
	  PAT_TYPE_S Pattern_list Type
	| PAT_TYPE_L Pattern_type_pair_list Pattern_list Type
	deriving (Eq, Show)

data Parameters_list = 
	  PARAM_S Brack_maybe_pattern_list
	| PARAM_L Parameters_list Brack_maybe_pattern_list
	deriving (Eq, Show)

data Brack_maybe_pattern_list 	= 
	  BRACK_MAY_NOTH
	| BRACK_MAY_JUST Pattern_list
	deriving (Eq, Show)
				
data Maybe_precondition = 
	  MAY_PRECOND_NOTH
	| MAY_PRECOND_JUST Expression
	deriving (Eq, Show)

-------------------------------------------------------------------------------
-- *** Operation Definition
-------------------------------------------------------------------------------

data Operation_definition_list = 
	  OPER_DEF_S Operation_definition
	| OPER_DEF_L Operation_definition_list Operation_definition
	deriving (Eq, Show)

data Operation_definition = 
	  EXPL_OPER_DEF Explicit_operation_definition
	| IMPL_OPER_DEF Implicit_operation_definition
	deriving (Eq, Show)

data Explicit_operation_definition = 
	  EXPL_OPER Name Discretionary_type Discretionary_type Name Brack_maybe_pattern_list Statement Maybe_precondition
	deriving (Eq, Show)

data Implicit_operation_definition = 
	  IMPL_OPER Name Parameter_type_list Maybe_externals Maybe_precondition Expression
	| IMPL_OPER_EXCEP Name Parameter_type_list Maybe_externals Maybe_precondition Expression Exception_list
	| IMPL_OPER_TYPE Name Parameter_type_list Name Type Maybe_externals Maybe_precondition Expression
	| IMPL_OPER_TYPE_EXCEP Name Parameter_type_list Name Type Maybe_externals Maybe_precondition Expression Exception_list
	deriving (Eq, Show)	


data Maybe_externals = 
	  MAY_EXTERN_NOTH
	| MAY_EXTERN_JUST External_list
	deriving (Eq, Show)

data External_list = 
	  EXTERNAL_S Var_information
	| EXTERNAL_L External_list Var_information
	deriving (Eq, Show)

data Var_information = 
	  RD Name_list
	| WR Name_list
	| RD_TYPE Name_list Type
	| WR_TYPE Name_list Type
	deriving (Eq, Show)

data Exception_list = 
	  EXCEPT_S Name Expression Expression
	| EXCEPT_L Exception_list Name Expression Expression
	deriving (Eq, Show)


-------------------------------------------------------------------------------
-- *** Expressions
-------------------------------------------------------------------------------

data Expression_list = 
	  EXPR_S Expression
	| EXPR_L Expression_list Expression
	deriving (Eq, Show)

data Expression = 
	  BRACK_EXPR Expression
	| LET Local_definition_list Expression 
	| LET_INSET Pattern Expression Expression 
	| LET_VAR Name Type Expression 
	| LET_PAT Pattern2 Type Expression 
	| LETBE_INSET Pattern Expression Expression Expression 
	| LETBE_VAR Name Type Expression Expression 
	| LETBE_PAT Pattern2 Type Expression Expression
	| DEF Patternbind_expr_list Expression 
	| IF_EXPR If_expression
	| CASE Expression Cases_alternative_list 
	| CASE_OTHER Expression Cases_alternative_list Expression 
	| PLUS  Expression -- +
	| MINUS Expression -- -
	| ABS Expression -- abs
	| FLOOR Expression -- floor
	| NOT Expression -- not
	| CARD Expression -- card
	| POWER Expression -- power
	| DUNION Expression -- dunion
	| DINTER Expression -- dinter
	| HD Expression -- hd
	| TL Expression -- tl
	| LEN Expression -- len
	| ELEMS Expression -- elems
	| INDS Expression -- inds
	| CONC Expression -- conc
	| DOM Expression -- dom
	| RANGE Expression -- range
	| MERGE1 Expression -- merge
	| INVERSE Expression -- inverse
	| ADD Expression Expression -- +
	| SUBTR Expression Expression -- -
	| MULT Expression Expression -- '*'
	| DIVIDE Expression Expression -- /
	| DIV Expression Expression -- div
	| REM Expression Expression -- rem 
	| MOD Expression Expression -- mod
	| LESS Expression Expression -- < 
	| LESS_EQ Expression Expression -- <=
	| GREAT Expression Expression -- >
	| GREAT_EQ Expression Expression -- >=
	| EQUAL Expression Expression -- =
	| NEQ Expression Expression -- <>
	| OR Expression Expression -- or
	| AND Expression Expression -- and
	| IMPLY Expression Expression -- =>
	| EQUIV Expression Expression -- <=>
	| IN_SET Expression Expression -- in set
	| NOT_IN_SET Expression Expression -- not in set 
	| SUBSET Expression Expression -- subset
	| PSUBSET Expression Expression -- psubset
	| UNION Expression Expression -- union
	| MERGE2 Expression Expression -- merge
	| SET_DIFF Expression Expression -- \
	| INTER Expression Expression -- inter
	| CONCAT Expression Expression -- ^
	| MODIFY Expression Expression -- ++
	| MUNION Expression Expression -- munion
	| DOM_RESTR_TO Expression Expression -- <:
	| DOM_RESTR_BY Expression Expression -- <-:
	| RNG_RESTR_TO Expression Expression -- :>
	| RNG_RESTR_BY Expression Expression -- :->
	| COMP Expression Expression -- comp
	| ITERATE Expression Expression -- '**'
	| FORALL Bind_list Expression 
	| EXISTS Bind_list Expression 
	| EXISTS1 Bind Expression 
	| IOTA Bind Expression 
	| EMPTY_SET 
	| SET Expression -- {}
	| SET2 Expression Expression -- {,}
	| SET_ENUM Expression Expression Expression_list
	| COMPR_SET Expression Bind_list 
	| COMPR_SET_COND Expression Bind_list Expression 
	| RANGE_SET Expression Expression -- {,..., }
	| EMPTY_SEQ -- []
	| SEQ Expression_list
	| COMPR_SEQ Expression Bind_list 
	| COMPR_SEQ_COND Expression Bind_list Expression 
	| SUBSEQ Expression Expression Expression
	| EMPTY_MAP -- { |-> }
	| MAP Map_enumeration_list
	| COMPR_MAP Expression Expression Bind_list 
	| COMPR_MAP_COND Expression Expression Bind_list Expression 
	| TUPLE Expression_list
	| RECORD_E String
	| RECORD_S String Expression_list
	| MU Expression Record_modification_list
	| APPLY0 Expression
	| APPLY1 Expression Expression
	| APPLYN Expression Expression Expression_list
	| FIELD_SEL Expression Name
	| TUPLE_SEL Expression Numeral
	| FUNCT_TYPE_INST Name Type_list
	| LAMBDA Type_bind_list Expression
	| IS_IDENT String Expression 
	| IDENT String 
	| OLD_NAME Name
	| SYMB_LIT Symbolic_literal
	deriving (Eq, Show)


data Symbolic_literal = 
	  LIT_NUM Lit_num
	| NUMERAL Numeral 
	| TRUE
	| FALSE
	| NIL
	| LIT_CHAR Lit_char
	| LIT_STR Lit_string
	| LIT_QUOTE Lit_quote
	deriving (Eq, Show)		

data Patternbind_expr_list = 
	  PATT_BIND_EXPR_S Pattern_bind Expression
	| PATT_BIND_EXPR_L Patternbind_expr_list Pattern_bind Expression
	deriving (Eq, Show)	

data Local_definition_list = 
	  LOCAL_DEF_S Local_definition
	| LOCAL_DEF_L Local_definition_list Local_definition
	deriving (Eq, Show)	

data If_expression = 
	  IF_THEN_ELSE Expression Expression Expression
	| IF_THEN_ELSEIF Expression Expression Elseif_expression Expression
	deriving (Eq, Show)

data Elseif_expression 	= 
	  ELSEIF Expression Expression
	| EXP_ELSEIF Elseif_expression Expression Expression
	deriving (Eq, Show)

data Cases_alternative_list = 
	  CASE_ALT_S Pattern_list Expression
	| CASE_ALT_L Cases_alternative_list Pattern_list Expression
	deriving (Eq, Show)

data Name_list = 
	  NAME_S Name
	| NAME_L Name_list Name
	deriving (Eq, Show)
	       
data Name = 
	  IDENTIFIER String
	| MK_ID String
	| IS_ID String
	deriving (Eq, Show)
	  
data Map_enumeration_list = 
	  MAP_ENUM_S Expression Expression
	| MAP_ENUM_L Map_enumeration_list Expression Expression
	deriving (Eq, Show)

data Record_modification_list = 
	  RECORD_MOD_S Name Expression
	| RECORD_MOD_L Record_modification_list Name Expression
	deriving (Eq, Show)

data State_designator = 
	  STATE_S Name
	| STATE_L State_designator Name
	| STATE_EXPR_L State_designator Expression
	deriving (Eq, Show)

-------------------------------------------------------------------------------
-- *** Statements
-------------------------------------------------------------------------------

data Statement_list = 
	  STATEMENT_S Statement
	| STATEMENT_L Statement_list Statement
	deriving (Eq, Show)	

data Statement = 
	  LET_STAT Local_definition_list Statement
	| LET_INSET_STAT Pattern Expression Statement
	| LET_VAR_STAT Name Type Statement
	| LET_PAT_STAT Pattern2 Type Statement
	| LETBE_INSET_STAT Pattern Expression Expression Statement
	| LETBE_VAR_STAT Name Type Expression Statement
	| LETBE_PAT_STAT Pattern2 Type Expression Statement
	| DEF_STAT Equals_definition_list Statement
	| MAYBE_DCL_STAT Maybe_dcl_statement_list Statement_list
	| CALL_STAT Call_statement
	| SKIP
	deriving (Eq, Show)

data Call_statement = 
	  CALL_ID_STAT Name
	| CALL_EXPR_STAT Name Expression_list
	| CALL_STATE_STAT Name State_designator
	| CALL_EXPR_STATE_STAT Name Expression_list State_designator
	deriving (Eq, Show)

data Equals_definition_list = 
	  EQUALS_DEF_S Pattern_bind Expression
	| EQUALS_DEF_L Equals_definition_list Pattern_bind Expression
	deriving (Eq, Show)	

data Maybe_dcl_statement_list = 
	  MAYBE_DCL_STAT_E
	| MAYBE_DCL_STAT_S Maybe_dcl_statement_list Name Type
	| MAYBE_DCL_STAT_L Maybe_dcl_statement_list Name Type Expression
	deriving (Eq, Show)



-------------------------------------------------------------------------------
-- *** Patterns
-------------------------------------------------------------------------------

data Pattern_list = 
	  PATTERN_S Pattern
	| PATTERN_L Pattern_list Pattern
	deriving (Eq, Show)
		  
data Pattern = 
	  PAT_NAME Name
	| PAT_PAT Pattern2
	deriving (Eq, Show)

data Pattern2 = 
	  IDENT_PAT
	| BRACK_PAT Expression
	| SYMB_LIT_PAT Symbolic_literal
	| SET_ENUM_PAT Pattern_list
	| SET_UNION_PAT Pattern Pattern
	| SEQ_ENUM_PAT Pattern_list
	| SEQ_CONC_PAT Pattern Pattern
	| TUPLE_PAT Pattern_list
	| LIST_PAT Name Brack_maybe_pattern_list
	deriving (Eq, Show)

data Pattern_bind = 
	  PAT_BIND_PAT Pattern
	| PAT_BIND_BIND Bind
	deriving (Eq, Show)

data Bind = 
	  BIND_EXPR Pattern Expression
	| BIND_TYPE Pattern Type
	deriving (Eq, Show)

data Bind_list = 
	  BIND_S1 Multiple_set_bind
	| BIND_S2 Multiple_type_bind
	| BIND_L1 Bind_list Multiple_set_bind
	| BIND_L2 Bind_list Multiple_type_bind
	deriving (Eq, Show)

data Multiple_set_bind = 
	  MULT_SET_BIND Pattern_list Expression
	  deriving (Eq, Show)

data Multiple_type_bind = 
	  MULT_TYPE_BIND Pattern_list Type
	  deriving (Eq, Show)

data Type_bind_list = 
	  TYPE_BIND_S Pattern Type
	| TYPE_BIND_L Type_bind_list Pattern Type
	deriving (Eq, Show)	

data Local_definition = 
	  L_PAT Pattern Expression
	| L_NAME Name Type Expression
	| L_PAT_TYPE Pattern2 Type Expression
	| L_EXPL_FUNC Name Function_type Name Parameters_list Expression Maybe_precondition
	| L_EXPL_FUNC_TYPE Name Type_variable_list Function_type Name Parameters_list Expression Maybe_precondition
	| L_IMPL_FUNC Name Parameter_type_list Name Type Maybe_precondition Expression
	| L_IMPL_FUNC_TYPE Name Type_variable_list Parameter_type_list Name Type Maybe_precondition Expression
	deriving (Eq, Show)
	
-------------------------------------------------------------------------------

