-------------------------------------------------------------------------------
--
-- | 
-- Module      :  Language.VDM_SL.Optimize
-- Copyright   :  (c) Tiago Alves, Paulo Silva, 2004
-- License     :  LGPL
-- 
-- Maintainer  :  joost.visser@di.uminho.pt
-- Stability   :  experimental
-- Portability :  portable
--
-- This module ??.
--
-------------------------------------------------------------------------------

module Language.VDM_SL.TransfEngine(applyTransf) where

import Language.VDM_SL.Syntax
import Language.VDM_SL.SyntaxTermInstances
import StrategyLib
import Language.VDM_SL.HLStratLib
import Control.Monad
import Control.Monad.Identity
import Language.VDM_SL.Optimize
import Language.VDM_SL.FuncLib
import Language.VDM_SL.AuxFuncLib

-- | Apply all transformations to a VDM_SL document.
applyTransf :: (Monad m) => Document -> m Document
applyTransf doc = 
   extractTypeList doc >>= constrRelType >>= transformation >>= addToDocument doc

-- Type Definition extraction
extractTypeList :: (Monad m) => Document -> m [Type_definition]
extractTypeList = stratTU getTypes 
	where 
		getTypes :: Type_definition_list -> [Type_definition]
		getTypes (TYPE_DEF x) = x

-- Definition of a new type with the necessary information
constrRelType :: (Monad m) => [Type_definition] -> m [Field] 
constrRelType tlst = toFieldList tlst >>= normalize  

toFieldList :: (Monad m) => [Type_definition] -> m [Field]
toFieldList = stratTU ap
	where
		ap :: Type_definition -> [Field]
		ap (TYPE_TYPE n t) = [FIELD_NAME n t]
		ap (TYPE_TYPE_INV n t i) = [FIELD_NAME n t]
		ap (TYPE_FIELD_L n flst) = [FIELD_NAME n (COMPOSE n flst)]
		ap (TYPE_FIELD_L_INV n flst i) = [FIELD_NAME n (COMPOSE n flst)]

-- Adds the new relational model to the VDM document
addToDocument :: (Monad m) => Document -> [Field] -> m Document
addToDocument doc flst = return (DEF_BLOCK_L doc (TYPES (TYPE_DEF [constr flst])))
	where
		constr :: [Field] -> Type_definition
		constr flst = TYPE_TYPE (IDENTIFIER "RELATIONAL_MODEL")
					(COMPOSE (IDENTIFIER "RELATIONAL_MODEL") 
						 (FIELD_LIST flst))

-- Normalizes the data types representation
normalize :: (Monad m) => [Field] -> m [Field] 
normalize flst = do
	dep <- extractDependencies flst
	tc  <- return ( transClosure dep )
	rl  <- 	if (verifyRecursive tc) 
		then error "Recursive data type definition" 
		else return (removeUsedTypes tc flst)
	l   <- replaceRef flst rl
	removeAtomic l

-- For every data type, extracts its references to other user defined types
extractDependencies :: (Monad m) => [Field] -> m [(String, [String])]
extractDependencies = strat_allTU ap 
	where 
		ap :: (Monad m) => Field -> m [(String, [String])]
		ap (FIELD_NAME x t) = add x t
		ap _ = return []
		add :: (Monad m) => Name -> Type -> m [(String, [String])]
		add n t = do 
			str <- stratTU ap2 t
			return [ (getIdentifier n, str) ]
		ap2 :: Type -> [String]
		ap2 (NAME_TYPE x) = [getIdentifier x]
		ap2 _ = []

-- Calculates the transitive closure on the references between data types
transClosure :: [(String, [String])] -> [(String, String)]
transClosure lst = 
	taux dep dep
	where
		dep = filterEquals [ (a,b) | (a, dep) <- lst, b <- dep ] 
		taux :: [(String, String)] -> [(String, String)] -> [(String, String)]
		taux ori tc = 
			let compost = tc `union` composition ori tc
			in if (tc == compost) then tc else taux ori compost

-- Verifies if there is any recursive data type definition
verifyRecursive :: [(String, String)] -> Bool
verifyRecursive = not . null . filter (\(x,y) -> x==y)

-- Removes the data types definitions of referenced types
removeUsedTypes :: [(String, String)] -> [Field] -> [Field]
removeUsedTypes tc = concatMap ap 
	where 
		ref = rng tc
		ap (t@(FIELD_NAME (IDENTIFIER id) _)) = 
			if elem id ref then [] else [t]
			
-------------------------------------------------------------------------------

-- Removes all the types that are atomic, i.e., that do not contain sets,
-- seqs or maps
removeAtomic :: (Monad m) => [Field] -> m [Field]
removeAtomic = strat_allTU rem 
	where
		rem :: (Monad m) => Field -> m [Field]
		rem fld = do 
			lst <- stratTU ap fld
			if (or lst) then return [fld] else return []
		ap :: Type -> [Bool]
		ap x = [isComposed x]

-- Replaces the reference to a data type by its definition
replaceRef :: (Monad m) => [Field] -> [Field] -> m [Field]
replaceRef flst rlst = stratTP (ap flst) rlst
	where 
		ap :: [Field] -> Type -> Type
		ap lst (NAME_TYPE x) = aux lst x
		ap lst x = x
		aux :: [Field] -> Name -> Type
		aux [] x = COMPOSE x (FIELD_LIST [])
		aux ((FIELD_NAME id t):xs) x = 
			if x == id 
			then case t of 
				COMPOSE n lst -> t
				other -> COMPOSE x (FIELD_LIST [FIELD_NAME x t])
			else aux xs x



type TransfFunction = Type -> Type

transformation :: (Monad m) => [Field] -> m [Field] 
transformation tdef = 
	stratTP transf1 tdef 
	>>= stratTP transf2 
	>>= stratTP transf3
	>>= stratTP transf4
	>>= return . transf5
    	>>= return . optimizeComposes
    	>>= return . optimizeTables
	>>= stratTP transf6
	
-- transf1: SEQ x = MAP NAT x
transf1 :: TransfFunction
transf1 ( SEQ0_OF CHAR ) = SEQ0_OF CHAR
transf1 ( SEQ1_OF CHAR ) = SEQ1_OF CHAR
transf1 ( SEQ0_OF x )    = 
	MAP_TYPE (COMPOSE (IDENTIFIER "seqID") 
		(FIELD_LIST [FIELD_NAME (IDENTIFIER "seqID") NAT ])) x
transf1 ( SEQ1_OF x )    = 
	MAP_TYPE (COMPOSE (IDENTIFIER "seqID") 
		(FIELD_LIST [FIELD_NAME (IDENTIFIER "seqID") NAT ])) x
transf1 x                = x

-- transf2: SET x = MAP x NIL
transf2 :: TransfFunction
transf2 ( SET_OF x ) = MAP_TYPE x NIL_TYPE
transf2 x            = x

-- transf3: A + B -> C = (A->C) * (B->C)
transf3 :: TransfFunction
transf3 ( MAP_TYPE (UNION_TYPE a b) c ) = 
	COMPOSE (IDENTIFIER "SPLIT1") (FIELD_LIST [
		FIELD_NAME (IDENTIFIER "C1") (MAP_TYPE a c),
		FIELD_NAME (IDENTIFIER "C2") (MAP_TYPE b c)])
transf3 ( MAP_TYPE (COMPOSE x (FIELD_LIST [FIELD_NAME y (UNION_TYPE a b)])) c) = 
	COMPOSE x (FIELD_LIST [
		FIELD_NAME (IDENTIFIER (getIdentifier y ++ "1")) (MAP_TYPE a c),
		FIELD_NAME (IDENTIFIER (getIdentifier y ++ "2")) (MAP_TYPE b c)])
transf3 x = x

--transf4: A -> (B +C) = (A->B) * (A->C)
transf4 :: TransfFunction
transf4 (MAP_TYPE a (UNION_TYPE b c)) = 
	COMPOSE (IDENTIFIER "SPLIT2") (FIELD_LIST [
		FIELD_NAME (IDENTIFIER "C1") (MAP_TYPE a b),
		FIELD_NAME (IDENTIFIER "C2") (MAP_TYPE a c)])
transf4 (MAP_TYPE a (COMPOSE x (FIELD_LIST [FIELD_NAME y (UNION_TYPE b c)]))) = 
	COMPOSE x (FIELD_LIST [
		FIELD_NAME (IDENTIFIER (getIdentifier y ++ "1")) (MAP_TYPE a b),
		FIELD_NAME (IDENTIFIER (getIdentifier y ++ "2")) (MAP_TYPE a c)])
transf4 x = x

--transf5: A -> (B*(C->D) = (A->B) * (A*C->D)
transf5 :: [Field] -> [Field]
transf5 = applyIter (concat . map func)

-- | 
func :: Field -> [Field]
func x@(FIELD_NAME ident (MAP_TYPE fromType toType))
   | isAnyComposed toType = applyDist (getIdentifier ident) fromType toType
   | otherwise = [x]
func x = [x]

-- | Function that transverses the type and returns true if there are
-- at least one composed type (MAP, SEQx or SET)
-- TODO: use a different Strafunski strategy to compute the same result
isAnyComposed a = or $ runIdentity (stratTU (\x -> [isComposed x]) a)

-- | Function that applied distibutivity
applyDist :: String -> Type -> Type -> [Field]
applyDist str fromType (MAP_TYPE x1 x2) =  -- case 1) X -> (A -> B)
   [ FIELD_NAME (IDENTIFIER (str ++ "_1")) (MAP_TYPE fromType NIL_TYPE)
   , FIELD_NAME (IDENTIFIER (str ++ "_2")) (MAP_TYPE (composeFusion fromType x1) x2)]
applyDist str fromType (COMPOSE name (FIELD_LIST lst)) = -- case 2) X -> (A * (B -> C) * ...)
   [ applyDist2 str fromType x | x <- lst] 
applyDist _ _ x = [] 


-- | 
applyDist2 :: String -> Type -> Field -> Field
applyDist2 str fromType (FIELD_NAME ident vdmType)
   | isComposed vdmType = FIELD_NAME (IDENTIFIER str) (MAP_TYPE fromType vdmType)
   | otherwise = FIELD_NAME new_id (MAP_TYPE fromType vdmType)
   where 
   	new_id = IDENTIFIER (str ++ "_" ++ getIdentifier(ident))
        toType = COMPOSE ident (FIELD_LIST [FIELD_NAME ident vdmType])



--transf6: MAP x NIL = SET x
transf6 :: TransfFunction
transf6 ( MAP_TYPE x NIL_TYPE ) = SET_OF x
transf6 x		       = x

   
