module Main where

import CGI hiding (div, head, map, span)
import System
import CGIGraphics
import Language.Java.ChaseImports
import Language.Java.PackageGraph
import Data.List (isSuffixOf)
import Data.Set
import Data.Relation.SetOfPairs

-----------------
-- * Main program

main :: IO ()
main = run mainCGI

mainCGI :: CGI ()
mainCGI = graphViewerNoImageCGI
    
-----------------
-- * File upload
 
uploadFileCGI :: Maybe FileReference -> CGI ()
uploadFileCGI maybeFileRef = 
  standardQuery "File Menu" $
  do displayCurrentFile
     text_S "Enter file to upload "
     fileH <- checkedFileInputField refuseUnnamed empty
     submit fileH anyGraphViewerCGI (fieldVALUE "UPLOAD")
  where
    displayCurrentFile = case maybeFileRef of
      Nothing -> <p><text><% "No file selected yet" %></text></p>
      (Just fileRef) 
        -> <p><text>
           <% "Current file: "++(fileReferenceExternalName fileRef) %>
           </text></p>

refuseUnnamed mf = 
  do FileReference {fileReferenceExternalName=frn} <- mf
     if null frn then fail "" else mf

fileButton :: Maybe FileReference -> WithHTML x CGI ()
fileButton maybeFileRef =
  submit0 (uploadFileCGI maybeFileRef) (fieldVALUE "File")
  
-------------------

chopButton :: FileReference -> WithHTML x CGI ()
chopButton fileRef =
  submit0 (chopParametersCGI fileRef) (fieldVALUE "Chop")

chopParametersCGI :: FileReference -> CGI ()
chopParametersCGI fileRef = do 
  nodes <- getNodes fileRef
  standardQuery "Chop Parameters" $ do
    <p>
    <table>
    <tr><td> Sources </td><td> Sinks </td></tr>
    <tr>
      <td><% sourcesH <- selectMultiple id [] nodes (0,maxBound) empty %></td>
      <td><% sinksH <- selectMultiple id [] nodes (0,maxBound) empty %></td>
    </tr>
    <tr>
      <td>
      <% submit (F2 sourcesH sinksH) (chopResultsCGI fileRef) (fieldVALUE "Chop now") %>
      </td>
    </tr>
    </table>
    </p>

chopResultsCGI :: FileReference 
               -> F2 (InputField [PGNode]) (InputField [PGNode]) VALID 
               -> CGI ()
chopResultsCGI fileRef (F2 sourcesH sinksH) = do
  pg <- getPackageGraph fileRef
  let sources = value sourcesH
  let sinks = value sinksH
  let pgChop = sliceOrChopWith intersect sources sinks (pgAsRel pg)
  let pg' = projectPG' pgChop pg
  packageGraphViewerCGI fileRef pg'

getPackageGraph :: FileReference -> CGI PackageGraph
getPackageGraph fileRef = io $ do
  maybeCU <- parseJavaFile (fileReferenceName fileRef)
  java2pcg' id "" maybeCU
  
getNodes :: FileReference -> CGI [PGNode]
getNodes fileRef = io $ do
  maybeCU <- parseJavaFile (fileReferenceName fileRef)
  (r1,r2,r3,r4) <- java2pcg' id "" maybeCU
  return . setToList $ (ent r1)`union`(ent r2)`union`(ent r3)`union`(ent r4) 
     
-------------------

graphViewerCGI :: WithHTML x CGI z -> WithHTML x' CGI z' -> CGI ()
graphViewerCGI menu image = do
  standardQuery "Graph Viewer" $
    <table>
      <tr><% menu %></tr>
      <tr><td><% image %></td></tr>
    </table>

graphViewerNoImageCGI :: CGI ()
graphViewerNoImageCGI = do
  graphViewerCGI graphViewerNoFileMenu image
  where
    image = <text><% "no file selected" %></text>

graphViewerNoFileMenu :: WithHTML x CGI ()
graphViewerNoFileMenu = do 
  <td><% fileButton Nothing %></td>

graphViewerMenu :: FileReference -> WithHTML x CGI ()
graphViewerMenu fileRef = do 
  <td><% fileButton (Just fileRef) %></td>
  <td><% chopButton fileRef %></td>

packageGraphViewerCGI :: FileReference -> PackageGraph -> CGI ()  
packageGraphViewerCGI fileRef pg = do
  let dotString = pg2dot "pg" pg
  dotGraphViewerCGI fileRef dotString

dotGraphViewerCGI :: FileReference -> String -> CGI ()  
dotGraphViewerCGI fileRef dotString = do
  imageFileName <- io $ callDot dotString "gif" "PackageGraph"
  let image = presentImage imageFileName
  graphViewerCGI (graphViewerMenu fileRef) image

anyGraphViewerCGI :: InputField FileReference VALID -> CGI ()
anyGraphViewerCGI fileH = do
  appropriateCGI
  where
   fileRef = value fileH
   fileName = fileReferenceExternalName fileRef
   appropriateCGI 
    | ".java" `isSuffixOf` fileName = do
        pg <- getPackageGraph fileRef
        packageGraphViewerCGI fileRef pg
    | ".dot" `isSuffixOf` fileName = do
        dotString <- io $ readFile (fileReferenceName fileRef)
        dotGraphViewerCGI fileRef dotString
    | otherwise = graphViewerNoImageCGI
  
-------------------

-- * Graph viewer

{-

menuOnly = displayWithMenu Nothing

menuWithImage fileH
  = displayWithMenu (Just fileH)

displayWithMenu :: Maybe (InputField FileReference VALID) -> CGI ()
displayWithMenu maybeFileH = do
  imageFileName <- tryCreateImage maybeFileRef
  standardQuery "ChopaChops: graph view." $
    <table>
      <tr><% menu %></tr>
      <tr><td><% maybePresentImage imageFileName %></td></tr>
    </table>
  where
    maybeFileRef = maybeMap value maybeFileH
    menu = do <td><% fileButton maybeFileRef %></td>
              <td><% chopButton' maybeFileRef %></td>
    
chopButton' (Just fileRef) = chopButton fileRef

tryCreateImage :: Maybe FileReference -> CGI (Maybe FilePath)
tryCreateImage Nothing
  = return Nothing
tryCreateImage (Just fileRef)
  | ".java" `isSuffixOf` (fileReferenceExternalName fileRef)
  = createJavaImage fileRef
  | ".dot" `isSuffixOf` (fileReferenceExternalName fileRef)
  = createDotImage fileRef
  | otherwise
  = return Nothing

createDotImage :: FileReference -> CGI (Maybe FilePath)
createDotImage fileRef = io $ do
  content <- readFile (fileReferenceName fileRef)
  fileName <- callDot content "gif" (fileReferenceExternalName fileRef)
  return (Just fileName)

createJavaImage :: FileReference -> CGI (Maybe FilePath)
createJavaImage fileRef = io $ do
  maybeCU <- parseJavaFile (fileReferenceName fileRef)
  pg <- java2pcg' id "" maybeCU
  let content = pg2dot "pg" pg
  fileName <- callDot content "gif" (fileReferenceExternalName fileRef)
  return (Just fileName)

maybePresentImage Nothing 
  = <text><% "no file selected" %></text>
maybePresentImage (Just imageFileName) 
  = presentImage imageFileName  


maybeMap :: (a->b) -> Maybe a -> Maybe b
maybeMap f
  = maybe Nothing (Just . f)
-}
{-
display :: InputField FileReference VALID -> CGI ()
display fileH =
  let fileRef = value fileH in
  standardQuery "Upload Successful" $
  do <p><text><% "Hoi" %></text></p>
     p_T (text (show fileRef))
     p_T (do text_S "Check file contents "
             submit0 (tell fileRef) (fieldVALUE "Show contents"))
     p_T (do text_S "Show file statistics "
             submit0 (statistics fileRef) (fieldVALUE "Statistics"))
     <p><% submit0 dotPage (fieldVALUE "Render") %></p>
-}
{-
mainCGI' =
  ask <html>
      <head><title>Hello World</title></head>
      <body>
      <h1>Hello World</h1>
      </body>
      </html>
-}

presentImage fileName
  = <img src=<% fileName %> alt=<% fileName %> />
  
callDot dotString outFormat fileBaseName
  = do writeFile dotFileName dotString
       putStrLn ("Command: "++dotCommand)
       system dotCommand
       return imageFileName
    where
      dotFileName = "/tmp/"++fileBaseName++".dot"
      imageFileName  = "/CGI-Files/"++fileBaseName++"."++outFormat
      dotCommand = "dot -T"++outFormat++" "++dotFileName++
                   " > "++documentRoot++imageFileName

documentRoot = "/Library/WebServer/Documents"
          