-----------------------------------------------------------------------------
-- |
-- Maintainer  :  joost.visser@di.uminho.pt
-- Stability   :  experimental
-- Portability :  portable
--
-- Online GUI for ChopaChops.
--
-----------------------------------------------------------------------------

module Main where

import CGI hiding (div, head, map, span)
import System
import CGIGraphics
import Language.Java.ChaseImports
import Language.Java.PackageGraph
import Data.List (isSuffixOf)
import Data.Set
import Data.Relation.SetOfPairs

import Control.ModelViewController
import Control.WashExtras

-----------------------------------------------------------------------------
-- * Main program.

main :: IO ()
main = run chopaChopsCGI -- mainCGI

-- | File upload page is start page.
mainCGI :: CGI ()
mainCGI = uploadCGI Nothing

-----------------------------------------------------------------------------
-- * Various pages.

-- | File upload.
uploadCGI :: Maybe FileReference -> CGI ()
uploadCGI maybeFileRef = 
  standardQuery "File Menu" $
  do displayCurrentFile
     mkFormBody "Upload" "Browse: " anyGraphViewerCGI undefined
  where
    displayCurrentFile = case maybeFileRef of
      Nothing -> <p><text><% "No file selected yet" %></text></p>
      (Just fileRef) 
        -> <p><text>
           <% "Current file: "++(fileReferenceExternalName fileRef) %>
           </text></p>

-- | Chopping package graphs.
chopResultsCGI :: FileReference 
               -> ([PGNode],[PGNode]) 
               -> CGI ()
chopResultsCGI fileRef (sources,sinks) = do
  pg <- getPackageGraph fileRef
  let pgChop = sliceOrChopWith intersect sources sinks (pgAsRel pg)
  let pg' = projectPG' pgChop pg
  packageGraphViewerCGI fileRef pg'

-- | Viewing arbitrary graphs (java package graphs or raw dot graphs at
--   the moment).
anyGraphViewerCGI :: FileReference -> CGI ()
anyGraphViewerCGI fileRef = do
  appropriateCGI
  where
   fileName = fileReferenceExternalName fileRef
   appropriateCGI 
    | ".java" `isSuffixOf` fileName = do
        pg <- getPackageGraph fileRef
        packageGraphViewerCGI fileRef pg
    | ".dot" `isSuffixOf` fileName = do
        dotString <- io $ readFile (fileReferenceName fileRef)
        dotGraphViewerCGI fileRef dotString
    | otherwise = graphViewerNoImageCGI
  
-- | Viewing a Java package graph.
packageGraphViewerCGI :: FileReference -> PackageGraph -> CGI ()  
packageGraphViewerCGI fileRef pg = do
  let dotString = pg2dot "pg" pg
  dotGraphViewerCGI fileRef dotString

-- | Viewing a raw dot graph.
dotGraphViewerCGI :: FileReference -> String -> CGI ()  
dotGraphViewerCGI fileRef dotString = do
  imageFileName <- io $ callDot dotString "gif" "PackageGraph"
  let image = presentImage imageFileName
  graphViewerCGI (graphViewerMenu fileRef) image

-- | Instantiation of general graph viewer without image.
graphViewerNoImageCGI :: CGI ()
graphViewerNoImageCGI = do
  graphViewerCGI menu image
  where
    image = <text><% "no file selected" %></text>
    menu = simpleMenuFor [fileButton Nothing]

-- | General graph viewer.
graphViewerCGI :: WithHTML x CGI z -> WithHTML x' CGI z' -> CGI ()
graphViewerCGI menu image = do
  standardQuery "Graph Viewer" $ do
    <table>
      <tr><td><% menu %></td></tr>
    </table>
    <table>
      <tr><td><% image %></td></tr>
    </table>

-- | Menu for the general graph viewer.
graphViewerMenu :: FileReference -> WithHTML x CGI ()
graphViewerMenu fileRef = 
  simpleMenuFor [ fileButton (Just fileRef), chopButton fileRef]
{-
  fullMenuFor [ ("FILE",[fileButton (Just fileRef)]),
                ("CHOP",[chopButton fileRef]),
                ("MANY",[fileButton (Just fileRef),chopButton fileRef])]
-}

-- | Create HTML image element from image file name.
presentImage :: FilePath -- ^ Image file name
             -> WithHTML x CGI ()
presentImage fileName
  = <img src=<% fileName %> alt=<% fileName %> />

-----------------------------------------------------------------------------
-- * Menus and continuation buttons.

fileButton :: Maybe FileReference -> MenuButton
fileButton maybeFileRef = ("File", uploadCGI maybeFileRef)
  
chopButton :: FileReference -> MenuButton
chopButton fileRef = ("Chop", chopParametersCGI fileRef) 

chopParametersCGI fileRef = do
  nodes <- getNodes fileRef
  mkForm "Chop NOW" ("Sources","Sinks") (chopResultsCGI fileRef) (nodes,nodes)

-- getNodes :: FileReference -> CGI [PGNode]
getNodes fileRef = io $ do
  maybeCU <- parseJavaFile (fileReferenceName fileRef)
  (r1,r2,r3,r4) <- java2pcg' id "" maybeCU
  return . setToList $ (ent r1)`union`(ent r2)`union`(ent r3)`union`(ent r4) 
  
-----------------------------------------------------------------------------
-- * Various IO operations.
  
getPackageGraph :: FileReference -> CGI PackageGraph
getPackageGraph fileRef = io $ do
  maybeCU <- parseJavaFile (fileReferenceName fileRef)
  java2pcg' id "" maybeCU
  
-- | Call the GraphViz program dot.
callDot :: String      -- ^ Content for dot file. 
        -> String      -- ^ Output format, e.g. gif.
        -> FilePath    -- ^ Base name of image file.
        -> IO FilePath -- ^ Full file name of produced image file.
callDot dotString outFormat fileBaseName
  = do writeFile dotFileName dotString
       putStrLn ("Command: "++dotCommand)
       system dotCommand
       return imageFileName
    where
      dotFileName = "/tmp/"++fileBaseName++".dot"
      imageFileName  = "/CGI-Files/"++fileBaseName++"."++outFormat
      dotCommand = "dot -T"++outFormat++" "++dotFileName++
                   " > "++documentRoot++imageFileName

-- | Set the document root of the webserver.
documentRoot = "/Library/WebServer/Documents"


-----------------------------------------------------------------------------
-- * Model View Controller

-- | Controller
chopaChopsController :: Controller PackageGraph
chopaChopsController = Controller {
  initModel = \fileRef -> do
    maybeCU <- parseJavaFile (fileReferenceName fileRef)
    java2pcg' id "" maybeCU
,      
  modelOperations = [
    ("Chop",chopPackageGraph)
  ]
,
  modelViews = [
    ("Package Graph (gif)",packageGraphView "gif"),
    ("Package Graph (jpg)",packageGraphView "jpg")
  ]
}

packageGraphView format = MV () () (\pg x -> do
  let dotString = pg2dot "pg" pg
  imageFileName <- ioHTML $ callDot dotString format "PackageGraph"
  let image = presentImage imageFileName
  <table><tr><td><% image %></td></tr></table>)

chopPackageGraph = MO ("Sources","Sinks") 
  (\(r1,r2,r3,r4) -> 
    let nodes 
         = setToList $ (ent r1)`union`(ent r2)`union`(ent r3)`union`(ent r4)
    in (nodes,nodes) 
  ) 
  (\pg (sources,sinks) -> 
    let pgChop = sliceOrChopWith intersect sources sinks (pgAsRel pg)
    in projectPG' pgChop pg
  )
  

chopaChopsCGI = mvc chopaChopsController

-----------------------------------------------------------------------------

