-----------------------------------------------------------------------------
-- |
-- Maintainer  :  joost.visser@di.uminho.pt
-- Stability   :  experimental
-- Portability :  portable
--
-- Slicing and chopping package Java package graphs.
--
-----------------------------------------------------------------------------

module Main where

import System.Console.GetOpt
import System.Environment
import Control.Monad

import ChaseImports (ChaseName)

import Language.Java.Syntax
import Language.Java.Util
import Language.Java.ChaseImports
import Language.Java.PackageGraph

import Data.Relation.SetOfPairs
import Data.Set

-----------------------------------------------------------------------------
-- * Main program and option handling.

-- | Main program.
main :: IO ()
main = do 
   args <- getArgs
   (flags,str)  <- cmdLine args
   if (Help `elem` flags || null flags) then provideHelp else do
   case (reqArgsFromFlags flags) of
     Nothing -> putStrLn cmdUsageInfo
     (Just (cp,is,o)) 
       -> jreach (breakPath cp) is o 
             (binop flags)
             (justFilter unFrom flags)
             (justFilter unTo flags)
             (justFilter unAddFrom flags)
             (justFilter unAddTo flags)
   where
      unFrom (SliceFrom s) = Just s
      unFrom _            = Nothing
      unTo (SliceTo s)     = Just s
      unTo _              = Nothing
      unAddFrom (AddSliceFrom s) = Just s
      unAddFrom _            = Nothing
      unAddTo (AddSliceTo s)     = Just s
      unAddTo _              = Nothing
      binop []              = intersect
      binop (BinOp "I" : _) = intersect
      binop (BinOp "U" : _) = union
      binop (_ : flags)     = binop flags

-- | Print usage information and graph visualization legend.
provideHelp :: IO ()
provideHelp
  = do putStrLn cmdUsageInfo
       writeFile "legend.dot" (pg2dot "pg" legendPG)
       putStrLn "Wrote package graph legend to file \"legend.dot\"."
     
-- | Extract the required arguments from the list of command line flags.
reqArgsFromFlags :: [Flag] -> Maybe (FilePath,[FilePath],FilePath)
reqArgsFromFlags flags
  = case (filter isCP flags, justFilter unIn flags, filter isOut flags) of
      ([ClassPath cp],is@(i:_),[OutputDot o]) 
         -> Just (cp,is,o)
      _  -> Nothing
    where
      isCP (ClassPath s) = True
      isCP _             = False
      unIn (InputJava s) = Just s
      unIn _             = Nothing
      isOut (OutputDot s) = True
      isOut _             = False

-- | Type of command line flags.
data Flag = ClassPath FilePath 
	  | InputJava FilePath
	  | OutputDot FilePath
	  | SliceFrom String
	  | SliceTo String
	  | AddSliceFrom String
	  | AddSliceTo String
	  | BinOp String
	  | Help
	  deriving (Eq, Show)

-- | Specification of command line flags.
flags :: [OptDescr Flag]
flags = 
  [ Option ['i'] ["input"] 
    (ReqArg InputJava "type") "Java class or interface"
  , Option ['o'] ["output"] 
    (ReqArg OutputDot "file") "Dot output file"
  , Option ['c'] ["classpath"] 
    (ReqArg ClassPath "path") "Class path"
  , Option ['f'] ["from"]
    (ReqArg SliceFrom "from") "Keep slice from ..."
  , Option ['t'] ["to"]
    (ReqArg SliceTo "to") "Keep slice to ..."
  , Option ['F'] ["From"]
    (ReqArg AddSliceFrom "From") "Add slice from ..."
  , Option ['T'] ["To"]
    (ReqArg AddSliceTo "To") "Add slice to ..."
  , Option ['I'] ["Intersect"]
    (NoArg (BinOp "I")) "Intersect 'Add' slice and 'Keep' slice (default)."
  , Option ['U'] ["Union"]
    (NoArg (BinOp "U")) "Take union of 'Add' slice and 'Keep' slice."
  , Option ['h','?'] ["help"]
    (NoArg Help) "Show this help screen"
  ]

-- | Parse the command line into flags and remaining arguments.
cmdLine :: [String] -> IO ([Flag], [String])
cmdLine argv = 
	case (getOpt RequireOrder flags argv) of
		(o, n, []) -> return (o,n)
		(_,_,errs) -> cmdLineError errs

-- | Print message to signal error in the command line.
cmdLineError :: [String] -> IO a
cmdLineError errs = ioError $ userError $ concat errs ++ cmdUsageInfo

-- | Command usage information.
cmdUsageInfo :: String
cmdUsageInfo = usageInfo header flags
	where header = "Usage: "

-- | Utility function.
justFilter :: (a -> Maybe b) -> [a] -> [b]
justFilter f [] = []
justFilter f (a:as) 
  = case (f a) of
      Nothing    -> justFilter f as
      (Just b)   -> (b:justFilter f as) 
   
-----------------------------------------------------------------------------
-- * Core functionality

-- | Extract package graph, perform chopping or slicing, and print result to
--   GraphViz format.
jreach :: [FilePath]    -- ^ class path
       -> [ChaseName]   -- ^ java input types
       -> FilePath      -- ^ output file
       -> (Rel PGNode PGNode -> Rel PGNode PGNode -> Rel PGNode PGNode)
       -> [Identifier]  -- sources ('keep')
       -> [Identifier]  -- sinks   ('keep')
       -> [Identifier]  -- sources ('add')
       -> [Identifier]  -- sinks   ('add')
       -> IO ()
jreach cp is o binop sources sinks sourcesU sinksU = do
  cus <- javaChaseImports cp is
  pg <- java2pcg' id "" cus
  let pgChopI = sliceOrChopWith intersect sources sinks (pgAsRel pg)
  let pgChopU = sliceOrChopWith union sourcesU sinksU (pgAsRel pg)
  let pgChop = pgChopI `binop` pgChopU
  let pg' = projectPG' pgChop pg
  writeFile o (pg2dot "pg" pg')
  
-----------------------------------------------------------------------------



