module VDM2PLSql(vdm2plsql) where

import Language.VDM_SL.Syntax
import Language.VDM_SL.SyntaxTermInstances
import Language.PLSql.Syntax
import StrategyLib
import Language.VDM_SL.HLStratLib
import Control.Monad
import Control.Monad.Identity


-- | 
vdm2plsql :: (Monad m) => Document -> m [Create_table_statement]
vdm2plsql doc = 
	applyTU ( full_tdTU ( adhocTU (constTU []) (return . ap) ) ) doc >>=
        return . applyTransl . cleanTypes . head
	where
		ap :: Type_definition -> [Type_definition]
		ap (t@(TYPE_TYPE (IDENTIFIER "RELATIONAL_MODEL") x )) = [t]
                ap _ = []

-- | 
applyTransl :: Type_definition -> [Create_table_statement]
applyTransl (TYPE_TYPE _ vdmType) = map vdmTable2sqlTable $ getTables vdmType


-- | This function cleans the structure by transforming everything to
-- maps. This simplifies the translation between VDM and PLSql.
cleanTypes :: (Term t) => t -> t
cleanTypes = runIdentity . stratTP step
   where step (BRACK_TYPE vdmType) = vdmType
         step x@(SEQ0_OF CHAR) = x
         step x@(SEQ1_OF CHAR) = x
         step (SEQ0_OF vdmType) = MAP_TYPE (setPK) vdmType
         step (SEQ1_OF vdmType) = MAP_TYPE (setPK) vdmType
         step a = a
         setPK = COMPOSE (IDENTIFIER "setID") (FIELD_LIST [FIELD_NAME (IDENTIFIER "setID") (NAT)])


-- | This function takes a VDM compose data type a returns a list of pairs
-- containing the name of the tag and the type of the tag. This is equivalent
-- to the name and the body of the tables to be created in sql.
-- This function must be called in the top-level compose
getTables :: Type -> [(String, Type)]
getTables (COMPOSE _ (FIELD_LIST lst)) = map field2pair lst


-- | This function separates the data inside a compose field, spliting 
-- the tag name from the data type creating a tuple.
field2pair :: Field -> (String, Type)
field2pair (FIELD_NAME name vdmType) = (fldName, vdmType)
   where (IDENTIFIER fldName) = name


-- | This function converts a VDM data type into its equivalent in SQL
-- The type can be a "set" or "map". The first case don't have primary
-- keys
vdmTable2sqlTable :: (String, Type) -> Create_table_statement
vdmTable2sqlTable (tableName, vdmCols) =
   let columns = collectWith field2pair vdmCols
       sqlTableItems = map defSqlColumn columns
       pkdef = case (getPrimaryKeyDef vdmCols) of 
                  Just pk -> [pk]
                  Nothing -> []
   in sqlTable tableName (sqlTableItems ++ pkdef)


-- | Function that gets the primary key for the sql table.
-- If the type is a MAP then get its domain, otherwise (a SET) we will not
-- use a primary key.
getPrimaryKeyDef :: Type -> Maybe Table_item
getPrimaryKeyDef (MAP_TYPE vdmType _ ) = Just (defPrimaryKeys lst)
   where lst = collectWith (fst . field2pair) vdmType
getPrimaryKeyDef _ = Nothing


-- | Converts a primitive VDM datatype in a scalar SQL datatype.
-- VDM primitive types are: bool, nat, nat1, int, rat, real, char and token.
-- The "seq of char" will be considered primitive, because it is used to
-- represent a string.
-- 
-- Using the PL/SQL language to generate SQL92 standard code it is a very
-- difficult task because PL/SQL it is not a super set of the SQL92, as we
-- initially thought. Thus, we found that there are SQL92 scalar data types
-- that do not exist in PL/SQL. For instance SMALLINT, FLOAT and DECIMAL
-- must be converted to INTEGER (Datatype_c2).
-- The SQL92 Standard does not allow arbitrary length strings, but PL/SQL do.
-- We opted to use the PL/SQL data type because it is a very common data type.
-- Some VDM data types are stricter than the correspondent ones in SQL. This
-- will be solved later by adding constraints to each type.
-- TODO: Enumerated types are not handled and "seq1 of char" are not tested
vdm2sqlType :: Type -> Datatype
vdm2sqlType (BOOL)  = Datatype_c2 -- Booleans: 0 or 1
vdm2sqlType (NAT)   = Datatype_c2 -- Natural numbers with 0
vdm2sqlType (NAT1)  = Datatype_c2 -- Natural numbers without 0
vdm2sqlType (INT)   = Datatype_c2 -- Integer numbers
vdm2sqlType (RAT)   = Datatype_c2 -- Rational numbers
vdm2sqlType (REAL)  = Datatype_c2 -- Real numbers
vdm2sqlType (CHAR)  = Datatype_c6 -- Characters
vdm2sqlType (TOKEN) = Datatype_c9 -- Tokens
vdm2sqlType (SEQ0_OF CHAR) = Datatype_c9
vdm2sqlType (SEQ1_OF CHAR) = Datatype_c9
--vdm2sqlType (QUOTE_TYPE l) = ?? -- enumerated data types
--HACK
vdm2sqlType x = error (show x)

-- |
defSqlColumn :: (String, Type) -> Table_item
defSqlColumn (name, vdmType) = Table_item_c1 colName sqlType Nothing constrs
   where colName = str2colName name
         sqlType = vdm2sqlType vdmType
         constrs = vdm2sqlConstrs name vdmType

-- | 
vdm2sqlConstrs :: String -> Type -> Maybe Constraints
vdm2sqlConstrs name (BOOL) = Just $ Constraints [vdmBool2sqlConstr name]
vdm2sqlConstrs name (NAT)  = Just $ Constraints [vdmNat2sqlConstr name]
vdm2sqlConstrs name (NAT1) = Just $ Constraints [vdmNat12sqlConstr name]
vdm2sqlConstrs _    _      = Nothing


-- | Creates a constraint for a VDM boolean data type.
-- CHECK ( column_name IN (0,1))
vdmBool2sqlConstr :: String -> Constraint
vdmBool2sqlConstr s = atBoolExp2Constr (atbe s)
   where atbe s = Atomic_boolean_expression_c5 (str2sqlExp s) Nothing eosb
         eosb = Expressions_or_subquery_brackets_c1 $ Expressions_brackets $ exprs
         exprs = Expressions [(int2sqlExp 0), (int2sqlExp 1)]

-- | Creates a constraint for a VDM nat data type
-- CHECK ( column_name >= 0)
vdmNat2sqlConstr :: String -> Constraint
vdmNat2sqlConstr s = atBoolExp2Constr( atbe s)
   where atbe s = Atomic_boolean_expression_c2 (str2sqlExp s) op (int2sqlExp 0)
         op = Relational_operator_c4

-- | Creates a constraint for a VDM nat1 data type
-- CHECK ( column_name > 0)
vdmNat12sqlConstr :: String -> Constraint
vdmNat12sqlConstr s = atBoolExp2Constr( atbe s)
   where atbe s = Atomic_boolean_expression_c2 (str2sqlExp s) op (int2sqlExp 0)
         op = Relational_operator_c3


-- | Converts an integer constant value to its correspondent in SQL (Expression)
int2sqlExp :: Int -> Language.PLSql.Syntax.Expression
int2sqlExp i = Expression_c1 $ Atomic_expression_c5 $ Number (show i)

str2sqlExp :: String -> Language.PLSql.Syntax.Expression
str2sqlExp s = Expression_c1 $ Atomic_expression_c4 $ (Text_c1 s)

atBoolExp2Constr :: Atomic_boolean_expression -> Constraint
atBoolExp2Constr abe = Constraint_c2 constr1 Nothing
   where constr1 = (Constraint_item_1_c3 $ Boolean_expression_c1 abe)


-- |
sqlTable :: String -> [Table_item] -> Create_table_statement
sqlTable s ti = Create_table_statement table_name table_items Nothing Nothing tablePLSqlOpts
   where table_name = str2qualName s
         table_items = Just (Table_items_opt_brackets_c1 (Table_items ti))
         tablePLSqlOpts = Create_table_clause_2 Nothing Nothing Nothing

-- | Converts a String to a PLSql Qualified_name.
-- A Qualified Name is a PLSql data structure that represents a table name
str2qualName :: String -> Qualified_name
str2qualName = Qualified_name_c1 . str2lexQualName

-- | Converts a String to a PLSql Column_name.
str2colName :: String -> Column_name
str2colName = Column_name . str2lexQualName

str2lexQualName :: String -> Lex_qualified_name
str2lexQualName = Lex_name . Lex_identifier


-- | Defines table primary keys from a list with the columns names
defPrimaryKeys :: [String] -> Table_item
defPrimaryKeys lst = Table_item_c2 $ Constraint_c2 pkfields Nothing
    where pkfields = Constraint_item_1_c1 Kw_unique_primary_key_c2 (Just columns)
          columns = Columns_brackets $ Column_names (map str2colName lst)
